@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/arc.c,v 4.26 2000/07/18 18:57:55 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/arc.c,v 4.28 2000/11/09 00:07:26 william Exp $
  */
 
 #define _INCLUDE_FROM_ARC_C_
@@ -325,7 +325,7 @@ void GetArcArrowInfo(obj_ptr, ptip_vs1, ptail_vs1, vs1, pa_angle1,
    }
    if (int_dx==0 && int_dy==0) {
       if ((style & LS_RIGHT) && vs2 != NULL) {
-         if (obj_ptr->ctm != NULL) {
+         if (obj_ptr->ctm == NULL) {
             vs2[0].x = vs2[1].x = vs2[2].x = vs2[3].x = x2;
             vs2[0].y = vs2[1].y = vs2[2].y = vs2[3].y = y2;
          } else {
@@ -2270,6 +2270,12 @@ void MakePreciseArc()
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return;
    }
+   if (r < 1) {
+      sprintf(gszMsgBox, TgLoadString(STID_CANNOT_CREATE_ARC_WITH_RAD1),
+            TOOL_NAME, r);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      return;
+   }
    switch (dir) {
    case ARC_CCW: theta1 += 90;        theta2 += 90;        break;
    case ARC_CW:  theta1 = -theta1+90; theta2 = -theta2+90; break;
