@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/attr.c,v 4.33 2000/08/21 12:37:35 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/attr.c,v 4.34 2000/09/20 20:06:11 william Exp $
  */
 
 #define _INCLUDE_FROM_ATTR_C_
@@ -200,17 +200,24 @@ void UpdateAttr(TextPtr, AttrPtr)
    /* This routine updates the name and value in the AttrRec */
    /*   and its ObjRec after an attribute was edited.        */
 {
-   char value[MAXSTRING+1], name[MAXSTRING+1];
-
    if (AttrPtr->nameshown) {
+      char *s=GetTextPtrFirstStrSeg(TextPtr)->dyn_str.s;
+      int buf_sz=(strlen(s)<<1)+5;
+      char *name=(char*)malloc(buf_sz), *value=(char*)malloc(buf_sz);
+
+      if (name == NULL || value == NULL) FailAllocMessage();
+      *name = *value = '\0';
       ParseAttrStr(GetTextPtrFirstStrSeg(TextPtr)->dyn_str.s,
-            name, sizeof(name), value, sizeof(value));
+            name, buf_sz, value, buf_sz);
       DynStrSet(&AttrPtr->attr_value, value);
       DynStrSet(&AttrPtr->attr_name, name);
       strcat(name, value);
       DynStrSet(&GetTextPtrFirstStrSeg(TextPtr)->dyn_str, name);
+      free(name);
+      free(value);
    } else {
-      strcpy(value, GetTextPtrFirstStrSeg(TextPtr)->dyn_str.s);
+      char *value=GetTextPtrFirstStrSeg(TextPtr)->dyn_str.s;
+
       DynStrSet(&AttrPtr->attr_value, value);
       DynStrSet(&GetTextPtrFirstStrSeg(TextPtr)->dyn_str, value);
    }
@@ -267,9 +274,10 @@ struct AttrRec *AddAttrByNameAndValue(ObjPtr, AttrName, AttrValue)
    struct AttrRec *attr_ptr=NULL;
    StrSegInfo *pStrSeg=NULL;
    MiniLinesInfo *minilines=NULL;
-   char s[MAXSTRING+1];
+   char *buf=(char*)malloc(strlen(AttrName)+strlen(AttrValue)+1);
 
-   sprintf(s, "%s%s", AttrName, AttrValue);
+   if (buf == NULL) FailAllocMessage();
+   sprintf(buf, "%s%s", AttrName, AttrValue);
 
    text_obj_ptr = (struct ObjRec *)malloc(sizeof(struct ObjRec));
    text_ptr = (struct TextRec *)malloc(sizeof(struct TextRec));
@@ -293,7 +301,7 @@ struct AttrRec *AddAttrByNameAndValue(ObjPtr, AttrName, AttrValue)
 
    pStrSeg = GetTextPtrFirstStrSeg(text_ptr);
 
-   DynStrSet(&pStrSeg->dyn_str, s);
+   DynStrSet(&pStrSeg->dyn_str, buf);
 
    pStrSeg->font = curFont;
    pStrSeg->style = curStyle;
@@ -412,14 +420,9 @@ static
 void AddAttr(ObjPtr, TextObjPtr)
    struct ObjRec *ObjPtr, *TextObjPtr;
 {
-   struct AttrRec *attr_ptr;
-   struct TextRec *text_ptr;
-   char name[MAXSTRING+1], value[MAXSTRING+1];
+   struct AttrRec *attr_ptr=NULL;
+   struct TextRec *text_ptr=TextObjPtr->detail.t;
 
-   text_ptr = TextObjPtr->detail.t; 
-
-   ParseAttrStr(GetTextPtrFirstStrSeg(text_ptr)->dyn_str.s,
-         name, sizeof(name), value, sizeof(value));
    topAttr = ObjPtr->fattr;
    botAttr = ObjPtr->lattr;
 
@@ -684,23 +687,13 @@ void UpdateFirstTextStringWithAttrNameAndValue(ObjPtr, AttrPtr)
    struct ObjRec *ObjPtr;
    struct AttrRec *AttrPtr;
 {
-   int len1=0, len2=0;
-   char buf[MAXSTRING+1];
-   struct DynStrRec *dest_dyn_str=(&GetTextObjFirstStrSeg(ObjPtr)->dyn_str);
+   char *buf=(char*)malloc(AttrPtr->attr_name.sz+AttrPtr->attr_value.sz);
 
-   DynStrCpy(dest_dyn_str, &AttrPtr->attr_name);
-
-   len1 = AttrPtr->attr_name.sz-1;
-   len2 = AttrPtr->attr_value.sz-1;
-   if (len1+len2 >= MAXSTRING) {
-      sprintf(gszMsgBox, TgLoadString(STID_STRING_LEN_EXCEEDS_AND_TRUNC),
-            MAXSTRING);
-      Msg(gszMsgBox);
-      AttrPtr->attr_value.s[MAXSTRING-len1] = '\0';
-      AttrPtr->attr_value.sz = MAXSTRING-len1+1;
-   }
-   sprintf(buf, "%s%s", dest_dyn_str->s, AttrPtr->attr_value.s);
-   DynStrSet(dest_dyn_str, buf);
+   /* one extra char allocated, no big deal */
+   if (buf == NULL) FailAllocMessage();
+   sprintf(buf, "%s%s", AttrPtr->attr_name.s, AttrPtr->attr_value.s);
+   DynStrSet(&GetTextObjFirstStrSeg(ObjPtr)->dyn_str, buf);
+   free(buf);
 }
 
 static
@@ -1759,7 +1752,6 @@ void CopyAndUpdateAttrs(ToObjPtr, FromObjPtr)
    struct ObjRec *ToObjPtr, *FromObjPtr;
 {
    struct AttrRec *to_attr_ptr, *from_attr_ptr;
-   char msg[MAXSTRING];
    int confirm_status=MB_ID_CANCEL;
 
    topAttr = botAttr = NULL;
@@ -1782,6 +1774,8 @@ void CopyAndUpdateAttrs(ToObjPtr, FromObjPtr)
          confirm_status = MB_ID_NO;
 
          if (from_attr_ptr->inherited) {
+            char msg[MAXSTRING];
+
             /* obsolete attribute? */
             confirm_status = (dropObsIconAttrWhenUpdate) ? MB_ID_YES :
                   MB_ID_CANCEL;
@@ -1828,7 +1822,7 @@ void CopyAndUpdateAttrs(ToObjPtr, FromObjPtr)
 void ToggleNamedAttrShown(attr_name)
    char *attr_name;
 {
-   char *paren_ptr, msg[MAXSTRING+1];
+   char *paren_ptr;
    struct AttrRec *attr_ptr;
    int ltx, lty, rbx, rby;
 
@@ -1843,6 +1837,8 @@ void ToggleNamedAttrShown(attr_name)
    }
    *paren_ptr = '\0';
    if ((attr_ptr=FindAttrWithName(topSel->obj, attr_name, NULL)) == NULL) {
+      char msg[MAXSTRING+1];
+
       sprintf(msg, TgLoadString(STID_CANNOT_FIND_NAMED_ATTR), attr_name);
       MsgBox(msg, TOOL_NAME, INFO_MB);
       return;
