@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/auxtext.c,v 4.30 2000/05/04 13:53:07 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/auxtext.c,v 4.32 2000/08/21 12:37:35 william Exp $
  */
 
 #define _INCLUDE_FROM_AUXTEXT_C_
@@ -1573,6 +1573,10 @@ void SaveStrSeg(FP, pStrSeg)
    if (fprintf(FP, "str_seg('%s',", colorMenuItems[pStrSeg->color]) == EOF) {
       writeFileFailed = TRUE;
    }
+   /*
+    * When the new file format (version 38) is implemented, make sure to
+    * save the DontReencode related information (dontreencode) in the file!
+    */
    if (fprintf(FP,
          "'%s',%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
          &font_str[1], pStrSeg->style, pStrSeg->sz_unit,
@@ -1861,6 +1865,7 @@ StrSegInfo *CreateStrSegFromString(buf, pOwnerStrBlock)
    pStrSeg->double_byte_mod_bytes = canvasFontDoubleByteModBytes;
    pStrSeg->double_byte_vertical = canvasFontDoubleByteVertical;
    pStrSeg->direction = canvasFontDirection;
+   pStrSeg->dontreencode = canvasFontDontReencode;
    pStrSeg->underline_on = curUnderlineOn;
 
    pStrSeg->asc = canvasFontAsc;
