@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/box.c,v 4.12 2000/05/01 20:18:31 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/box.c,v 4.13 2000/07/18 18:57:43 william Exp $
  */
 
 #define _INCLUDE_FROM_BOX_C_
@@ -382,8 +382,6 @@ int CreateBoxObj(X1, Y1, X2, Y2, CreateAbsolute)
    return TRUE;
 }
 
-static XComposeStatus c_stat;
- 
 static
 void ContinueBox(OrigX, OrigY)
    int  OrigX, OrigY;
@@ -472,13 +470,7 @@ void ContinueBox(OrigX, OrigY)
          DrawIntervalRulers(OrigX, OrigY, grid_x, grid_y);
          while (XCheckMaskEvent(mainDisplay, PointerMotionMask, &ev)) ;
       } else if (input.type == KeyPress) {
-         KeySym key_sym;
-         char s[80];
-         int has_ch;
-
-         has_ch = XLookupString(&(input.xkey), s, sizeof(s), &key_sym, &c_stat);
-         TranslateKeys(s, &key_sym);
-         if (CharIsESC(&(input.xkey), s, key_sym, &has_ch)) {
+         if (KeyPressEventIsEscape(&input.xkey)) {
             XUngrabPointer(mainDisplay, CurrentTime);
             MyBox(drawWindow, drawGC, OrigX, OrigY, saved_x, saved_y);
             EndIntervalRulers(grid_x, grid_y);
