@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/choice.c,v 4.39 2000/01/19 04:49:58 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/choice.c,v 4.41 2000/07/24 17:15:36 william Exp $
  */
 
 #define _INCLUDE_FROM_CHOICE_C_
@@ -990,7 +990,7 @@ void SetCurChoice(NewChoice)
    case NOTHING:
       if (topSel != NULL) {
          if (NewChoice == VERTEXMODE) {
-            UnSelNonVertexObjs(TRUE); /* with highlight */
+            UnSelNonVertexObjs(TRUE, TRUE); /* with highlight */
             UpdSelBBox();
          } else if (NewChoice != ROTATEMODE) {
             if (NewChoice == DRAWTEXT && topSel != NULL && topSel == botSel &&
@@ -1068,7 +1068,7 @@ void SetCurChoice(NewChoice)
    case ROTATEMODE:
       if (topSel != NULL) {
          if (NewChoice == VERTEXMODE) {
-            UnSelNonVertexObjs(TRUE); /* with highlight */
+            UnSelNonVertexObjs(TRUE, TRUE); /* with highlight */
             UpdSelBBox();
          } else if (NewChoice != NOTHING) {
             if (NewChoice == DRAWTEXT && topSel != NULL && topSel == botSel &&
@@ -1974,8 +1974,11 @@ void RedrawModeWindow()
    XClearWindow(mainDisplay, modeWindow);
 
    for (i=0; i < MAXCHOICES; i++) {
+      Pixmap bitmap=((i==DRAWPOLY && connectingPortsByWire) ? wireBitmap :
+            choicePixmap[i]);
+
       if (threeDLook) {
-         values.stipple = choicePixmap[i];
+         values.stipple = bitmap;
          values.ts_x_origin = x+windowPadding;
          values.ts_y_origin = y+windowPadding;
          XChangeGC(mainDisplay, rasterGC,
@@ -1996,11 +1999,11 @@ void RedrawModeWindow()
             XSetForeground(mainDisplay, textMenuGC, myFgPixel);
             XFillRectangle(mainDisplay, modeWindow, textMenuGC,
                   x+windowPadding, y+windowPadding, choiceImageW, choiceImageH);
-            XSetStipple(mainDisplay, rvPixmapMenuGC, choicePixmap[i]);
+            XSetStipple(mainDisplay, rvPixmapMenuGC, bitmap);
             XFillRectangle(mainDisplay, modeWindow, rvPixmapMenuGC,
                   x+windowPadding, y+windowPadding, choiceImageW, choiceImageH);
          } else {
-            XSetStipple(mainDisplay, rasterGC, choicePixmap[i]);
+            XSetStipple(mainDisplay, rasterGC, bitmap);
             XFillRectangle(mainDisplay, modeWindow, rasterGC, x+windowPadding,
                   y+windowPadding, choiceImageW, choiceImageH);
          }
