@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/cmd.c,v 4.38 2000/02/26 22:55:28 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/cmd.c,v 4.39 2000/03/08 02:22:31 william Exp $
  */
 
 #define _INCLUDE_FROM_CMD_C_
@@ -499,6 +499,13 @@ void UndoDeleteCmd(CmdPtr)
          count++;
          sel_ptr = sel_ptr->next;
          if (count == CmdPtr->count_before) break;
+
+         if (CmdPtr->pos_before_has_ids) {
+            pos_to_match = GetPositionFromFullID(
+                  (char*)(CmdPtr->pos_before[count]));
+         } else {
+            pos_to_match = CmdPtr->pos_before[count];
+         }
       } else {
          next_obj = obj_ptr->next;
       }
