@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/cmdids.h,v 4.46 2000/05/20 22:37:57 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/cmdids.h,v 4.55 2000/08/04 13:24:09 william Exp $
  */
 
 #ifndef _CMDIDS_H_
@@ -383,7 +383,12 @@
 #define CMDID_TOGGLESHOWMEASUREMENTINTT    430
 #define CMDID_IMPORTJPEGFILE               431
 #define CMDID_TOGGLEAUTOEPSPREVIEWBITMAP   432
+#define CMDID_CREATETHUMBNAILS             433
+#define CMDID_CONNECT2PORTSBYWIRE          434
+#define CMDID_RENAMESIGNAMEFORAPORT        435
+#define CMDID_CLEARSIGNAMEFORAPORT         436
+#define CMDID_TOGGLESHOWWIRESIGNALNAME     437
 
-#define MAXCMDIDS 433
+#define MAXCMDIDS 438
 
 #endif /*_CMDIDS_H_*/
