@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/cursor.c,v 4.8 2000/01/15 22:23:34 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/cursor.c,v 4.9 2000/07/18 19:18:05 william Exp $
  */
 
 #define _INCLUDE_FROM_CURSOR_C_
@@ -336,6 +336,12 @@ void SetDripCursor(window)
    XDefineCursor(mainDisplay, window, dripCursor);
 }
 
+void SetHandCursor(window)
+   Window window;
+{
+   XDefineCursor(mainDisplay, window, handCursor);
+}
+
 void SetDefaultCursor(window)
    Window window;
 {
