@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/dialog.c,v 4.19 2000/04/17 20:56:27 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/dialog.c,v 4.20 2000/07/18 18:43:36 william Exp $
  */
 
 #define _INCLUDE_FROM_DIALOG_C_
@@ -109,13 +109,7 @@ unsigned int DrawWindowLoop(OrigX, OrigY, cursor, snap_to_grid)
          *OrigY = input.xbutton.y;
          return input.xbutton.button;
       } else if (input.type == KeyPress) {
-         XKeyEvent *key_ev=(&(input.xkey));
-         KeySym key_sym;
-         char s[80];
-         int has_ch=XLookupString(key_ev, s, sizeof(s), &key_sym, &c_stat);
-
-         TranslateKeys(s, &key_sym);
-         if (CharIsESC(key_ev, s, key_sym, &has_ch)) {
+         if (KeyPressEventIsEscape(&input.xkey)) {
             XUngrabPointer(mainDisplay, CurrentTime);
             XSync(mainDisplay, False);
             return (unsigned int)(-1);
@@ -154,13 +148,7 @@ unsigned int PickAPoint(OrigX, OrigY, cursor)
          *OrigY = input.xbutton.y;
          return input.xbutton.button;
       } else if (input.type == KeyPress) {
-         XKeyEvent *key_ev=(&(input.xkey));
-         KeySym key_sym;
-         char s[80];
-         int has_ch=XLookupString(key_ev, s, sizeof(s), &key_sym, &c_stat);
-
-         TranslateKeys(s, &key_sym);
-         if (CharIsESC(key_ev, s, key_sym, &has_ch)) {
+         if (KeyPressEventIsEscape(&input.xkey)) {
             XUngrabPointer(mainDisplay, CurrentTime);
             XSync(mainDisplay, False);
             return (unsigned int)(-1);
