@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/eps.c,v 4.28 2000/08/07 19:59:55 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/eps.c,v 4.29 2000/10/02 20:50:51 william Exp $
  */
 
 #define _INCLUDE_FROM_EPS_C_
@@ -501,7 +501,7 @@ int ConvertTiffToXbm(fp, tiff_offset, tiff_sz, xbm_fname, xbm_sz)
    tiff_fp = NULL;
 
    /* convert TIFF to XBM */
-   MkTempFile(xbm_fname, sizeof(xbm_fname), tmpDir, TOOL_NAME);
+   MkTempFile(xbm_fname, xbm_sz, tmpDir, TOOL_NAME);
    if ((xbm_fp=fopen(xbm_fname, "w")) == NULL) {
       return ErrorInConvertTiffToXbm(fp, NULL, NULL, NULL);
    }
