@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/grid.c,v 4.38 2000/05/10 17:34:00 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/grid.c,v 4.39 2000/07/22 20:01:22 william Exp $
  */
 
 #define _INCLUDE_FROM_GRID_C_
@@ -546,6 +546,9 @@ void WhereToPrintMsg()
          break;
       case PNG_FILE: Msg(TgLoadString(STID_WILL_EXPORT_PNG_FILE)); break;
       case JPEG_FILE: Msg(TgLoadString(STID_WILL_EXPORT_JPEG_FILE)); break;
+      case NETLIST_FILE:
+         Msg(TgLoadString(STID_WILL_EXPORT_NETLIST_FILE));
+         break;
       }
    }
 }
