@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,30 +28,28 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/group.c,v 3.15 1998/11/05 22:56:14 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/group.c,v 4.12 2000/01/15 22:23:45 william Exp $
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <X11/Xlib.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_GROUP_C_
+
+#include "tgifdefs.h"
 
 #include "attr.e"
 #include "choice.e"
 #include "cmd.e"
+#include "dialog.e"
 #include "drawing.e"
 #include "dup.e"
 #include "file.e"
-#ifndef _NO_EXTERN
 #include "group.e"
-#endif
 #include "mark.e"
 #include "msg.e"
 #include "obj.e"
 #include "page.e"
 #include "select.e"
 #include "setup.e"
+#include "strtbl.e"
 
 int gnDeleteAttrsWhileUngrouping=FALSE;
 
@@ -69,6 +67,8 @@ void CreateGroupObj(TopObjPtr, BotObjPtr)
    group_ptr->rotate = ROTATE0;
    group_ptr->flip = NO_FLIP;
    group_ptr->deck_index = (-1);
+   group_ptr->pin_connected = 0;
+   group_ptr->first_conn = group_ptr->last_conn = NULL;
    obj_ptr = (struct ObjRec *)malloc(sizeof(struct ObjRec));
    if (obj_ptr == NULL) FailAllocMessage();
    memset(obj_ptr, 0, sizeof(struct ObjRec));
@@ -105,6 +105,7 @@ void SaveGroupObj(FP, ObjPtr, Level)
          ObjPtr->invisible) == EOF) {
       writeFileFailed = TRUE;
    }
+   if (serializingFile) SaveCreatorID(FP, ObjPtr, "\t");
    SaveAttrs(FP, ObjPtr->lattr);
    if (fprintf(FP, ")") == EOF) writeFileFailed = TRUE;
 }
@@ -121,6 +122,7 @@ void SaveCompObj(FP, ObjPtr, Level)
          ObjPtr->invisible) == EOF) {
       writeFileFailed = TRUE;
    }
+   if (serializingFile) SaveCreatorID(FP, ObjPtr, "\t");
    SaveAttrs(FP, ObjPtr->lattr);
    if (fprintf(FP, ")") == EOF) writeFileFailed = TRUE;
 }
@@ -138,10 +140,41 @@ void SaveIconObj(FP, ObjPtr, Level)
          ObjPtr->detail.r->flip, ObjPtr->locked, ObjPtr->invisible) == EOF) {
       writeFileFailed = TRUE;
    }
+   if (serializingFile) SaveCreatorID(FP, ObjPtr, "\t");
    SaveAttrs(FP, ObjPtr->lattr);
    if (fprintf(FP, ")") == EOF) writeFileFailed = TRUE;
 }
 
+void SavePinObj(FP, ObjPtr, Level)
+   FILE *FP;
+   struct ObjRec *ObjPtr;
+   int Level;
+{
+   if (fprintf(FP, "pin([\n") == EOF) writeFileFailed = TRUE;
+   Save(FP, ObjPtr->detail.r->last, Level+1, INVALID);
+   if (fprintf(FP, "],\n") == EOF) writeFileFailed = TRUE;
+/*
+ * Need to check for pins.
+ * Need to handle the case where connection objects need to be dupped!
+ */
+   if (fprintf(FP, "\"%s\",%1d,%1d,%1d,%1d,%1d,%1d,",
+         ObjPtr->detail.r->s, ObjPtr->id, ObjPtr->detail.r->rotate,
+         ObjPtr->detail.r->flip, ObjPtr->locked, ObjPtr->invisible,
+         ObjPtr->detail.r->pin_connected) == EOF) {
+      writeFileFailed = TRUE;
+   }
+   if (serializingFile) SaveCreatorID(FP, ObjPtr, "\t");
+   SaveAttrs(FP, ObjPtr->lattr);
+   if (fprintf(FP, ")") == EOF) writeFileFailed = TRUE;
+}
+
+static int gnPinWarning=FALSE;
+
+#define GETGROUPVALUE(val,name) ScanValue("%d", &(val), name, "group")
+#define GETSYMVALUE(val,name) ScanValue("%d", &(val), name, "sym")
+#define GETICONVALUE(val,name) ScanValue("%d", &(val), name, "icon")
+#define GETPINVALUE(val,name) ScanValue("%d", &(val), name, "pin")
+
 void ReadGroupObj(FP, ObjType, ObjPtr)
    FILE *FP;
    int ObjType;
@@ -151,7 +184,7 @@ void ReadGroupObj(FP, ObjType, ObjPtr)
    struct ObjRec *top_obj=NULL, *bot_obj=NULL, *obj_ptr;
    int ltx, lty, rbx, rby, id=0, locked=FALSE;
    int obj_ltx, obj_lty, obj_rbx, obj_rby, rotate=0, flip=0;
-   int invisible=FALSE;
+   int invisible=FALSE, pin_connected=0;
    char line[MAXSTRING+1], *s, *s1, tmp_str[MAXSTRING+1];
 
    *ObjPtr = NULL;
@@ -178,16 +211,16 @@ void ReadGroupObj(FP, ObjType, ObjPtr)
       case OBJ_GROUP:
          InitScan(line, "\t\n, []");
          if (fileVersion <= 25) {
-            if (ScanValue("%d", &id, "id", "group") == INVALID) return;
+            if (GETGROUPVALUE(id, "id") == INVALID) return;
          } else if (fileVersion <= 32) {
-            if (ScanValue("%d", &id, "id", "group") == INVALID ||
-                ScanValue("%d", &locked, "locked", "group") == INVALID) {
+            if (GETGROUPVALUE(id, "id") == INVALID ||
+                GETGROUPVALUE(locked, "locked") == INVALID) {
                return;
             }
          } else {
-            if (ScanValue("%d", &id, "id", "group") == INVALID ||
-                ScanValue("%d", &locked, "locked", "group") == INVALID ||
-                ScanValue("%d", &invisible, "invisible", "group") == INVALID) {
+            if (GETGROUPVALUE(id, "id") == INVALID ||
+                GETGROUPVALUE(locked, "locked") == INVALID ||
+                GETGROUPVALUE(invisible, "invisible") == INVALID) {
                return;
             }
          }
@@ -196,16 +229,16 @@ void ReadGroupObj(FP, ObjType, ObjPtr)
       case OBJ_SYM:
          InitScan(line, "\t\n, []");
          if (fileVersion <= 25) {
-            if (ScanValue("%d", &id, "id", "sym") == INVALID) return;
+            if (GETSYMVALUE(id, "id") == INVALID) return;
          } else if (fileVersion <= 32) {
-            if (ScanValue("%d", &id, "id", "sym") == INVALID ||
-                ScanValue("%d", &locked, "locked", "sym") == INVALID) {
+            if (GETSYMVALUE(id, "id") == INVALID ||
+                GETSYMVALUE(locked, "locked") == INVALID) {
                return;
             }
          } else {
-            if (ScanValue("%d", &id, "id", "sym") == INVALID ||
-                ScanValue("%d", &locked, "locked", "sym") == INVALID ||
-                ScanValue("%d", &invisible, "invisible", "group") == INVALID) {
+            if (GETSYMVALUE(id, "id") == INVALID ||
+                GETSYMVALUE(locked, "locked") == INVALID ||
+                GETSYMVALUE(invisible, "invisible") == INVALID) {
                return;
             }
          }
@@ -219,38 +252,71 @@ void ReadGroupObj(FP, ObjType, ObjPtr)
          if (fileVersion <= 12) {
             s1 = FindChar((int)',', s);
             InitScan(s1, "\t\n, ");
-            if (ScanValue("%d", &id, "id", "icon") == INVALID) return;
+            if (GETICONVALUE(id, "id") == INVALID) return;
          } else if (fileVersion <= 25) {
             s1 = FindChar((int)',', s);
             InitScan(s1, "\t\n, ");
-            if (ScanValue("%d", &id, "id", "icon") == INVALID ||
-                ScanValue("%d", &rotate, "rotation", "icon") == INVALID ||
-                ScanValue("%d", &flip, "flip", "icon") == INVALID) {
+            if (GETICONVALUE(id, "id") == INVALID ||
+                GETICONVALUE(rotate, "rotation") == INVALID ||
+                GETICONVALUE(flip, "flip") == INVALID) {
                return;
             }
          } else if (fileVersion <= 32) {
             s1 = FindChar((int)',', s);
             InitScan(s1, "\t\n, ");
-            if (ScanValue("%d", &id, "id", "icon") == INVALID ||
-                ScanValue("%d", &rotate, "rotation", "icon") == INVALID ||
-                ScanValue("%d", &flip, "flip", "icon") == INVALID ||
-                ScanValue("%d", &locked, "locked", "icon") == INVALID) {
+            if (GETICONVALUE(id, "id") == INVALID ||
+                GETICONVALUE(rotate, "rotation") == INVALID ||
+                GETICONVALUE(flip, "flip") == INVALID ||
+                GETICONVALUE(locked, "locked") == INVALID) {
                return;
             }
          } else {
             s1 = FindChar((int)',', s);
             InitScan(s1, "\t\n, ");
-            if (ScanValue("%d", &id, "id", "icon") == INVALID ||
-                ScanValue("%d", &rotate, "rotation", "icon") == INVALID ||
-                ScanValue("%d", &flip, "flip", "icon") == INVALID ||
-                ScanValue("%d", &locked, "locked", "icon") == INVALID ||
-                ScanValue("%d", &invisible, "invisible", "icon") == INVALID) {
+            if (GETICONVALUE(id, "id") == INVALID ||
+                GETICONVALUE(rotate, "rotation") == INVALID ||
+                GETICONVALUE(flip, "flip") == INVALID ||
+                GETICONVALUE(locked, "locked") == INVALID ||
+                GETICONVALUE(invisible, "invisible") == INVALID) {
                return;
             }
          }
          if (id >= objId) objId = id+1;
          *(--s) = '\0';
          break;
+      case OBJ_PIN:
+         strcpy(tmp_str, FindChar((int)'"', line));
+         s = FindChar((int)'"', tmp_str);
+         if (fileVersion == INVALID) return;
+
+         if (fileVersion >= 34) {
+            s1 = FindChar((int)',', s);
+            InitScan(s1, "\t\n, ");
+/*
+ * Need to check for pins.
+ * Need to handle the case where connection objects need to be dupped!
+ */
+            if (GETPINVALUE(id, "id") == INVALID ||
+                GETPINVALUE(rotate, "rotation") == INVALID ||
+                GETPINVALUE(flip, "flip") == INVALID ||
+                GETPINVALUE(locked, "locked") == INVALID ||
+                GETPINVALUE(invisible, "invisible") == INVALID ||
+                GETPINVALUE(pin_connected, "pin_connected") == INVALID) {
+               return;
+            }
+         }
+         if (id >= objId) objId = id+1;
+         *(--s) = '\0';
+         if (!gnPinWarning) {
+            gnPinWarning = TRUE;
+            strcpy(gszMsgBox, TgLoadString(STID_WARN_PIN_NOT_SUPPORTED));
+            if (PRTGIF) {
+               fprintf(stderr, "%s\n", gszMsgBox);
+            } else {
+               MsgBox(gszMsgBox, TOOL_NAME, STOP_MB);
+            }
+         }
+         break;
       }
    }
 
@@ -268,8 +334,11 @@ void ReadGroupObj(FP, ObjType, ObjPtr)
    group_ptr->rotate = rotate;
    group_ptr->flip = flip;
    group_ptr->deck_index = (-1);
-   if (ObjType == OBJ_ICON) strcpy(group_ptr->s, tmp_str);
-
+   group_ptr->pin_connected = pin_connected;
+   group_ptr->first_conn = group_ptr->last_conn = NULL;
+   if (ObjType == OBJ_ICON || ObjType == OBJ_PIN) {
+      strcpy(group_ptr->s, tmp_str);
+   }
    ltx = top_obj->bbox.ltx;
    lty = top_obj->bbox.lty;
    rbx = top_obj->bbox.rbx;
@@ -308,12 +377,12 @@ void ReadGroupObj(FP, ObjType, ObjPtr)
    (*ObjPtr)->invisible = invisible;
 }
 
-void FreeGroupObj (ObjPtr)
+void FreeGroupObj(ObjPtr)
    struct ObjRec *ObjPtr;
 {
    register struct ObjRec *ptr, *next_obj;
 
-   for (ptr = ObjPtr->detail.r->first; ptr != NULL; ptr = next_obj) {
+   for (ptr=ObjPtr->detail.r->first; ptr != NULL; ptr=next_obj) {
       next_obj = ptr->next;
       FreeObj(ptr);
    }
@@ -329,49 +398,107 @@ void UngroupObj(ObjPtr, TopSelPtr, BotSelPtr)
    /*    *TopSelPtr will point to the top of the list, and     */
    /*    *BotSelPtr will point to the bottom of the list.      */
 {
-   register struct SelRec *sel_ptr;
-   register struct ObjRec *obj_ptr=ObjPtr->detail.r->last;
+   struct ObjRec *obj_ptr=ObjPtr->detail.r->last;
 
    for ( ; obj_ptr != NULL; obj_ptr=obj_ptr->prev) {
-      sel_ptr = (struct SelRec *)malloc(sizeof(struct SelRec));
-      if (sel_ptr == NULL) FailAllocMessage();
-      sel_ptr->obj = obj_ptr;
-      sel_ptr->next = (*TopSelPtr);
-      if (*TopSelPtr == NULL) {
-         *BotSelPtr = sel_ptr;
-      } else {
-         (*TopSelPtr)->prev = sel_ptr;
-      }
-      *TopSelPtr = sel_ptr;
+      AddObjIntoSel(obj_ptr, NULL, *TopSelPtr, TopSelPtr, BotSelPtr);
    }
    (*TopSelPtr)->prev = NULL;
 }
 
 static
-int NoObjToUngroup ()
+int NoObjToUngroup(pn_force_ungroup)
+   int *pn_force_ungroup;
 {
-   register struct SelRec *sel_ptr;
+   struct SelRec *sel_ptr=NULL;
+   int every_obj_is_sym_or_icon=TRUE;
 
-   for (sel_ptr = topSel; sel_ptr != NULL; sel_ptr = sel_ptr->next)
-      if (sel_ptr->obj->type == OBJ_GROUP)
-         return (FALSE);
-   return (TRUE);
+   if (pn_force_ungroup != NULL && topSel != NULL && topSel == botSel) {
+      int obj_type=topSel->obj->type;
+
+      *pn_force_ungroup = FALSE;
+      if (obj_type==OBJ_ICON || obj_type==OBJ_SYM) {
+         /*
+          * Need to check for pins.
+          */
+         if (MsgBox(TgLoadString(STID_ONE_SIMPLE_GROUP_UNGROUP_ANY), TOOL_NAME,
+               YNC_MB) == MB_ID_YES) {
+            *pn_force_ungroup = TRUE;
+            return FALSE;
+         }
+         return TRUE;
+      }
+   }
+   for (sel_ptr=topSel; sel_ptr != NULL; sel_ptr=sel_ptr->next) {
+      int obj_type=sel_ptr->obj->type;
+
+      if (obj_type == OBJ_GROUP) {
+         return FALSE;
+      } else if (obj_type==OBJ_ICON || obj_type==OBJ_SYM) {
+         /*
+          * Need to check for pins.
+          */
+      } else {
+         every_obj_is_sym_or_icon = FALSE;
+      }
+   }
+   if (every_obj_is_sym_or_icon) {
+      if (MsgBox(TgLoadString(STID_ALL_SIMPLE_GROUP_UNGROUP_ANY), TOOL_NAME,
+            YNC_MB) == MB_ID_YES) {
+         *pn_force_ungroup = TRUE;
+         return FALSE;
+      }
+   }
+   return TRUE;
 }
 
-void UngroupSelObj()
+void UngroupSelObj(highlight, record_cmd)
+   int highlight, record_cmd;
 {
-   register struct SelRec *sel_ptr, *next_sel;
-   register struct ObjRec *obj_ptr;
+   struct SelRec *sel_ptr=NULL, *next_sel=NULL;
+   struct ObjRec *obj_ptr=NULL;
    int sel_ltx, sel_lty, sel_rbx, sel_rby;
-   int changed=FALSE;
+   int changed=FALSE, force_ungroup_single_obj=FALSE;
 
-   if (topSel==NULL || NoObjToUngroup()) return;
+   if (topSel==NULL || NoObjToUngroup(&force_ungroup_single_obj)) return;
 
    sel_ltx = selLtX; sel_lty = selLtY;
    sel_rbx = selRbX; sel_rby = selRbY;
 
-   HighLightReverse();
-   StartCompositeCmd();
+   if (highlight) HighLightReverse();
+   if (record_cmd) StartCompositeCmd();
+   if (force_ungroup_single_obj) {
+      for (sel_ptr=topSel; sel_ptr != NULL; sel_ptr=sel_ptr->next) {
+         struct AttrRec *attr_ptr=NULL;
+
+         obj_ptr = sel_ptr->obj;
+
+         switch (obj_ptr->type) {
+         case OBJ_ICON:
+            PrepareToReplaceAnObj(obj_ptr);
+            obj_ptr->type = OBJ_GROUP;
+            attr_ptr = obj_ptr->fattr;
+            for ( ; attr_ptr != NULL; attr_ptr=attr_ptr->next) {
+               attr_ptr->inherited = FALSE;
+            }
+            AdjObjBBox(obj_ptr);
+            RecordReplaceAnObj(obj_ptr);
+            break;
+         case OBJ_SYM:
+            PrepareToReplaceAnObj(obj_ptr);
+            obj_ptr->type = OBJ_GROUP;
+            AdjObjBBox(obj_ptr);
+            RecordReplaceAnObj(obj_ptr);
+            break;
+         case OBJ_PIN:
+            /*
+             * Need to check for pins.
+             */
+            break;
+         default: break;
+         }
+      }
+   }
    for (sel_ptr=topSel; sel_ptr != NULL; sel_ptr=next_sel) {
       next_sel = sel_ptr->next;
       obj_ptr = sel_ptr->obj;
@@ -380,7 +507,7 @@ void UngroupSelObj()
          struct SelRec *tmp_sel_ptr, *tmp_top_sel=NULL, *tmp_bot_sel=NULL;
 
          changed = TRUE;
-         PrepareToReplaceAnObj(obj_ptr);
+         if (record_cmd) PrepareToReplaceAnObj(obj_ptr);
 
          UngroupObj(obj_ptr, &tmp_top_sel, &tmp_bot_sel);
          if (gnDeleteAttrsWhileUngrouping) {
@@ -405,8 +532,9 @@ void UngroupSelObj()
                tmp_sel_ptr=tmp_sel_ptr->next) {
             count++;
          }
-         RecordCmd(CMD_ONE_TO_MANY, NULL, tmp_top_sel, tmp_bot_sel, count);
-
+         if (record_cmd) {
+            RecordCmd(CMD_ONE_TO_MANY, NULL, tmp_top_sel, tmp_bot_sel, count);
+         }
          tmp_top_sel->prev = sel_ptr->prev;
          if (sel_ptr->prev == NULL) {
             topSel = tmp_top_sel;
@@ -420,10 +548,15 @@ void UngroupSelObj()
             sel_ptr->next->prev = tmp_bot_sel;
          }
          free(sel_ptr);
+         /*
+          * What about obj_ptr->detail.r->first_conn and
+          *       obj_ptr->detail.r->last_conn?
+          */
+         free(obj_ptr->detail.r);
          free(obj_ptr);
       }
    }
-   EndCompositeCmd();
+   if (record_cmd) EndCompositeCmd();
    if (changed) {
       UpdSelBBox();
       RedrawAreas(botObj,
@@ -433,73 +566,77 @@ void UngroupSelObj()
             selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1));
       SetFileModified(TRUE);
       justDupped = FALSE;
-      Msg("Selected objects are ungrouped.");
+      Msg(TgLoadString(STID_SEL_OBJ_ARE_UNGROUPED));
    }
-   HighLightForward();
+   if (highlight) HighLightForward();
 }
 
-void LockSelObj ()
+void LockSelObj()
 {
    register struct SelRec *sel_ptr;
    register struct ObjRec *obj_ptr;
    int changed=FALSE;
 
-   if (topSel==NULL) { Msg ("No object to lock!"); return; }
-   if (curChoice==VERTEXMODE) { Msg ("Can not lock in vertex mode!"); return; }
-
-   HighLightReverse ();
-   StartCompositeCmd ();
-   for (sel_ptr = topSel; sel_ptr != NULL; sel_ptr = sel_ptr->next)
-   {
+   if (topSel==NULL) {
+      Msg(TgLoadString(STID_NO_OBJ_TO_LOCK));
+      return;
+   }
+   if (curChoice==VERTEXMODE) {
+      Msg(TgLoadString(STID_CANNOT_LOCK_IN_VERTEX_MODE));
+      return;
+   }
+   HighLightReverse();
+   StartCompositeCmd();
+   for (sel_ptr=topSel; sel_ptr != NULL; sel_ptr=sel_ptr->next) {
       obj_ptr = sel_ptr->obj;
-      if (!obj_ptr->locked)
-      {
+      if (!obj_ptr->locked) {
          changed = TRUE;
-         PrepareToReplaceAnObj (obj_ptr);
+         PrepareToReplaceAnObj(obj_ptr);
          obj_ptr->locked = TRUE;
-         RecordReplaceAnObj (obj_ptr);
+         RecordReplaceAnObj(obj_ptr);
       }
    }
-   EndCompositeCmd ();
-   HighLightForward ();
-   if (changed)
-   {
-      UpdSelBBox ();
-      SetFileModified (TRUE);
+   EndCompositeCmd();
+   HighLightForward();
+   if (changed) {
+      UpdSelBBox();
+      SetFileModified(TRUE);
       justDupped = FALSE;
-      Msg ("Selected objects are locked.");
+      Msg(TgLoadString(STID_SEL_OBJ_ARE_LOCKED));
    }
 }
 
-void UnlockSelObj ()
+void UnlockSelObj()
 {
    register struct SelRec *sel_ptr;
    register struct ObjRec *obj_ptr;
    int changed=FALSE;
 
-   if (topSel==NULL) { Msg ("No object to unlock!"); return; }
-   if (curChoice==VERTEXMODE) { Msg("Can not unlock in vertex mode!"); return; }
-
-   HighLightReverse ();
-   StartCompositeCmd ();
-   for (sel_ptr = topSel; sel_ptr != NULL; sel_ptr = sel_ptr->next)
-   {
+   if (topSel==NULL) {
+      Msg(TgLoadString(STID_NO_OBJ_TO_UNLOCK));
+      return;
+   }
+   if (curChoice==VERTEXMODE) {
+      Msg(TgLoadString(STID_CANNOT_UNLOCK_IN_VERTEX_MODE));
+      return;
+   }
+   HighLightReverse();
+   StartCompositeCmd();
+   for (sel_ptr=topSel; sel_ptr != NULL; sel_ptr=sel_ptr->next) {
       obj_ptr = sel_ptr->obj;
-      if (obj_ptr->locked)
-      {
+      if (obj_ptr->locked) {
          changed = TRUE;
-         PrepareToReplaceAnObj (obj_ptr);
+         PrepareToReplaceAnObj(obj_ptr);
          obj_ptr->locked = FALSE;
-         RecordReplaceAnObj (obj_ptr);
+         RecordReplaceAnObj(obj_ptr);
       }
    }
-   EndCompositeCmd ();
-   HighLightForward ();
-   if (changed)
-   {
-      UpdSelBBox ();
-      SetFileModified (TRUE);
+   EndCompositeCmd();
+   HighLightForward();
+   if (changed) {
+      UpdSelBBox();
+      SetFileModified(TRUE);
       justDupped = FALSE;
-      Msg ("Selected objects are unlocked.");
+      Msg(TgLoadString(STID_SEL_OBJ_ARE_UNLOCKED));
    }
 }
