@@ -1,11 +1,11 @@
 /*
- * Code in this file is derived from the public domain code in
+ * Some code in this file is derived from the public domain code in
  *              WWW/Library/Implementation/HTTP.c distributed with lynx-2.2,
  *              whose original author is Tim Berners-lee <timbl@info.cern.ch>.
  *
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -22,7 +22,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -32,40 +32,30 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/http.c,v 3.21 1998/11/05 22:56:15 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/http.c,v 4.18 2000/01/15 22:23:46 william Exp $
  */
 
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/time.h>
-#include <sys/ioctl.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <errno.h>
-#ifdef isc /* SunSoft/Interactive UNIX */
-#include <net/errno.h>
-#endif
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
+#define _INCLUDE_FROM_HTTP_C_
 
-#include "const.h"
-#include "types.h"
-#include "patchlvl.h"
+#include "tgifdefs.h"
 
+#include "dialog.e"
+#include "drawing.e"
 #include "file.e"
-#ifndef _NO_EXTERN
 #include "http.e"
-#endif /* !_NO_EXTERN */
+#include "mainloop.e"
 #include "msg.e"
 #include "remote.e"
+#include "setup.e"
+#include "stream.e"
+#include "strtbl.e"
 #include "tcp.e"
 #include "util.e"
 #include "version.e"
 
 int debugHttp=0;
+int gnHttpKeepAlive=FALSE;
+char *gpszFakedReferer=NULL;
 
 typedef struct TgifHttpLineInfo {
    char *name;
@@ -77,7 +67,9 @@ typedef struct TgifHttpHeaderInfo {
    char *version;
    int resp_code;
    char *resp_status;
-   char *date;
+   char *last_modified;
+   char *server;
+   char *connection;
    char *location;
    char *www_authenticate;
    char *content_encoding;
@@ -104,6 +96,13 @@ static char SZ_HTTP_VERSION[]="HTTP/1.0";
 static char SZ_USER_AGENT[128];
 static char SZ_USER_NAME[128];
 
+typedef struct tagRefererInfo {
+   char *referer;
+   struct tagRefererInfo *next;
+} RefererInfo;
+
+static RefererInfo *topReferer=NULL;
+
 void HttpFreeBuf(buf)
    char *buf;
 {
@@ -132,6 +131,7 @@ void CleanUpAuthInfo()
    botAuthInfo = NULL;
 }
 
+/* do not translate -- program constants */
 static char gszEncode[] =
       "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 static int gnDecode[] = {
@@ -294,6 +294,51 @@ int SetAuthorization(pszHost, nPort, pszScheme, pszRealm, pszEncodedAuth)
    return TRUE;
 }
 
+void HttpClearReferer()
+{
+   RefererInfo *pri=topReferer;
+
+#ifdef _TGIF_DBG /* debug, do not translate */
+   TgAssert(pri != NULL, "NULL topReferer in HttpClearReferer()", NULL);
+#endif /* TGIF_DBG */
+   if (pri == NULL) return;
+   pri = pri->next;
+   UtilFree(topReferer->referer);
+   free(topReferer);
+   topReferer = pri;
+}
+
+void HttpSetReferer(buf)
+   char *buf;
+{
+   RefererInfo *pri=(RefererInfo*)malloc(sizeof(RefererInfo));
+
+   if (pri == NULL) FailAllocMessage();
+   pri->next = topReferer;
+   pri->referer = UtilStrDup(buf);
+   if (pri->referer == NULL) FailAllocMessage();
+   topReferer = pri;
+}
+
+void HttpFakeReferer(buf)
+   char *buf;
+{
+   UtilFree(gpszFakedReferer);
+   gpszFakedReferer = NULL;
+   if (buf != NULL && *buf != '\0') {
+      gpszFakedReferer = UtilStrDup(buf);
+      if (gpszFakedReferer == NULL) FailAllocMessage();
+   }
+}
+
+static
+void CleanUpHttpReferer()
+{
+   while (topReferer != NULL) {
+      HttpClearReferer();
+   }
+}
+
 static
 void CleanUpHttpHeaderInfo()
 {
@@ -303,7 +348,13 @@ void CleanUpHttpHeaderInfo()
    if (tgifHttpHeaderInfo.resp_status != NULL) {
       free(tgifHttpHeaderInfo.resp_status);
    }
-   if (tgifHttpHeaderInfo.date != NULL) free(tgifHttpHeaderInfo.date);
+   if (tgifHttpHeaderInfo.last_modified != NULL) {
+      free(tgifHttpHeaderInfo.last_modified);
+   }
+   if (tgifHttpHeaderInfo.server != NULL) free(tgifHttpHeaderInfo.server);
+   if (tgifHttpHeaderInfo.connection != NULL) {
+      free(tgifHttpHeaderInfo.connection);
+   }
    if (tgifHttpHeaderInfo.location != NULL) free(tgifHttpHeaderInfo.location);
    if (tgifHttpHeaderInfo.www_authenticate != NULL) {
       free(tgifHttpHeaderInfo.www_authenticate);
@@ -328,6 +379,12 @@ void CleanUpHttp()
    ResetAuthorization();
    CleanUpAuthInfo();
    CleanUpHttpHeaderInfo();
+   CleanUpHttpReferer();
+
+   UtilFree(gpszFakedReferer);
+   gpszFakedReferer = NULL;
+
+   if (gnDecode) { }
 }
 
 void InitHttp()
@@ -335,6 +392,7 @@ void InitHttp()
    topAuthInfo = botAuthInfo = NULL;
    memset(&curAuthorization, 0, sizeof(struct AuthInfo));
    memset(&tgifHttpHeaderInfo, 0, sizeof(struct TgifHttpHeaderInfo));
+   topReferer = NULL;
 }
 
 char *HttpHeaderGetVersion() { return tgifHttpHeaderInfo.version; }
@@ -343,7 +401,11 @@ int HttpHeaderGetResponseCode() { return tgifHttpHeaderInfo.resp_code; }
 
 char *HttpHeaderGetResponseStatus() { return tgifHttpHeaderInfo.resp_status; }
 
-char *HttpHeaderGetDate() { return tgifHttpHeaderInfo.date; }
+char *HttpHeaderGetLastModified() { return tgifHttpHeaderInfo.last_modified; }
+
+char *HttpHeaderGetServer() { return tgifHttpHeaderInfo.server; }
+
+char *HttpHeaderGetConnection() { return tgifHttpHeaderInfo.connection; }
 
 char *HttpHeaderGetLocation() { return tgifHttpHeaderInfo.location; }
 
@@ -394,21 +456,21 @@ int HttpDoConnect(psz_host, us_port, pn_socket)
    char *msg=(char*)malloc((len+1)*sizeof(char));
 
    if (msg == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
-   sprintf(msg, "Making an HTTP connection to \"%s:%1d\"...", psz_host,
-         us_port);
+   sprintf(msg, TgLoadCachedString(CSTID_MAKING_CONN_TO_HOST_PORT),
+         "HTTP", psz_host, us_port);
    ShowRemoteStatus(msg);
 
    rc = TcpDoConnect(psz_host, us_port, pn_socket);
 
    if (rc == TG_REMOTE_STATUS_OK) {
-      sprintf(msg, "HTTP: connection to \"%s:%1d\" established.", psz_host,
-            us_port);
+      sprintf(msg, TgLoadCachedString(CSTID_CONN_TO_HOST_PORT_ESTB),
+            "HTTP", psz_host, us_port);
    } else {
-      sprintf(msg, "Fail to connect to HTTP server on \"%s:%1d\".", psz_host,
-            us_port);
+      sprintf(msg, TgLoadString(STID_FAIL_TO_CONN_TO_HOST_PORT),
+            "HTTP", psz_host, us_port);
    }
    ShowRemoteStatus(msg);
    free(msg);
@@ -416,39 +478,6 @@ int HttpDoConnect(psz_host, us_port, pn_socket)
    return rc;
 }
 
-static char SZ_CONTENT_TYPE[]="Content-type:";
-static char SZ_CONTENT_LENGTH[]="Content-length:";
-static char SZ_DEF_GIF_NAME[]="/tmp/htclient.gif";
-static char SZ_POST_CONTENT_TYPE[]="application/x-www-form-urlencoded";
-
-static char *pszAccept[] = {
-   "text/plain",
-   "text/html",
-   "application/x-tgif",
-   "*/*",
-   NULL
-};
-
-static
-char *AppendAcceptStrings(buf)
-   char *buf;
-{
-   char **s_ptr;
-   int cur_len=strlen(buf);
-
-   for (s_ptr=pszAccept; *s_ptr != NULL; s_ptr++) {
-      int len=strlen(*s_ptr);
-      int new_len=cur_len+len+2+8;
-
-      if ((buf=(char*)realloc(buf, new_len+1)) == NULL) {
-         return NULL;
-      }
-      sprintf(&buf[cur_len], "Accept: %s\r\n", *s_ptr);
-      cur_len = new_len;
-   }
-   return buf;
-}
-
 static
 char *AppendSimpleString(buf, name, value)
    char *buf, *name, *value;
@@ -473,11 +502,72 @@ char *AppendSimpleString(buf, name, value)
    return buf;
 }
 
+/* do not translate -- program constants */
+static char SZ_CONTENT_TYPE[]="Content-type:";
+static char SZ_CONTENT_LENGTH[]="Content-length:";
+static char SZ_POST_CONTENT_TYPE[]="application/x-www-form-urlencoded";
+
+static char *pszAccept[] = {
+/* "image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png", */
+   "text/plain",
+   "text/html",
+   "application/x-tgif",
+   "*/*",
+   NULL
+};
+
+static
+char *AppendAcceptStrings(buf)
+   char *buf;
+{
+   char **s_ptr;
+   int cur_len=strlen(buf);
+
+   for (s_ptr=pszAccept; *s_ptr != NULL; s_ptr++) {
+      int len=strlen(*s_ptr);
+      int new_len=cur_len+len+2+8;
+
+      if ((buf=(char*)realloc(buf, new_len+1)) == NULL) {
+         return NULL;
+      }
+      /* do not translate -- program constants */
+      sprintf(&buf[cur_len], "Accept: %s\r\n", *s_ptr);
+      cur_len = new_len;
+   }
+   return buf;
+}
+
+static
+char *AppendKeepAliveStrings(buf)
+   char *buf;
+{
+   if (gnHttpKeepAlive == FALSE) return buf;
+
+   /* do not translate -- program constants */
+   return AppendSimpleString(buf, "Connection", "Keep-Alive");
+}
+
+static
+char *AppendRefererStrings(buf)
+   char *buf;
+{
+   if (topReferer == NULL) {
+      if (gpszFakedReferer == NULL) {
+         return buf;
+      }
+      /* do not translate -- program constants */
+      return AppendSimpleString(buf, "Referer", gpszFakedReferer);
+   }
+   /* do not translate -- program constants */
+   return AppendSimpleString(buf, "Referer", topReferer->referer);
+}
+
 static
 char *AppendUserAgentString(buf)
    char *buf;
 {
    InitUserAgentName();
+   /* do not translate -- program constants */
    return AppendSimpleString(buf, "User-Agent", SZ_USER_AGENT);
 }
 
@@ -486,21 +576,20 @@ char *AppendFromString(buf)
    char *buf;
 {
    InitUserAgentName();
+   /* do not translate -- program constants */
    return AppendSimpleString(buf, "From", SZ_USER_NAME);
 }
 
 static
-char *AppendHostString(buf)
-   char *buf;
+char *AppendHostString(buf, psz_host, us_port)
+   char *buf, *psz_host;
+   int us_port;
 {
-   char *c_ptr;
+   char tmp_buf[MAXSTRING];
 
-   InitUserAgentName();
-   if ((c_ptr=strchr(SZ_USER_NAME, '@')) == NULL) {
-      return AppendSimpleString(buf, "Host", SZ_USER_NAME);
-   } else {
-      return AppendSimpleString(buf, "Host", ++c_ptr);
-   }
+   sprintf(tmp_buf, "%s:%1d", psz_host, us_port);
+   /* do not translate -- program constants */
+   return AppendSimpleString(buf, "Host", tmp_buf);
 }
 
 static
@@ -515,6 +604,7 @@ char *AppendAuthorizationString(buf)
    if (pszAuth == NULL) return NULL;
    sprintf(pszAuth, "%s %s", curAuthorization.scheme,
          curAuthorization.authorization);
+   /* do not translate -- program constants */
    pszReturn = AppendSimpleString(buf, "Authorization", pszAuth);
    free(pszAuth);
    return pszReturn;
@@ -524,6 +614,7 @@ static
 char *AppendPostContentTypeString(buf)
    char *buf;
 {
+   /* do not translate -- program constants */
    return AppendSimpleString(buf, "Content-type", SZ_POST_CONTENT_TYPE);
 }
 
@@ -535,6 +626,7 @@ char *AppendPostContentLengthString(buf, content_length)
    char len_str[20];
 
    sprintf(len_str, "%1d", content_length);
+   /* do not translate -- program constants */
    return AppendSimpleString(buf, "Content-length", len_str);
 }
 
@@ -555,7 +647,9 @@ char *AppendPostContentString(buf, fp, content_length)
          fp)) > 0) {
       if (bytes_read+total_read > content_length) {
          bytes_read = content_length-total_read;
-         fprintf(stderr, "Lines too long in AppendPostContentString().\n");
+         fprintf(stderr, TgLoadString(STID_LINES_TOO_LONG_CONTENT_LENGTH),
+               "HTTP");
+         fprintf(stderr, "\n");
       }
       strncpy(&buf[cur_len+total_read], tmp_buf, bytes_read);
       total_read += bytes_read;
@@ -571,43 +665,56 @@ char *AppendCRLFString(buf)
    return AppendSimpleString(buf, NULL, NULL);
 }
 
-int HttpDoWrite(n_socket, psz_path)
-   int n_socket;
-   char *psz_path;
+int HttpDoWrite(n_socket, psz_path, psz_host, us_port)
+   int n_socket, us_port;
+   char *psz_path, *psz_host;
 {
    int status=0, total_sz=0;
    FILE *fp=NULL;
    char *buf=(char*)malloc((strlen(psz_path)+5+2+31)*sizeof(char)), msg[40];
 
    if (buf == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
    if (postingCGIQuery) {
+      /* do not translate -- program constants */
       sprintf(buf, "POST %s %s\r\n", psz_path, SZ_HTTP_VERSION);
+   } else if (gettingHttpHeaderOnly) {
+      /* do not translate -- program constants */
+      sprintf(buf, "HEAD %s %s\r\n", psz_path, SZ_HTTP_VERSION);
    } else {
+      /* do not translate -- program constants */
       sprintf(buf, "GET %s %s\r\n", psz_path, SZ_HTTP_VERSION);
    }
+   if ((buf=AppendKeepAliveStrings(buf)) == NULL) {
+      FailAllocMessage();
+      return TG_REMOTE_STATUS_MEM;
+   }
+   if ((buf=AppendRefererStrings(buf)) == NULL) {
+      FailAllocMessage();
+      return TG_REMOTE_STATUS_MEM;
+   }
    if ((buf=AppendAcceptStrings(buf)) == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
    if ((buf=AppendUserAgentString(buf)) == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
    if ((buf=AppendFromString(buf)) == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
-   if ((buf=AppendHostString(buf)) == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+   if ((buf=AppendHostString(buf, psz_host, us_port)) == NULL) {
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
    if (curAuthorization.scheme != NULL &&
          curAuthorization.authorization != NULL) {
       if ((buf=AppendAuthorizationString(buf)) == NULL) {
-         fprintf(stderr, "Memory allocation failed.\n");
+         FailAllocMessage();
          return TG_REMOTE_STATUS_MEM;
       }
    }
@@ -616,8 +723,9 @@ int HttpDoWrite(n_socket, psz_path)
       char tmp_buf[512];
 
       if ((fp=fopen(fnameForPostingCGIQuery, "r")) == NULL) {
-         fprintf(stderr, "Fail to open '%s' for read.\n",
+         fprintf(stderr, TgLoadString(STID_CANNOT_OPEN_FILE_FOR_READING),
                fnameForPostingCGIQuery);
+         fprintf(stderr, "\n");
          return TG_REMOTE_STATUS_READ;
       }
       while ((bytes_read=fread(tmp_buf, sizeof(char), sizeof(tmp_buf),
@@ -627,42 +735,45 @@ int HttpDoWrite(n_socket, psz_path)
       rewind(fp);
       if ((buf=AppendPostContentTypeString(buf)) == NULL) {
          fclose(fp);
-         fprintf(stderr, "Memory allocation failed.\n");
+         FailAllocMessage();
          return TG_REMOTE_STATUS_MEM;
       }
       if ((buf=AppendPostContentLengthString(buf, total_sz)) == NULL) {
          fclose(fp);
-         fprintf(stderr, "Memory allocation failed.\n");
+         FailAllocMessage();
          return TG_REMOTE_STATUS_MEM;
       }
    }
    if ((buf=AppendCRLFString(buf)) == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
    if (fp != NULL) {
       buf = AppendPostContentString(buf, fp, total_sz);
       fclose(fp);
       if (buf == NULL) {
-         fprintf(stderr, "Memory allocation failed.\n");
+         FailAllocMessage();
          return TG_REMOTE_STATUS_MEM;
       }
    }
-   sprintf(msg, "HTTP: sending requests...");
+   sprintf(msg, TgLoadCachedString(CSTID_SENDING_REQUESTS_DOTS), "HTTP");
    ShowRemoteStatus(msg);
 
    status = TcpDoWrite(n_socket, buf, (int)strlen(buf));
    free(buf);
 
    if (status != TG_REMOTE_STATUS_OK) {
-      sprintf(msg, "HTTP: fail to send requests.");
+      sprintf(msg, TgLoadString(STID_FAIL_TO_SEND_REQ), "HTTP");
       ShowRemoteStatus(msg);
    }
    return status;
 }
 
+#ifdef _TGIF_DBG /* debug, do not translate */
+static char SZ_DEF_GIF_NAME[]="/tmp/htclient.gif";
+
 static
-void HttpDumpResponse(buf)
+void DebugHttpDumpResponse(buf)
    char *buf;
 {
    char *c_ptr=strchr(buf, '\n'), *line_ptr=buf, *content_type=NULL;
@@ -703,7 +814,8 @@ void HttpDumpResponse(buf)
       }
       line_ptr = &c_ptr[1];
       if (content_type != NULL && content_length != (-1)) {
-         if (strcmp(content_type, "image/gif") == 0 &&
+         /* do not translate -- program constants */
+         if (UtilStrICmp(content_type, "image/gif") == 0 &&
                UtilStrNCaseCmp(line_ptr, "GIF87a", 6) == 0) {
             gif87a = TRUE;
             break;
@@ -715,13 +827,15 @@ void HttpDumpResponse(buf)
       FILE *gif_fp;
 
       if ((gif_fp=fopen(SZ_DEF_GIF_NAME, "w")) == NULL) {
-         fprintf(stderr, "Fail to open '%s' for write.\n", SZ_DEF_GIF_NAME);
+         fprintf(stderr, TgLoadString(STID_CANNOT_OPEN_FILE_FOR_WRITING),
+               SZ_DEF_GIF_NAME);
+         fprintf(stderr, "\n");
       } else {
          if (fwrite(line_ptr, sizeof(char), content_length, gif_fp) !=
                content_length) {
-            fprintf(stderr, "Write to '%s' failed.\n", SZ_DEF_GIF_NAME);
-            fprintf(stderr, "Fail system may be full.\n");
+            FailToWriteFileMessage(SZ_DEF_GIF_NAME);
          } else {
+            /* debug, do not translate */
             fprintf(stderr, "GIF87a image written to '%s'.\n", SZ_DEF_GIF_NAME);
          }
          fclose(gif_fp);
@@ -738,12 +852,14 @@ void HttpDumpResponse(buf)
       }
    }
 }
+#endif /* _TGIF_DBG */
 
 static
 int ParseBadFormat(buf)
    char *buf;
 {
-   fprintf(stderr, "Invalid format in the HTTP header.\n");
+   fprintf(stderr, TgLoadString(STID_INVALID_FORMAT_IN_HEADER), "HTTP");
+   fprintf(stderr, "\n");
    if (buf != NULL) free(buf);
    return TG_REMOTE_STATUS_FORMAT;
 }
@@ -752,7 +868,7 @@ static
 int ParseNoMem(buf)
    char *buf;
 {
-   fprintf(stderr, "Memory allocation failed while parsing the HTTP header.\n");
+   FailAllocMessage();
    if (buf != NULL) free(buf);
    return TG_REMOTE_STATUS_MEM;
 }
@@ -794,10 +910,19 @@ int HttpParseHeaderLine(buf, first_line)
    if (colon_ptr == NULL) return ParseBadFormat(dup_buf);
    *colon_ptr = '\0';
    UtilTrimBlanks(dup_buf);
-   if (UtilStrICmp(dup_buf, "Date") == 0) {
-      tgifHttpHeaderInfo.date = UtilStrDup(&colon_ptr[1]);
-      if (tgifHttpHeaderInfo.date == NULL) return ParseNoMem(dup_buf);;
-      UtilTrimBlanks(tgifHttpHeaderInfo.date);
+   /* do not translate -- program constants */
+   if (UtilStrICmp(dup_buf, "Last-Modified") == 0) {
+      tgifHttpHeaderInfo.last_modified = UtilStrDup(&colon_ptr[1]);
+      if (tgifHttpHeaderInfo.last_modified == NULL) return ParseNoMem(dup_buf);;
+      UtilTrimBlanks(tgifHttpHeaderInfo.last_modified);
+   } else if (UtilStrICmp(dup_buf, "Server") == 0) {
+      tgifHttpHeaderInfo.server = UtilStrDup(&colon_ptr[1]);
+      if (tgifHttpHeaderInfo.server == NULL) return ParseNoMem(dup_buf);;
+      UtilTrimBlanks(tgifHttpHeaderInfo.server);
+   } else if (UtilStrICmp(dup_buf, "Connection") == 0) {
+      tgifHttpHeaderInfo.connection = UtilStrDup(&colon_ptr[1]);
+      if (tgifHttpHeaderInfo.connection == NULL) return ParseNoMem(dup_buf);;
+      UtilTrimBlanks(tgifHttpHeaderInfo.connection);
    } else if (UtilStrICmp(dup_buf, "Location") == 0) {
       tgifHttpHeaderInfo.location = UtilStrDup(&colon_ptr[1]);
       if (tgifHttpHeaderInfo.location == NULL) return ParseNoMem(dup_buf);;
@@ -855,8 +980,15 @@ void HttpDumpHeader()
       }
       fprintf(stderr, "\n");
    }
-   if (tgifHttpHeaderInfo.date != NULL) {
-      fprintf(stderr, "Date: %s\n", tgifHttpHeaderInfo.date);
+   /* do not translate -- program constants */
+   if (tgifHttpHeaderInfo.last_modified != NULL) {
+      fprintf(stderr, "Last-Modified: %s\n", tgifHttpHeaderInfo.last_modified);
+   }
+   if (tgifHttpHeaderInfo.server != NULL) {
+      fprintf(stderr, "Server: %s\n", tgifHttpHeaderInfo.server);
+   }
+   if (tgifHttpHeaderInfo.connection != NULL) {
+      fprintf(stderr, "Connection: %s\n", tgifHttpHeaderInfo.connection);
    }
    if (tgifHttpHeaderInfo.location != NULL) {
       fprintf(stderr, "Location: %s\n", tgifHttpHeaderInfo.location);
@@ -878,8 +1010,9 @@ void HttpDumpHeader()
    }
    for (pthli=tgifHttpHeaderInfo.misc; pthli != NULL; pthli=pthli->next) {
       fprintf(stderr, "%s: %s\n",
-            (pthli->name == NULL ? "(unknown)" : pthli->name),
-            (pthli->value == NULL ? "(none)" : pthli->value));
+            (pthli->name == NULL ? TgLoadCachedString(CSTID_PARANED_UNKNOWN) :
+            pthli->name), (pthli->value == NULL ?
+            TgLoadCachedString(CSTID_PARANED_NONE) : pthli->value));
    }
 }
 
@@ -909,7 +1042,7 @@ char *HttpExtractText(buf, pn_buf_sz, pn_html, ppsz_content_type)
       }
       if (debugHttp > 0) {
          if (debugHttp == 99 && cmdLineDumpURL && !cmdLineDumpURLWithHeader) {
-         } else {
+         } else if (!deserializingFile) {
             fprintf(fp, "%s\n", line_ptr);
          }
       }
@@ -932,10 +1065,14 @@ char *HttpExtractText(buf, pn_buf_sz, pn_html, ppsz_content_type)
             UtilStrNCaseCmp(line_ptr, SZ_CONTENT_TYPE, len1) == 0) {
          content_type = UtilStrDup(&line_ptr[len1]);
          if (content_type != NULL) {
+            char *psz;
+
             UtilTrimBlanks(content_type);
+            if ((psz=strchr(content_type, ';')) != NULL) *psz = '\0';
             if (ppsz_content_type != NULL) {
                *ppsz_content_type = UtilStrDup(content_type);
             }
+            /* do not translate -- program constants */
             if (UtilStrNCaseCmp(content_type, "text/", 5) == 0) {
                text_type = TRUE;
                if (strcmp(&content_type[5], "html") == 0) {
@@ -962,7 +1099,9 @@ char *HttpExtractText(buf, pn_buf_sz, pn_html, ppsz_content_type)
       c_ptr = strchr(line_ptr, '\n');
    }
    if (content_type != NULL) free(content_type);
-   if (text_type || content_length == (-1)) {
+   if (cmdLineDumpURL && cmdLineDumpURLWithHeader && cmdLineDumpURLHeaderOnly) {
+      /* only want the header */
+   } else if (text_type || content_length == (-1)) {
       int buf_len=strlen(line_ptr);
       char *return_buf;
 
@@ -973,7 +1112,7 @@ char *HttpExtractText(buf, pn_buf_sz, pn_html, ppsz_content_type)
          return_buf = (char*)malloc((content_length+1)*sizeof(char));
       }
       if (return_buf == NULL) {
-         fprintf(stderr, "Memory allocation failed.\n");
+         FailAllocMessage();
          return NULL;
       }
       if (buf_len <= content_length) {
@@ -990,18 +1129,18 @@ char *HttpExtractText(buf, pn_buf_sz, pn_html, ppsz_content_type)
          memcpy(return_buf, line_ptr, content_length);
       }
       return_buf[content_length] = '\0';
-      if (pn_buf_sz != NULL) *pn_buf_sz = (content_length+1);
+      if (pn_buf_sz != NULL) *pn_buf_sz = content_length;
       return return_buf;
    } else if (content_length != (-1)) {
       char *return_buf=(char*)malloc((content_length+1)*sizeof(char));
 
       if (return_buf == NULL) {
-         fprintf(stderr, "Memory allocation failed.\n");
+         FailAllocMessage();
          return NULL;
       }
       memcpy(return_buf, line_ptr, content_length);
       return_buf[content_length] = '\0';
-      if (pn_buf_sz != NULL) *pn_buf_sz = (content_length+1);
+      if (pn_buf_sz != NULL) *pn_buf_sz = content_length;
       return return_buf;
    }
    return NULL;
@@ -1013,68 +1152,96 @@ int GetContentLength(buf, pn_header_len)
    int *pn_header_len;
 {
    char *c_ptr, *line_ptr=buf;
-   int content_len=(-1);
+   int content_len=(-1), just_lf=IsJustLF(buf);
+   int inc=(just_lf?1:2);
 
-   c_ptr = strstr(line_ptr, "\r\n");
-   if (c_ptr == NULL) {
-      for (c_ptr=strchr(line_ptr, '\n'); c_ptr != NULL;
-            c_ptr=strchr(line_ptr, '\n')) {
-         char *colon_ptr;
+   for (c_ptr=GetHeaderLine(line_ptr, just_lf); c_ptr != NULL;
+         c_ptr=GetHeaderLine(line_ptr, just_lf)) {
+      char *colon_ptr=NULL;
 
-         if (c_ptr == line_ptr) {
-            *pn_header_len = ((&c_ptr[1]) - buf);
-            return (content_len==(-1) ? 0 : content_len);
-         }
-         *c_ptr = '\0';
-         colon_ptr = strchr(line_ptr, ':');
-         if (colon_ptr != NULL) {
-            *colon_ptr = '\0';
-            if (UtilStrICmp(line_ptr, "Content-Length") == 0) {
-               char length_str[MAXSTRING];
-               int len;
-
-               UtilStrCpy(length_str, MAXSTRING-1, &colon_ptr[1]);
-               UtilTrimBlanks(length_str);
-               if (sscanf(length_str, "%d", &len) == 1) {
-                  content_len = len;
-               }
-            }
-            *colon_ptr = ':';
-         }
-         *c_ptr = '\n';
-         line_ptr = (&c_ptr[1]);
+      if (c_ptr == line_ptr) {
+         /* reach the end of header, now decide if the header is good */
+         *pn_header_len = ((&c_ptr[inc]) - buf);
+         return (content_len==(-1) ? 0 : content_len);
       }
-   } else {
-      for ( ; c_ptr != NULL; c_ptr=strstr(line_ptr, "\r\n")) {
-         char *colon_ptr;
+      *c_ptr = '\0';
+      colon_ptr = strchr(line_ptr, ':');
+      if (colon_ptr != NULL) {
+         *colon_ptr = '\0';
+         /* do not translate -- program constants */
+         if (UtilStrICmp(line_ptr, "Content-Length") == 0) {
+            char length_str[MAXSTRING];
+            int len=0;
 
-         if (c_ptr == line_ptr) {
-            *pn_header_len = ((&c_ptr[2]) - buf);
-            return (content_len==(-1) ? 0 : content_len);
-         }
-         *c_ptr = '\0';
-         colon_ptr = strchr(line_ptr, ':');
-         if (colon_ptr != NULL) {
-            *colon_ptr = '\0';
-            if (UtilStrICmp(line_ptr, "Content-Length") == 0) {
-               char length_str[MAXSTRING];
-               int len;
-
-               UtilStrCpy(length_str, MAXSTRING-1, &colon_ptr[1]);
-               UtilTrimBlanks(length_str);
-               if (sscanf(length_str, "%d", &len) == 1) {
-                  content_len = len;
-               }
+            UtilStrCpyN(length_str, MAXSTRING-1, &colon_ptr[1]);
+            UtilTrimBlanks(length_str);
+            if (sscanf(length_str, "%d", &len) == 1) {
+               content_len = len;
             }
-            *colon_ptr = ':';
          }
-         *c_ptr = '\r';
-         line_ptr = (&c_ptr[2]);
+         *colon_ptr = ':';
       }
+      *c_ptr = (just_lf ? '\n' : '\r');
+      line_ptr = (&c_ptr[inc]);
    }
    return (-1);
 }
 
+static
+int ReadFromSocket(n_socket)
+   int n_socket;
+   /* return TRUE means there's data at n_socket */
+   /* return FALSE means abort */
+{
+   for (;;) {
+      struct timeval timeout;
+      fd_set fdset;
+      int select_width=XConnectionNumber(mainDisplay)+1, status=0, nfds=0;
+
+      nfds = max(select_width, n_socket+1);
+
+      timeout.tv_sec = 0;
+      timeout.tv_usec = 100000;
+
+      FD_ZERO(&fdset);
+      FD_SET(select_width-1, &fdset);
+      FD_SET(n_socket, &fdset);
+#ifdef __hpux
+      status = select(nfds, (int*)&fdset, NULL, NULL, &timeout);
+#else /* !__hpux */
+      status = select(nfds, &fdset, NULL, NULL, &timeout);
+#endif /* __hpux */
+      if (status < 0) {
+         sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
+               "ReadFromSocket()"),
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         break;
+      } else if (status == 0) {
+         /* spin the interrupt thing */
+         CheckInterrupt(TRUE);
+      } else if (status == 1) {
+         if (FD_ISSET(n_socket, &fdset)) {
+            /* character waiting to be processed at n_socket */
+            return TRUE;
+         } else if (FD_ISSET(select_width-1, &fdset)) {
+            while (XPending(mainDisplay) > 0) {
+               if (TryProcessAnAbortXEvent(TRUE, NULL, NULL) != 0) {
+                  return FALSE;
+               }
+            }
+         }
+      } else if (status == 2) {
+         return TRUE;
+      } else {
+         sprintf(gszMsgBox, TgLoadString(STID_FUNC_INVALID_RC_FOR_SELECT),
+               "WaitForEvent()", status);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         break;
+      }
+   }
+   return FALSE;
+}
+
 #define MIN_READ_SIZE 0x100
 
 int HttpDoRead(n_socket, ppsz_buf, pn_buf_sz)
@@ -1088,72 +1255,107 @@ int HttpDoRead(n_socket, ppsz_buf, pn_buf_sz)
    if (pn_buf_sz != NULL) *pn_buf_sz = 0;
    *ppsz_buf = NULL;
    if (buf == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
-   sprintf(msg, "HTTP: reading responses...");
+   sprintf(msg, TgLoadCachedString(CSTID_READING_RESPONSE_DOTS), "HTTP");
    ShowRemoteStatus(msg);
+   CleanUpDownloadStats();
+   ResetMultipartReplace(FALSE);
 
    do {
-      int bytes_read;
+      int bytes_read=0;
+      char progress_buf[MAXSTRING];
 
+      *progress_buf = '\0';
       if (buf_sz - len < MIN_READ_SIZE) {
          buf_sz += 0x400;
          if ((buf=(char*)realloc(buf, buf_sz)) == NULL) {
-            fprintf(stderr, "Memory allocation failed.\n");
+            FailAllocMessage();
             status = TG_REMOTE_STATUS_MEM;
             break;
          }
       }
-      bytes_read = read(n_socket, &buf[len], buf_sz-len-1);
+      if (PRTGIF || ReadFromSocket(n_socket)) {
+         bytes_read = read(n_socket, &buf[len], buf_sz-len-1);
+      } else {
+         /* abort */
+         status = TG_REMOTE_STATUS_INTR;
+         break;
+      }
       if (bytes_read <= 0) {
          if (bytes_read < 0 && (errno == ENOTCONN || errno == ECONNRESET ||
                errno == EPIPE)) {
-            fprintf(stderr, "Network read error.\n");
+            fprintf(stderr, TgLoadString(STID_NETWORK_READ_ERROR), "HTTP");
+            fprintf(stderr, "\n");
             status = TG_REMOTE_STATUS_READ;
          } else if (bytes_read < 0) {
-            fprintf(stderr, "Network error.\n");
+            fprintf(stderr, TgLoadString(STID_NETWORK_ERROR), "HTTP");
+            fprintf(stderr, "\n");
             status = TG_REMOTE_STATUS_NET;
          }
+         if (!UpdateDownloadStats(0, progress_buf)) {
+            *progress_buf = '\0';
+         }
          end_of_file = TRUE;
       } else {
+         if (!UpdateDownloadStats(bytes_read, progress_buf)) {
+            *progress_buf = '\0';
+         }
          len += bytes_read;
       }
-      if (status == TG_REMOTE_STATUS_OK && !end_of_file && UserAbortComm()) {
-         if (buf != NULL) free(buf);
-         ShowRemoteStatus("HTTP: aborted by the user.");
-         if (PRTGIF && cmdLineDumpURL && cmdLineDumpURLShowStatus) {
-            fprintf(stderr, "\n");
-         }
-         return TG_REMOTE_STATUS_INTR;
+      if (status == TG_REMOTE_STATUS_OK && !end_of_file &&
+            (UserAbortComm() || OtherAbortComm())) {
+         /* abort */
+         status = TG_REMOTE_STATUS_INTR;
+         break;
       } else {
+         int chopped_len=(-1);
+
          if (status == TG_REMOTE_STATUS_OK && content_len == (-1)) {
             buf[len] = '\0';
             content_len = GetContentLength(buf, &header_len);
          }
          if (content_len == (-1) || content_len == 0) {
-            sprintf(msg, "HTTP: %1d bytes...", len);
+            /* do not translate -- program constants */
+            sprintf(msg, "HTTP: %1d bytes %s...", len,
+                  (*progress_buf=='\0' ? "" : progress_buf));
+            if (len >= 200) {
+               chopped_len = len;
+               HandleMultipartReplace(buf, &chopped_len);
+            }
          } else {
-            sprintf(msg, "HTTP: %1d of %1d bytes...", len-header_len,
-                  content_len);
+            /* do not translate -- program constants */
+            sprintf(msg, "HTTP: %1d of %1d bytes %s...", len-header_len,
+                  content_len, (*progress_buf=='\0' ? "" : progress_buf));
+            if (status == TG_REMOTE_STATUS_OK &&
+                  len == header_len+content_len) {
+               end_of_file = TRUE;
+            }
          }
          ShowRemoteStatus(msg);
          if (PRTGIF && cmdLineDumpURL && cmdLineDumpURLShowStatus) {
             if (content_len == (-1) || content_len == 0) {
-               fprintf(stderr, "HTTP: %1d bytes...\r", len);
+               /* do not translate -- program constants */
+               fprintf(stderr, "HTTP: %1d bytes %s...\r", len,
+                     (*progress_buf=='\0' ? "" : progress_buf));
             } else {
-               fprintf(stderr, "HTTP: %1d of %1d bytes...\r", len-header_len,
-                     content_len);
+               /* do not translate -- program constants */
+               fprintf(stderr, "HTTP: %1d of %1d bytes %s...\r", len-header_len,
+                     content_len, (*progress_buf=='\0' ? "" : progress_buf));
             }
          }
+         if (chopped_len != (-1)) len = chopped_len;
       }
    } while (status == TG_REMOTE_STATUS_OK && !end_of_file);
+
    if (status == TG_REMOTE_STATUS_OK) {
       buf[len] = '\0';
       *ppsz_buf = buf;
-      if (pn_buf_sz != NULL) *pn_buf_sz = (len+1);
-      sprintf(msg, "HTTP: responses received.");
+      if (pn_buf_sz != NULL) *pn_buf_sz = len;
+      sprintf(msg, TgLoadCachedString(CSTID_RESPONSES_RECEIVED), "HTTP");
       if (debugHttp == 99) {
+         /* debug, do not translate */
          if (!cmdLineDumpURL) {
             fprintf(stdout, "\n==========>>>\n");
             fwrite(buf, sizeof(char), len, stdout);
@@ -1162,7 +1364,20 @@ int HttpDoRead(n_socket, ppsz_buf, pn_buf_sz)
       }
    } else {
       if (buf != NULL) free(buf);
-      sprintf(msg, "HTTP: error encountered in receiving responses.");
+
+      if (status == TG_REMOTE_STATUS_INTR) {
+         if (PRTGIF && cmdLineDumpURL && cmdLineDumpURLShowStatus) {
+            fprintf(stderr, "\n");
+         }
+         /* should get rid of all key presses */
+         while (ESCPressed()) ;
+
+         ResetMultipartReplace(TRUE);
+
+         sprintf(msg, TgLoadString(STID_CONN_ABORT_BY_USER), "HTTP");
+      } else {
+         sprintf(msg, TgLoadString(STID_ERROR_ENCOUNTERED_WHILE_RECV), "HTTP");
+      }
    }
    ShowRemoteStatus(msg);
    if (PRTGIF && cmdLineDumpURL && cmdLineDumpURLShowStatus) {
