@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/import.c,v 4.56 2000/05/20 22:40:56 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/import.c,v 4.61 2000/07/25 20:25:05 william Exp $
  */
 
 #define _INCLUDE_FROM_IMPORT_C_
@@ -38,17 +38,21 @@
 
 #include "attr.e"
 #include "auxtext.e"
+#include "box.e"
 #include "choice.e"
 #include "choose.e"
 #include "cmd.e"
 #include "color.e"
 #include "cursor.e"
+#include "cutpaste.e"
 #include "dialog.e"
 #include "drawing.e"
 #include "dup.e"
 #include "eps.e"
 #include "file.e"
 #include "font.e"
+#include "grid.e"
+#include "imgproc.e"
 #include "import.e"
 #include "mainloop.e"
 #include "mainmenu.e"
@@ -57,6 +61,7 @@
 #include "menu.e"
 #include "move.e"
 #include "names.e"
+#include "navigate.e"
 #include "obj.e"
 #include "page.e"
 #include "raster.e"
@@ -82,6 +87,14 @@ static struct ImportInfoRec *topImportInfo=NULL, *botImportInfo=NULL;
 static int gnMaxImportFilters=0;
 static int showFileNameOnBrowse=TRUE;
 
+static int thumbnailW=160;
+static int thumbnailH=120;
+static int thumbnailPadding=8;
+static int thumbnailXGap=16;
+static int thumbnailYGap=0;
+static int thumbnailX=32;
+static int thumbnailY=32;
+
 void MillisecondSleep(ms)
    int ms;
 {
@@ -351,6 +364,99 @@ void InitImport()
          }
       }
    }
+   thumbnailW = 160;
+   thumbnailH = 120;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"ThumbnailGeometry")) != NULL) {
+      int w=0, h=0;
+
+      if (ParseWHSpec(c_ptr, &w, &h) && w >= 40 && h >= 30) {
+         if (w >= 40 && h >= 30) {
+            thumbnailW = w;
+            thumbnailH = h;
+         } else {
+            fprintf(stderr, TgLoadString(STID_INVALID_GEOM_USE_ALT_GEOM),
+                  TOOL_NAME, "ThumbnailGeometry", c_ptr, "40x30", "160x120");
+            fprintf(stderr, "\n");
+         }
+      }
+   }
+   thumbnailPadding = 8;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"ThumbnailPadding")) != NULL) {
+      int padding=0;
+
+      UtilTrimBlanks(c_ptr);
+      if (sscanf(c_ptr, "%d", &padding) != 1) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
+               TOOL_NAME, "ThumbnailPadding", c_ptr, 8);
+         fprintf(stderr, "\n");
+      } else if (padding < 0 || padding > 100) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_RNG_USE_ALT_VAL),
+               TOOL_NAME, "ThumbnailPadding", c_ptr, 0, 100, 8);
+         fprintf(stderr, "\n");
+      } else {
+         thumbnailPadding = padding;
+      }
+   }
+   thumbnailXGap = 16;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"ThumbnailXGap")) != NULL) {
+      int gap=0;
+
+      UtilTrimBlanks(c_ptr);
+      if (sscanf(c_ptr, "%d", &gap) != 1) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
+               TOOL_NAME, "ThumbnailXGap", c_ptr, 16);
+         fprintf(stderr, "\n");
+      } else if (gap < 0) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
+               TOOL_NAME, "ThumbnailXGap", c_ptr, 16);
+         fprintf(stderr, "\n");
+      } else {
+         thumbnailXGap = gap;
+      }
+   }
+   thumbnailYGap = 0;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"ThumbnailYGap")) != NULL) {
+      int gap=0;
+
+      UtilTrimBlanks(c_ptr);
+      if (sscanf(c_ptr, "%d", &gap) != 1) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
+               TOOL_NAME, "ThumbnailYGap", c_ptr, 16);
+         fprintf(stderr, "\n");
+      } else if (gap < 0) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
+               TOOL_NAME, "ThumbnailYGap", c_ptr, 16);
+         fprintf(stderr, "\n");
+      } else {
+         thumbnailYGap = gap;
+      }
+   }
+   thumbnailX = 32;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"ThumbnailX")) != NULL) {
+      int x=0;
+
+      UtilTrimBlanks(c_ptr);
+      if (sscanf(c_ptr, "%d", &x) != 1) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
+               TOOL_NAME, "ThumbnailX", c_ptr, 32);
+         fprintf(stderr, "\n");
+      } else {
+         thumbnailX = x;
+      }
+   }
+   thumbnailY = 32;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"ThumbnailY")) != NULL) {
+      int y=0;
+
+      UtilTrimBlanks(c_ptr);
+      if (sscanf(c_ptr, "%d", &y) != 1) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
+               TOOL_NAME, "ThumbnailY", c_ptr, 32);
+         fprintf(stderr, "\n");
+      } else {
+         thumbnailY = y;
+      }
+   }
 }
 
 void ImportXBitmapFile()
@@ -584,7 +690,7 @@ void ImportEPSFile(Embed)
 {
    char eps_fname[MAXPATHLENGTH+1], *rest=NULL, tmp_fname[MAXPATHLENGTH+1];
    char **lines=NULL, write_date[32];
-   int rc, short_name, num_lines, epsf_level, image_w, image_h;
+   int i, rc, short_name, num_lines=0, epsf_level, image_w, image_h;
    int remote_file=FALSE;
    float llx, lly, urx, ury;
    Pixmap bitmap=None;
@@ -634,8 +740,8 @@ void ImportEPSFile(Embed)
    SetWatchCursor(drawWindow);
    SetWatchCursor(mainWindow);
    rc = MyReadEPSFile(remote_file ? tmp_fname : eps_fname, &image_w, &image_h,
-         &bitmap, &image, &num_lines, &lines, &epsf_level,
-         &llx, &lly, &urx, &ury, write_date);
+         &bitmap, &image, (Embed ? &num_lines : NULL), (Embed ? &lines : NULL),
+         &epsf_level, &llx, &lly, &urx, &ury, write_date);
 
    if (remote_file) {
       short_name = FALSE;
@@ -664,6 +770,12 @@ void ImportEPSFile(Embed)
    }
    saveEPSLines = FALSE;
 
+   if (!Embed && lines != NULL) {
+      for (i=0; i < num_lines; i++) {
+         if (lines[i] != NULL) free(lines[i]);
+      }
+      free(lines);
+   }
    if (strcmp(defaultEPSScalingStr,"1") != 0) {
       ScaleAnEPSObj(obj_ptr, &defaultEPSScaling);
    }
@@ -773,17 +885,16 @@ void InitGifToXpm()
    }
 }
 
-int ConvertGifToXpm(pszGifPath, pszXpmPath)
+int ConvertGifToXpm(pszGifPath, pszXpmPath, xpm_path_sz)
    char *pszGifPath, *pszXpmPath;
+   int xpm_path_sz;
 {
    FILE *pFile=NULL, *pPipe=NULL;
    char *pszCmd=NULL, szBuf[MAXSTRING];
    int bytes_read;
 
    InitGifToXpm();
-   sprintf(pszXpmPath, "%sTgifXXXXXX", TMP_DIR);
-   mktemp(pszXpmPath);
-   unlink(pszXpmPath);
+   MkTempFile(pszXpmPath, xpm_path_sz, tmpDir, TOOL_NAME);
    pszCmd = (char*)malloc(
          (strlen(gifToXpmCmd)+strlen(pszGifPath)+10)*sizeof(char));
    if (pszCmd == NULL) {
@@ -870,7 +981,8 @@ void ImportGIFFile()
    SetWatchCursor(drawWindow);
    SetWatchCursor(mainWindow);
    SaveStatusStrings();
-   rc = ConvertGifToXpm(remote_file ? tmp_fname : szGifPath, file_name);
+   rc = ConvertGifToXpm((remote_file ? tmp_fname : szGifPath), file_name,
+         sizeof(file_name));
    RestoreStatusStrings();
    SetDefaultCursor(mainWindow);
    ShowCursor();
@@ -906,17 +1018,16 @@ void ImportGIFFile()
    importingFile = FALSE;
 }
 
-int ConvertPngToXpm(pszPngPath, pszXpmPath)
+int ConvertPngToXpm(pszPngPath, pszXpmPath, xpm_path_sz)
    char *pszPngPath, *pszXpmPath;
+   int xpm_path_sz;
 {
    FILE *pFile=NULL, *pPipe=NULL;
    char *pszCmd=NULL, szBuf[MAXSTRING];
    int bytes_read;
 
    InitGifToXpm();
-   sprintf(pszXpmPath, "%sTgifXXXXXX", TMP_DIR);
-   mktemp(pszXpmPath);
-   unlink(pszXpmPath);
+   MkTempFile(pszXpmPath, xpm_path_sz, tmpDir, TOOL_NAME);
    pszCmd = (char*)malloc(
          (strlen(pngToXpmCmd)+strlen(pszPngPath)+10)*sizeof(char));
    if (pszCmd == NULL) {
@@ -1003,7 +1114,8 @@ void ImportPNGFile()
    SetWatchCursor(drawWindow);
    SetWatchCursor(mainWindow);
    SaveStatusStrings();
-   rc = ConvertPngToXpm(remote_file ? tmp_fname : szPngPath, file_name);
+   rc = ConvertPngToXpm((remote_file ? tmp_fname : szPngPath), file_name,
+         sizeof(file_name));
    RestoreStatusStrings();
    SetDefaultCursor(mainWindow);
    ShowCursor();
@@ -1039,17 +1151,16 @@ void ImportPNGFile()
    importingFile = FALSE;
 }
 
-int ConvertJpegToXpm(pszJpegPath, pszXpmPath)
+int ConvertJpegToXpm(pszJpegPath, pszXpmPath, xpm_path_sz)
    char *pszJpegPath, *pszXpmPath;
+   int xpm_path_sz;
 {
    FILE *pFile=NULL, *pPipe=NULL;
    char *pszCmd=NULL, szBuf[MAXSTRING];
    int bytes_read;
 
    InitGifToXpm();
-   sprintf(pszXpmPath, "%sTgifXXXXXX", TMP_DIR);
-   mktemp(pszXpmPath);
-   unlink(pszXpmPath);
+   MkTempFile(pszXpmPath, xpm_path_sz, tmpDir, TOOL_NAME);
    pszCmd = (char*)malloc(
          (strlen(jpegToXpmCmd)+strlen(pszJpegPath)+10)*sizeof(char));
    if (pszCmd == NULL) {
@@ -1136,7 +1247,8 @@ void ImportJPEGFile()
    SetWatchCursor(drawWindow);
    SetWatchCursor(mainWindow);
    SaveStatusStrings();
-   rc = ConvertJpegToXpm(remote_file ? tmp_fname : szJpegPath, file_name);
+   rc = ConvertJpegToXpm((remote_file ? tmp_fname : szJpegPath), file_name,
+         sizeof(file_name));
    RestoreStatusStrings();
    SetDefaultCursor(mainWindow);
    ShowCursor();
@@ -1253,17 +1365,16 @@ int SelectAnImportFilter(pszSelected)
 }
 
 static
-int ConvertAnyToXpm(pii, pszAnyPath, pszXpmPath)
+int ConvertAnyToXpm(pii, pszAnyPath, pszXpmPath, xpm_path_sz)
    struct ImportInfoRec *pii;
    char *pszAnyPath, *pszXpmPath;
+   int xpm_path_sz;
 {
    FILE *pFile=NULL, *pPipe=NULL;
    char *pszCmd=NULL, szBuf[MAXSTRING];
    int bytes_read;
 
-   sprintf(pszXpmPath, "%sTgifXXXXXX", TMP_DIR);
-   mktemp(pszXpmPath);
-   unlink(pszXpmPath);
+   MkTempFile(pszXpmPath, xpm_path_sz, tmpDir, TOOL_NAME);
    pszCmd =
          (char*)malloc((strlen(pii->cmd)+strlen(pszAnyPath)+10)*sizeof(char));
    if (pszCmd == NULL) {
@@ -1362,7 +1473,8 @@ int DoImportOtherFile(pii, psz_path)
    SetWatchCursor(drawWindow);
    SetWatchCursor(mainWindow);
    SaveStatusStrings();
-   rc = ConvertAnyToXpm(pii, remote_file ? tmp_fname : szOtherPath, szXpmPath);
+   rc = ConvertAnyToXpm(pii, (remote_file ? tmp_fname : szOtherPath), szXpmPath,
+         sizeof(szXpmPath));
    RestoreStatusStrings();
    SetDefaultCursor(mainWindow);
    ShowCursor();
@@ -1483,8 +1595,9 @@ int ImportSpecifiedFileType(psz_path, psz_filter)
    return DoImportOtherFile(pii, psz_path);
 }
 
-int ConvertOtherToXpm(pszFile, pszFormat, pszXpmPath)
+int ConvertOtherToXpm(pszFile, pszFormat, pszXpmPath, xpm_path_sz)
    char *pszFile, *pszFormat, *pszXpmPath;
+   int xpm_path_sz;
 {
    struct ImportInfoRec *pii=NULL;
 
@@ -1499,7 +1612,7 @@ int ConvertOtherToXpm(pszFile, pszFormat, pszXpmPath)
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return FALSE;
    }
-   return ConvertAnyToXpm(pii, pszFile, pszXpmPath);
+   return ConvertAnyToXpm(pii, pszFile, pszXpmPath, xpm_path_sz);
 }
 
 void ImportGIFToXPixmapDeck()
@@ -1547,9 +1660,7 @@ void ImportGIFToXPixmapDeck()
       ExposeEventHandler(&ev, TRUE);
    }
    if (!remote_file) {
-      sprintf(tmp_fname, "%sTgifXXXXXX", TMP_DIR);
-      mktemp(tmp_fname);
-      unlink(tmp_fname);
+      MkTempFile(tmp_fname, sizeof(tmp_fname), tmpDir, TOOL_NAME);
       if (UtilCopyFile(gif_fname, tmp_fname) != TG_REMOTE_STATUS_OK) {
          sprintf(gszMsgBox, TgLoadString(STID_CANT_CREATE_NAMED_TMP_FILE),
                tmp_fname);
@@ -1561,7 +1672,7 @@ void ImportGIFToXPixmapDeck()
    }
    /* explode the animated GIF file into parts */
    InitGifToXpm();
-   sprintf(cmd, "cd %s; %s %s", TMP_DIR, gifAnimExplodeCmd,
+   sprintf(cmd, "cd %s; %s %s", tmpDir, gifAnimExplodeCmd,
          (remote_file ? tmp_fname : gif_fname));
    SetWatchCursor(drawWindow);
    SetWatchCursor(mainWindow);
@@ -1643,7 +1754,7 @@ void ImportGIFToXPixmapDeck()
             sprintf(part_fname, part_fname_format, name, i);
             if (!UtilPathExists(part_fname)) {
                sprintf(gszMsgBox, TgLoadString(STID_CANT_IMP_ANIM_GIF_TMP_DIR),
-                     part_fname, TMP_DIR);
+                     part_fname, tmpDir);
                MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
                break;
             }
@@ -1652,7 +1763,7 @@ void ImportGIFToXPixmapDeck()
          /* got all the GIF parts */
          break;
       }
-      rc = ConvertAnyToXpm(&ii, part_fname, xpm_fname);
+      rc = ConvertAnyToXpm(&ii, part_fname, xpm_fname, sizeof(xpm_fname));
       if (!rc) {
          unlink(part_fname);
          break;
@@ -1723,7 +1834,7 @@ void ImportGIFToXPixmapDeck()
       HighLightForward();
 
       historyDepth = 0;
-      GroupSelObj(TRUE, TRUE);
+      GroupSelObj(TRUE, TRUE, TRUE);
       historyDepth = saved_history_depth;
 
       HighLightReverse();
@@ -1761,17 +1872,28 @@ void ImportGIFToXPixmapDeck()
 static int origBrowseX=0, origBrowseY=0, maxBrowseX=0, maxBrowseY=0;
 static int curBrowseX=0, curBrowseY=0, curBrowseRowHeight=0;
 static int savedDirNameLen=0;
+static int browseObjType=INVALID;
 static char savedDirName[MAXPATHLENGTH+1];
 
 static
-void InitBrowse(pszDir)
+void InitBrowse(pszDir, ObjType)
    char *pszDir;
+   int ObjType;
 {
-   curBrowseX = origBrowseX = drawOrigX+ABS_SIZE(EIGHTH_INCH);
-   maxBrowseX = drawOrigX+drawWinW;
-   curBrowseY = origBrowseY = drawOrigY+ABS_SIZE(EIGHTH_INCH);
-   maxBrowseY = drawOrigY+drawWinH;
-   curBrowseRowHeight = 0;
+   browseObjType = ObjType;
+   if (ObjType == OBJ_ICON) {
+      curBrowseX = origBrowseX = thumbnailX;
+      maxBrowseX = paperWidth - curBrowseX;
+      curBrowseY = origBrowseY = thumbnailY;
+      maxBrowseY = paperHeight - curBrowseY;
+      curBrowseRowHeight = 0;
+   } else {
+      curBrowseX = origBrowseX = drawOrigX+ABS_SIZE(EIGHTH_INCH);
+      maxBrowseX = drawOrigX+drawWinW;
+      curBrowseY = origBrowseY = drawOrigY+ABS_SIZE(EIGHTH_INCH);
+      maxBrowseY = drawOrigY+drawWinH;
+      curBrowseRowHeight = 0;
+   }
    if (pszDir != NULL) {
       strcpy(savedDirName, pszDir);
       savedDirNameLen = strlen(savedDirName);
@@ -1796,7 +1918,9 @@ int CheckExecInterrupt()
          return TRUE;
       } else if (ev.type == ConfigureNotify) {
          Reconfigure(FALSE);
-         maxBrowseX = drawOrigX+drawWinW;
+         if (browseObjType != OBJ_ICON) {
+            maxBrowseX = drawOrigX+drawWinW;
+         }
       }
    }
    while (XCheckMaskEvent(mainDisplay, VisibilityChangeMask, &ev)) {
@@ -1829,6 +1953,210 @@ int CheckExecInterrupt()
    return FALSE;
 }
 
+static
+struct ObjRec *ImportForThumbnails(fname, p_bbox_w, p_bbox_h)
+   char *fname;
+   int *p_bbox_w, *p_bbox_h;
+{
+   struct ObjRec *imported_obj=NULL;
+   struct ObjRec *saved_top_obj=topObj, *saved_bot_obj=botObj;
+   struct BBRec saved_draw_win_bbox;
+
+   (*p_bbox_w) = (*p_bbox_h) = 0;
+   curPage->top = curPage->bot = topObj = botObj = NULL;
+
+   importingFile = TRUE;
+   pastingFile = TRUE;
+   memcpy(&saved_draw_win_bbox, &drawWinBBox, sizeof(struct BBRec));
+   drawWinBBox.ltx = drawWinBBox.rbx = drawOrigX-128;
+   drawWinBBox.lty = drawWinBBox.rby = drawOrigY-128;
+   if (ImportGivenFile(fname, FALSE, FALSE) == TRUE) {
+      if (numObjSelected > 0) {
+         struct BoxRec *box_ptr=NULL;
+
+         CreateBoxObj(selLtX, selLtY, selRbX, selRbY, TRUE);
+         box_ptr = topObj->detail.b;
+         box_ptr->fill = NONEPAT;
+         box_ptr->pen = NONEPAT;
+         RemoveAllSel();
+         SelAllObj(FALSE);
+         GroupSelObj(FALSE, FALSE, FALSE);
+
+         imported_obj = topObj;
+         *p_bbox_w = imported_obj->bbox.rbx-imported_obj->bbox.ltx;
+         *p_bbox_h = imported_obj->bbox.rby-imported_obj->bbox.lty;
+#ifdef _TGIF_DBG /* debug, do not translate */
+         TgAssert(topObj == botObj,
+               "too many objects found in ImportForThumbnails()", NULL);
+#endif /* _TGIF_DBG */
+         RemoveAllSel();
+      }
+      MakeQuiescent();
+   }
+   memcpy(&drawWinBBox, &saved_draw_win_bbox, sizeof(struct BBRec));
+   importingFile = FALSE;
+   pastingFile = FALSE;
+
+   curPage->top = topObj = saved_top_obj;
+   curPage->bot = botObj = saved_bot_obj;
+
+   return imported_obj;
+}
+
+static
+int RegenerateImageFile(pszPath)
+   char *pszPath;
+{
+   int saved_colordump=colorDump, saved_wheretoprint=whereToPrint;
+
+   *gszImageProcXPmFile = '\0';
+   gnConvolving = FALSE;
+
+   colorDump = TRUE;
+   whereToPrint = XBM_FILE;
+   gnInImageProc = TRUE;
+
+   SetWatchCursor(drawWindow);
+   SetWatchCursor(mainWindow);
+   DumpXBitmapFile(gnInImageProc, TRUE);
+   SetDefaultCursor(mainWindow);
+   ShowCursor();
+
+   gnInImageProc = FALSE;
+   whereToPrint = saved_wheretoprint;
+   colorDump = saved_colordump;
+   if (*gszImageProcXPmFile == '\0') return FALSE;
+   strcpy(pszPath, gszImageProcXPmFile);
+   return TRUE;
+}
+
+static
+struct ObjRec *MyRegenerateImage(obj_ptr)
+   struct ObjRec *obj_ptr;
+{
+   int image_w=0, image_h=0, w=0, h=0, short_name=FALSE, rc=INVALID;
+   int ltx=0, lty=0;
+   int ncolors=0, chars_per_pixel=0, first_pixel_is_bg=0, *pixels=NULL;
+   char *color_char=NULL, **color_str=NULL, *xpm_data=NULL, *rest=NULL;
+   char szPath[MAXPATHLENGTH+1];
+   Pixmap pixmap=None, bitmap=None;
+   XImage *image=NULL, *bitmap_image=NULL;
+   struct ObjRec *new_obj=NULL;
+
+   ltx = obj_ptr->obbox.ltx;
+   lty = obj_ptr->obbox.lty;
+
+   AddObj(NULL, topObj, obj_ptr);
+   PushPageInfo();
+
+   rc = RegenerateImageFile(szPath);
+
+   DelAllObj();
+   PopPageInfo();
+   if (!rc) {
+      return NULL;
+   }
+   SetWatchCursor(drawWindow);
+   SetWatchCursor(mainWindow);
+   rc = MyReadPixmapFile(szPath, &image_w, &image_h, &w, &h, &pixmap,
+         &image, &bitmap, &bitmap_image, &ncolors, &chars_per_pixel,
+         &first_pixel_is_bg, &color_char, &color_str, &pixels, &xpm_data);
+   SetDefaultCursor(mainWindow);
+   ShowCursor();
+
+   if ((short_name=IsPrefix(bootDir, szPath, &rest))) ++rest;
+   if (rc != BitmapSuccess) {
+      sprintf(gszMsgBox, TgLoadString(STID_CANNOT_IMPORT_XPM_FILE),
+            (short_name ? rest : szPath));
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      unlink(szPath);
+      return NULL;
+   }
+   unlink(szPath);
+   new_obj = CreateXPmObj(image_w, image_h, w, h, pixmap, image, bitmap,
+         bitmap_image, ncolors, chars_per_pixel, first_pixel_is_bg, color_char,
+         color_str, pixels, xpm_data);
+   MoveObj(new_obj, ltx-new_obj->obbox.ltx, lty-new_obj->obbox.lty);
+   new_obj->tmp_parent = NULL;
+   SetFileModified(TRUE);
+   justDupped = FALSE;
+
+   return new_obj;
+}
+
+static
+struct ObjRec *GenerateXpmThumbnail(imported_obj)
+   struct ObjRec *imported_obj;
+{
+   struct ObjRec *saved_top_obj=topObj, *saved_bot_obj=botObj, *new_obj=NULL;
+   double orig_aspect_ratio=(double)0, aspect_ratio=(double)0;
+   struct BBRec saved_draw_win_bbox;
+   int bbox_w=(imported_obj->bbox.rbx-imported_obj->bbox.ltx);
+   int bbox_h=(imported_obj->bbox.rby-imported_obj->bbox.lty);
+   int final_w=0, final_h=0;
+
+   curPage->top = curPage->bot = topObj = botObj = NULL;
+
+   aspect_ratio = (((double)thumbnailW)/((double)thumbnailH));
+   orig_aspect_ratio = (((double)bbox_w)/((double)bbox_h));
+
+   memcpy(&saved_draw_win_bbox, &drawWinBBox, sizeof(struct BBRec));
+   drawWinBBox.ltx = drawWinBBox.rbx = drawOrigX-128;
+   drawWinBBox.lty = drawWinBBox.rby = drawOrigY-128;
+
+   if (bbox_w <= thumbnailW && bbox_h <= thumbnailH) {
+      /* do not scale */
+      final_w = thumbnailW;
+      final_h = thumbnailH;
+   } else {
+      double scale=(double)0;
+
+      if (orig_aspect_ratio > aspect_ratio) {
+         final_w = thumbnailW;
+         final_h = round(((double)thumbnailW) / orig_aspect_ratio);
+         scale = ((double)bbox_w) / ((double)thumbnailW);
+      } else {
+         final_h = thumbnailH;
+         final_w = round(((double)thumbnailH) * orig_aspect_ratio);
+         scale = ((double)bbox_h) / ((double)thumbnailH);
+      }
+      ScaleObjLikeScaleEverything(imported_obj, scale, FALSE);
+   }
+   new_obj = MyRegenerateImage(imported_obj); /* imported_obj is freed here */
+   if (new_obj != NULL) {
+      struct BoxRec *box_ptr=NULL;
+      int obj_w=(new_obj->obbox.rbx-new_obj->obbox.ltx);
+      int obj_h=(new_obj->obbox.rby-new_obj->obbox.lty);
+      int new_ltx=((thumbnailW+(thumbnailPadding<<1)-obj_w)>>1);
+      int new_lty=((thumbnailH+(thumbnailPadding<<1)-obj_h)>>1);
+
+      CreateBoxObj(0, 0, thumbnailW+(thumbnailPadding<<1),
+            thumbnailH+(thumbnailPadding<<1), TRUE);
+      box_ptr = topObj->detail.b;
+      box_ptr->fill = BACKPAT;
+      box_ptr->pen = SOLIDPAT;
+      box_ptr->dash = 0;
+      box_ptr->width = 0;
+      strcpy(box_ptr->width_spec, "0");
+      AdjObjBBox(topObj);
+      MoveObj(new_obj, new_ltx-new_obj->obbox.ltx, new_lty-new_obj->obbox.lty);
+
+      AddObj(NULL, topObj, new_obj);
+      SelAllObj(FALSE);
+      GroupSelObj(FALSE, FALSE, FALSE);
+
+      RemoveAllSel();
+      new_obj = topObj;
+      UnlinkObj(topObj);
+   }
+   memcpy(&drawWinBBox, &saved_draw_win_bbox, sizeof(struct BBRec));
+
+   curPage->top = topObj = saved_top_obj;
+   curPage->bot = botObj = saved_bot_obj;
+
+   return new_obj;
+}
+
 static
 int BrowseDir(DirName, ExtStr, ExtStrLen, ObjType, pii)
    char *DirName, *ExtStr;
@@ -1837,7 +2165,7 @@ int BrowseDir(DirName, ExtStr, ExtStrLen, ObjType, pii)
    /* returns TRUE if interrupted */
 {
    char name[MAXPATHLENGTH+1];
-   int interrupted=FALSE;
+   int interrupted=FALSE, esave_len=0;
 #ifdef VMS
    int len;
 #endif /* VMS */
@@ -1845,6 +2173,7 @@ int BrowseDir(DirName, ExtStr, ExtStrLen, ObjType, pii)
    DIR_ENTRY *d;
 
    if ((dirp=opendir(DirName)) == NULL) return interrupted;
+   esave_len = strlen("EmergencySave.");
    if (pii == NULL) {
       sprintf(gszMsgBox, TgLoadString(STID_BROWSING_DIR_FOR_TYPE_FILES),
             DirName, ExtStr);
@@ -1855,11 +2184,12 @@ int BrowseDir(DirName, ExtStr, ExtStrLen, ObjType, pii)
    Msg(gszMsgBox);
    while ((d=readdir(dirp)) != NULL) {
       int ncolors, chars_per_pixel, first_pixel_is_bg, * pixels;
-      int image_w=0, image_h=0, w=0, h=0, x_hot, y_hot, rc;
+      int image_w=0, image_h=0, w=0, h=0, x_hot, y_hot, rc=INVALID;
       unsigned int tmp_w, tmp_h;
       char *color_char, **color_str, *xpm_data=NULL;
       Pixmap bitmap=None, pixmap=None;
       XImage *image=NULL, *bitmap_image=NULL;
+      struct ObjRec *imported_obj=NULL;
 
       if (CheckExecInterrupt()) {
          interrupted = TRUE;
@@ -1868,6 +2198,9 @@ int BrowseDir(DirName, ExtStr, ExtStrLen, ObjType, pii)
       if (!ExtensionMatch(ExtStr, d->d_name)) {
          continue;
       }
+      if (strncmp(d->d_name, "EmergencySave.", esave_len) == 0) {
+         continue;
+      }
       sprintf(name, "%s%c%s", DirName, DIR_SEP, d->d_name);
       sprintf(gszMsgBox, TgLoadCachedString(CSTID_OPENING_GIVEN),
             &name[savedDirNameLen+1]);
@@ -1876,12 +2209,12 @@ int BrowseDir(DirName, ExtStr, ExtStrLen, ObjType, pii)
          rc = XReadBitmapFile(mainDisplay, mainWindow, name, &tmp_w, &tmp_h,
                &bitmap, &x_hot, &y_hot);
          w = (int)tmp_w; h = (int)tmp_h;
-      } else {
+      } else if (ObjType == OBJ_XPM) {
          if (pii != NULL) {
             char szXpmPath[MAXPATHLENGTH+1];
 
             SaveStatusStrings();
-            rc = ConvertAnyToXpm(pii, name, szXpmPath);
+            rc = ConvertAnyToXpm(pii, name, szXpmPath, sizeof(szXpmPath));
             RestoreStatusStrings();
             if (!rc) continue;
             rc = MyReadPixmapFile(szXpmPath, &image_w, &image_h, &w, &h,
@@ -1895,24 +2228,48 @@ int BrowseDir(DirName, ExtStr, ExtStrLen, ObjType, pii)
                   &chars_per_pixel, &first_pixel_is_bg, &color_char,
                   &color_str, &pixels, &xpm_data);
          }
+      } else if (ObjType == OBJ_ICON) {
+         imported_obj = ImportForThumbnails(name, &tmp_w, &tmp_h);
+         if (imported_obj == NULL) continue;
+         rc = BitmapSuccess;
+         w = (int)tmp_w;
+         h = (int)tmp_h;
+      } else {
       }
       if (rc == BitmapSuccess) {
-         struct ObjRec *obj_ptr;
+         struct ObjRec *obj_ptr=NULL;
          int short_name;
-         char *rest;
+         char *rest=NULL;
 
          if (ObjType == OBJ_XBM) {
             obj_ptr = CreateXBmObj(w, h, w, h, bitmap, NULL);
-         } else {
+         } else if (ObjType == OBJ_XPM) {
             obj_ptr = CreateXPmObj(image_w, image_h, w, h, pixmap, image,
                   bitmap, bitmap_image, ncolors, chars_per_pixel,
                   first_pixel_is_bg, color_char, color_str, pixels, xpm_data);
+         } else if (ObjType == OBJ_ICON) {
+            obj_ptr = GenerateXpmThumbnail(imported_obj);
+            /* imported_obj is deleted in GenerateXpmThumbnail() */
+            if (obj_ptr == NULL) {
+               continue;
+            }
+            w = (int)(obj_ptr->obbox.rbx-obj_ptr->obbox.ltx);
+            h = (int)(obj_ptr->obbox.rby-obj_ptr->obbox.lty);
+         } else {
          }
          if ((short_name=IsPrefix((curDirIsLocal ? curDir : curLocalDir), name,
                &rest))) {
             ++rest;
          }
-         AddAttrByNameAndValue(obj_ptr, "file_name=", short_name ? rest : name);
+         if (ObjType == OBJ_ICON) {
+            if (imported_obj->type != OBJ_TEXT) {
+               AddAttrByNameAndValue(obj_ptr, "href=",
+                     (short_name ? rest : name));
+            }
+         } else {
+            AddAttrByNameAndValue(obj_ptr, "file_name=",
+                  (short_name ? rest : name));
+         }
          if (showFileNameOnBrowse && obj_ptr->fattr != NULL &&
                obj_ptr->fattr == obj_ptr->lattr) {
             struct AttrRec *attr_ptr=obj_ptr->fattr;
@@ -1928,33 +2285,53 @@ int BrowseDir(DirName, ExtStr, ExtStrLen, ObjType, pii)
             attr_lty = obj_ptr->bbox.rby;
             MoveObj(attr_ptr->obj, attr_ltx-attr_ptr->obj->obbox.ltx,
                   attr_lty-attr_ptr->obj->obbox.lty);
-            if (attr_w > w) w = attr_w;
+            if (attr_w > w && ObjType != OBJ_ICON) w = attr_w;
             h += attr_h;
          }
          AdjObjBBox(obj_ptr);
-         if (curBrowseX+w > maxBrowseX) {
-            curBrowseX = origBrowseX;
-            curBrowseY += curBrowseRowHeight;
-            while (curBrowseY > maxBrowseY) {
-               ForceScrollDown(TRUE);
-               maxBrowseY = drawOrigY+drawWinH;
+         if (ObjType == OBJ_ICON) {
+fprintf(stderr, "maxBrowseX = %1d\n", maxBrowseX);
+            if (curBrowseX+w > maxBrowseX-origBrowseX) {
+               curBrowseX = origBrowseX;
+               curBrowseY += curBrowseRowHeight+thumbnailYGap;
+               while (curBrowseY > maxBrowseY) {
+                  ForceScrollDown(TRUE);
+                  maxBrowseY = drawOrigY+drawWinH;
+               }
+               curBrowseRowHeight = h;
+            } else if (h > curBrowseRowHeight) {
+               curBrowseRowHeight = h;
+            }
+         } else {
+            if (curBrowseX+w > maxBrowseX) {
+               curBrowseX = origBrowseX;
+               curBrowseY += curBrowseRowHeight;
+               while (curBrowseY > maxBrowseY) {
+                  ForceScrollDown(TRUE);
+                  maxBrowseY = drawOrigY+drawWinH;
+               }
+               curBrowseRowHeight = h;
+            } else if (h > curBrowseRowHeight) {
+               curBrowseRowHeight = h;
             }
-            curBrowseRowHeight = h;
-         } else if (h > curBrowseRowHeight) {
-            curBrowseRowHeight = h;
          }
          obj_ptr->tmp_parent = NULL;
          AddObj(NULL, topObj, obj_ptr);
 
-         MoveObj(topObj, curBrowseX-topObj->bbox.ltx,
-               curBrowseY-topObj->bbox.lty);
+         MoveObj(topObj, curBrowseX-topObj->obbox.ltx,
+               curBrowseY-topObj->obbox.lty);
          numRedrawBBox = 0;
          DrawObj(drawWindow, topObj);
          RecordNewObjCmd();
          SetFileModified(TRUE);
          justDupped = FALSE;
          XSync(mainDisplay, False);
-         curBrowseX += w;
+         if (ObjType == OBJ_ICON) {
+            curBrowseX += w + thumbnailXGap;
+         } else {
+            curBrowseX += w;
+         }
+      } else {
       }
    }
    closedir(dirp);
@@ -2018,7 +2395,7 @@ void StartBrowse(DirName, ExtStr, ExtStrLen, ObjType, pii)
    SetWatchCursor(mainWindow);
    MakeQuiescent();
    SaveStatusStrings();
-   InitBrowse(DirName);
+   InitBrowse(DirName, ObjType);
    StartCompositeCmd();
    ShowInterrupt(1);
    textJust = JUST_C;
@@ -2037,6 +2414,40 @@ void StartBrowse(DirName, ExtStr, ExtStrLen, ObjType, pii)
    XFlush(mainDisplay);
 }
 
+void CreateThumbnails()
+{
+   static char stszObjFileExt[20], stszGzObjFileExt[20];
+   static char stszSymFileExt[20], stszPinFileExt[20];
+   static int initialized=FALSE;
+   char buf[MAXSTRING];
+   int len=0;
+
+   MakeQuiescent();
+
+   if (firstCmd != NULL) {
+      if (!OkToFlushUndoBuffer(
+            TgLoadString(STID_CREATE_THUMBNAIL_CAUSE_FLUSH))) {
+         SetCurChoice(curChoiceBeforeMakeQuiescent);
+         return;
+      }
+   }
+   if (!initialized) {
+      sprintf(stszObjFileExt, ".%s", OBJ_FILE_EXT);
+      sprintf(stszGzObjFileExt, ".%s.gz", OBJ_FILE_EXT);
+      initialized = TRUE;
+   }
+   sprintf(buf, "%s;%s;%s;%s", stszObjFileExt, stszGzObjFileExt,
+         stszSymFileExt, stszPinFileExt);
+   len = strlen(buf);
+   if (strcmp(OBJ_FILE_EXT,"obj") != 0) {
+      strcat(buf, ";.obj");
+   }
+   strcat(buf, ";.obj.gz;.tgo;.tgo.gz");
+   StartBrowse((curDirIsLocal ? curDir : curLocalDir), buf, strlen(buf),
+         OBJ_ICON, NULL);
+   CleanUpCmds();
+}
+
 void BrowseXBitmap()
 {
    StartBrowse((curDirIsLocal ? curDir : curLocalDir), XBM_FILE_EXT,
@@ -2218,9 +2629,7 @@ void ImportCapturedImage()
    SetWatchCursor(drawWindow);
    SetWatchCursor(mainWindow);
 
-   sprintf(tmp_fname, "%sTgifXXXXXX", TMP_DIR);
-   mktemp(tmp_fname);
-   unlink(tmp_fname);
+   MkTempFile(tmp_fname, sizeof(tmp_fname), tmpDir, TOOL_NAME);
 
    if (installedColormaps != NULL) {
       saved_colormap = mainColormap;
