@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/mainloop.c,v 4.62 2000/05/10 23:55:38 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/mainloop.c,v 4.69 2000/08/07 20:04:10 william Exp $
  */
 
 #define _INCLUDE_FROM_MAINLOOP_C_
@@ -85,6 +85,7 @@
 #include "setup.e"
 #include "shape.e"
 #include "shortcut.e"
+#include "special.e"
 #include "stk.e"
 #include "strtbl.e"
 #include "tangram2.e"
@@ -347,6 +348,7 @@ void CleanUp()
    CleanUpHtml();
    CleanUpRecentFiles();
    CleanUpMeasureTooltip();
+   CleanUpSpecial();
 
    if (iconWindowCreated) {
       XDestroyWindow(mainDisplay, iconBaseWindow);
@@ -522,14 +524,12 @@ char *GunzipFileIntoTemp(gzipped_fname)
    FILE *pfp=NULL, *out_fp=NULL;
    char *uncompress_cmd=gunzipCmd, *tmp_fname=NULL, *cmd=NULL;
 
-   tmp_fname = (char*)malloc((strlen(TMP_DIR)+20)*sizeof(char));
+   tmp_fname = (char*)malloc((strlen(tmpDir)+20)*sizeof(char));
    if (tmp_fname == NULL) {
       FailAllocMessage();
       return NULL;
    }
-   sprintf(tmp_fname, "%sTgifXXXXXX", TMP_DIR);
-   mktemp(tmp_fname);
-   unlink(tmp_fname);
+   MkTempFile(tmp_fname, (strlen(tmpDir)+19)*sizeof(char), tmpDir, TOOL_NAME);
    if ((out_fp=fopen(tmp_fname, "w")) == NULL) {
       sprintf(gszMsgBox, TgLoadString(STID_CANNOT_OPEN_FILE_FOR_WRITING),
             tmp_fname);
@@ -922,7 +922,7 @@ int JustLoadAFile(pszFile, pnNeedToCheckAutoExec, nAttemptToMerge, pnAbortMerge)
          curFileDefined = TRUE;
          if (file_is_remote) {
             SetCurDir(remote_fname);
-            if (!foundGoodStateObject) PasteString(remote_buf);
+            if (!foundGoodStateObject) PasteString(remote_buf, TRUE, TRUE);
          } else {
             if (*gzipped_fname == '\0') {
                if (*file_name == DIR_SEP) {
