@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/menuinfo.c,v 4.85 2000/05/20 22:52:01 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/menuinfo.c,v 4.97 2000/08/04 13:39:18 william Exp $
  */
 
 #define _INCLUDE_FROM_MENUINFO_C_
@@ -54,6 +54,7 @@
 #include "raster.e"
 #include "scroll.e"
 #include "shape.e"
+#include "special.e"
 #include "stream.e"
 #include "tangram2.e"
 #include "text.e"
@@ -464,6 +465,10 @@ static TgMenuInfo driverMenuInfo={ TGMUTYPE_TEXT, driverMenuItemInfo,
                                    TgCreateMenuFromMenuInfo };
 
 static TgMenuItemInfo browseMenuItemInfo[] = {
+   { N_("Create Thumbnails"), NULL,
+     N_("Create a thumbnail map of all object files"), NULL,
+     CMDID_CREATETHUMBNAILS },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Browse X Bitmap"), NULL,
      N_("Recursively read in all X11 bitmap files"), NULL,
      CMDID_BROWSEXBITMAP },
@@ -1329,6 +1334,37 @@ static TgMenuItemInfo animateMenuItemInfo[] = {
 static TgMenuInfo animateMenuInfo={ TGMUTYPE_TEXT, animateMenuItemInfo,
                                     TgCreateMenuFromMenuInfo };
 
+static TgMenuItemInfo portMenuItemInfo[] = {
+   { N_("Connect Two Ports by a Wire..."), NULL,
+     N_("Connect two ports by a wire"), NULL,
+     CMDID_CONNECT2PORTSBYWIRE },
+   { N_("Show Wire Signal Name"), NULL,
+     N_("Toggle between showing (with placing) and hiding the wire signal name when connecting ports"), NULL,
+     CMDID_TOGGLESHOWWIRESIGNALNAME },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Clear Signal Name for a Port..."), NULL,
+     N_("Clear the 'signal_name=' attribute for a port"), NULL,
+     CMDID_CLEARSIGNAMEFORAPORT },
+   { N_("Rename Signal Name for a Port..."), NULL,
+     N_("Rename the 'signal_name=' attribute for a port"), NULL,
+     CMDID_RENAMESIGNAMEFORAPORT },
+   { NULL, NULL, NULL, NULL, INVALID }
+};
+static TgMenuInfo portMenuInfo={ TGMUTYPE_TEXT, portMenuItemInfo,
+                                 CreatePortMenu };
+
+static TgMenuItemInfo tableMenuItemInfo[] = {
+   { N_("Merge With Table..."), NULL,
+     N_("Merge selected object with a table file"), NULL,
+     CMDID_MERGEWITHTABLE },
+   { N_("Export To Table..."), NULL,
+     N_("Export attributes of selected objects to a table file"), NULL,
+     CMDID_EXPORTTOTABLE },
+   { NULL, NULL, NULL, NULL, INVALID }
+};
+static TgMenuInfo tableMenuInfo={ TGMUTYPE_TEXT, tableMenuItemInfo,
+                                  TgCreateMenuFromMenuInfo };
+
 static TgMenuItemInfo specialMenuItemInfo[] = {
    { N_("Make Symbolic"), "Alt+Cntrl+m",
      N_("Turn a selected grouped/icon object into a symbol object"), NULL,
@@ -1372,18 +1408,15 @@ static TgMenuItemInfo specialMenuItemInfo[] = {
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Domain"), TGMUITEM_SUBMENU,
      N_("Domain submenu >>>"), &domainMenuInfo, INVALID },
+   { N_("Ports and Signals"), TGMUITEM_SUBMENU,
+     N_("Ports and Signal submenu >>>"), &portMenuInfo, INVALID },
+   { N_("Table"), TGMUITEM_SUBMENU,
+     N_("Table submenu >>>"), &tableMenuInfo, INVALID },
    { N_("Misc"), TGMUITEM_SUBMENU,
      N_("Misc submenu >>>"), &animateMenuInfo, INVALID },
    { N_("Return"), TGMUITEM_SUBMENU,
      N_("Return submenu >>>"), &driverMenuInfo, INVALID },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
-   { N_("Merge With Table..."), NULL,
-     N_("Merge selected object with a table file"), NULL,
-     CMDID_MERGEWITHTABLE },
-   { N_("Export To Table..."), NULL,
-     N_("Export attributes of selected objects to a table file"), NULL,
-     CMDID_EXPORTTOTABLE },
-   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Exec Cmds From File..."), NULL,
      N_("Execute internal commands from a file"), NULL,
      CMDID_EXECCMDSFROMFILE },
