@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/msg.c,v 4.46 2000/08/28 16:49:23 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/msg.c,v 4.47 2000/10/29 15:45:28 william Exp $
  */
 
 #define _INCLUDE_FROM_MSG_C_
@@ -126,20 +126,28 @@ int FailToWriteFileMessage(file_name)
    return FALSE;
 }
 
-int FailToOpenMessage(file_name, rw, extra_msg)
-   char *file_name, *rw, *extra_msg;
+int FailToOpenMessage(file_name, rwa, extra_msg)
+   char *file_name, *rwa, *extra_msg;
+   /* rwa is either "r", "w", or "a" */
 {
    char msg[MAXPATHLENGTH<<1];
+   int msgid=0;
 
+   switch (*rwa) {
+   case 'r': msgid = STID_CANNOT_OPEN_FILE_FOR_READING; break;
+   case 'w': msgid = STID_CANNOT_OPEN_FILE_FOR_WRITING; break;
+   case 'a': msgid = STID_CANNOT_OPEN_FILE_FOR_APPEND; break;
+   default: msgid = INVALID; break;
+   }
    if (extra_msg == NULL) {
-      sprintf(msg, TgLoadString(STID_CANNOT_OPEN_FILE_FOR_READING), file_name);
+      sprintf(msg, TgLoadString(msgid), file_name);
       if (PRTGIF) {
          fprintf(stderr, "%s\n", msg);
       } else {
          MsgBox(msg, TOOL_NAME, INFO_MB);
       }
    } else {
-      sprintf(msg, TgLoadString(STID_CANNOT_OPEN_FILE_FOR_READING), file_name);
+      sprintf(msg, TgLoadString(msgid), file_name);
       strcat(msg, "  ");
       strcat(msg, extra_msg);
       if (PRTGIF) {
