@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/names.c,v 4.45 2000/05/02 15:03:59 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/names.c,v 4.46 2000/10/25 19:05:09 william Exp $
  */
 
 #define _INCLUDE_FROM_NAMES_C_
@@ -582,7 +582,7 @@ void InitDomain()
    int default_found=FALSE, need_to_copy_domain_into_ini=FALSE;
    char *pszDomainNames=NULL, *psz_default_path=NULL;
 
-   sprintf(gszDomainIniFile, "%s/domain.ini", tgifDir);
+   sprintf(gszDomainIniFile, "%s%cdomain.ini", tgifDir, DIR_SEP);
 
    domainInIni = FALSE;
    domainInResource = TRUE;
