@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/navigate.c,v 4.34 2000/02/22 00:32:49 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/navigate.c,v 4.35 2000/10/25 19:04:43 william Exp $
  */
 
 #define _INCLUDE_FROM_NAVIGATE_C_
@@ -489,7 +489,8 @@ int InitHotListFileName()
 
          hotListFileName = (char*)malloc((len+1)*sizeof(char));
          if (hotListFileName == NULL) FailAllocMessage();
-         sprintf(hotListFileName, "%s/.%s_hotlist", homeDir, TOOL_NAME);
+         sprintf(hotListFileName, "%s%c.%s_hotlist", homeDir, DIR_SEP,
+               TOOL_NAME);
       }
    }
    validHotListFileName = TRUE;
