@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/obj.c,v 4.9 2000/01/15 22:24:00 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/obj.c,v 4.10 2000/11/10 17:09:57 william Exp $
  */
 
 #define _INCLUDE_FROM_OBJ_C_
@@ -838,6 +838,10 @@ void ClearObjCachesInAllPages()
  
       AdjCaches();
       AdjSplineVs();
+      curPage->draw_orig_x = drawOrigX;
+      curPage->draw_orig_y = drawOrigY;
+      curPage->zoom_scale = zoomScale;
+      curPage->zoomed_in = zoomedIn;
    }
    topObj = saved_top_obj;
    botObj = saved_bot_obj;
