@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/page.c,v 4.48 2000/05/10 19:31:28 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/page.c,v 4.51 2000/07/22 20:17:31 william Exp $
  */
 
 #define _INCLUDE_FROM_PAGE_C_
@@ -610,14 +610,6 @@ int EditOrSelectPageNames(TopStr, Which, entries, num_entries)
    return rc;
 }
 
-static
-int BlankStr(s)
-   register char *s;
-{
-   while (*s == ' ') s++;
-   return (*s == '\0');
-}
-
 void GotoPage()
 {
    int num_entries=0;
@@ -939,7 +931,7 @@ void AddPage(AfterCurPage)
          sprintf(file_name, "%s%c%s", bootDir, DIR_SEP, attr_ptr->attr_value.s);
       }
       importingFile = TRUE;
-      if (ImportGivenFile(file_name, TRUE)) {
+      if (ImportGivenFile(file_name, TRUE, TRUE)) {
          sprintf(gszMsgBox, TgLoadString(STID_NAMED_TEMPLATE_LOADED),
                attr_ptr->attr_value.s);
          Msg(gszMsgBox);
@@ -1107,6 +1099,7 @@ void PrintOnePage()
       switch (whereToPrint) {
       case XBM_FILE:
       case TEXT_FILE:
+      case NETLIST_FILE:
       case GIF_FILE:
       case HTML_FILE:
       case PNG_FILE:
@@ -1476,6 +1469,7 @@ void PrintPages()
    case LATEX_FIG:
    case XBM_FILE:
    case TEXT_FILE:
+   case NETLIST_FILE:
    case EPSI_FILE:
    case GIF_FILE:
    case HTML_FILE:
