@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/polygon.c,v 4.15 2000/04/20 16:28:29 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/polygon.c,v 4.16 2000/06/26 02:36:49 william Exp $
  */
 
 #define _INCLUDE_FROM_POLYGON_C_
@@ -1050,7 +1050,7 @@ void InputPolygonPts()
    } while (more_polygon);
    if (num_polygons > 1 || started_composite) {
       SelectAndHighLightNewObjects(saved_top_obj);
-      GroupSelObj(TRUE, TRUE);
+      GroupSelObj(TRUE, TRUE, TRUE);
       EndCompositeCmd();
 
       SetFileModified(TRUE);
