@@ -28,13 +28,14 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/ps.c,v 4.14 2000/01/15 22:24:05 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/ps.c,v 4.18 2000/08/23 22:26:49 william Exp $
  */
 
 #define _INCLUDE_FROM_PS_C_
 
 #include "tgifdefs.h"
 
+#include "file.e"
 #include "msg.e"
 #include "ps.e"
 #include "util.e"
@@ -62,6 +63,7 @@ static int psUseCenterText=FALSE;
 static int psUseRightText=FALSE;
 static int psUseMiniLines=FALSE;
 static int psUseReencode=FALSE;
+static int psUseReencodeFF8=FALSE;
 
 #ifdef _PS_USE_EUC_FONT
 static int psUseEucFont=FALSE;
@@ -179,7 +181,13 @@ void PSUseReencode(font_str)
       psUseReencode = TRUE;
       psDictCount += 2;
    }
-   if (preDumpSetup && psUseReencode) {
+#ifdef _PRTGIF_USE_ISO_LATIN_1_ALWAYS
+   if (PRTGIF && !psUseReencodeFF8) {
+      psUseReencodeFF8 = TRUE;
+      psDictCount += 2;
+   }
+#endif /* _PRTGIF_USE_ISO_LATIN_1_ALWAYS */
+   if (font_str != NULL && preDumpSetup && psUseReencode) {
       struct ReEncodeRec *p_reencode;
 
       for (p_reencode=topReEncodeInfo; p_reencode != NULL;
@@ -369,6 +377,7 @@ void ResetPSInfo()
    psUseRightText = FALSE;
    psUseMiniLines = FALSE;
    psUseReencode = FALSE;
+   psUseReencodeFF8 = FALSE;
    psUseShortHand = FALSE;
 #ifdef _PS_USE_EUC_FONT
    psUseEucFont = FALSE;
@@ -808,6 +817,18 @@ static char *psReencodeMacro[] = {
   NULL
 };
 
+#ifdef _PRTGIF_USE_ISO_LATIN_1_ALWAYS
+static char *psReEncodeFF8Macro[] = {
+  "/TGFF8",
+  " { FontDirectory 1 index known",
+  "   { pop findfont exch pop }",
+  "   { 1 index 4 1 roll tgifReEncodeSmall findfont } ifelse",
+  " } bind def",
+  "",
+  NULL
+};
+#endif /* _PRTGIF_USE_ISO_LATIN_1_ALWAYS */
+
 static char *psShortHandMacro[] = {
   "/bd { bind def } bind def",
   "",
@@ -992,6 +1013,14 @@ void DumpPSMacro(FP)
          fprintf(FP, "%s\n", psMiniLinesMacro[i]);
       }
    }
+#ifdef _PRTGIF_USE_ISO_LATIN_1_ALWAYS
+   if (PRTGIF && psUseReencodeFF8) {
+      for (i=0; psReEncodeFF8Macro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psReEncodeFF8Macro[i]);
+      }
+      DumpReEncodeVector(FP, "latin1", ".vec", NULL);
+   }
+#endif /* _PRTGIF_USE_ISO_LATIN_1_ALWAYS */
    if (psUseReencode) {
       for (i=0; psReencodeMacro[i] != NULL; i++) {
          fprintf(FP, "%s\n", psReencodeMacro[i]);
