@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/raster.c,v 4.13 2000/05/10 17:34:20 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/raster.c,v 4.15 2000/07/22 20:17:42 william Exp $
  */
 
 #define _INCLUDE_FROM_RASTER_C_
@@ -226,6 +226,7 @@
 #include "xbm/tiffepsi.xbm"
 #include "xbm/png.xbm"
 #include "xbm/jpeg.xbm"
+#include "xbm/netlist.xbm"
 
 #include "xbm/file.xbm"
 #include "xbm/landscap.xbm"
@@ -265,6 +266,7 @@
 #include "xbm/rot_180.xbm"
 #include "xbm/rot_270.xbm"
 
+#include "xbm/wire.xbm"
 #include "xbm/run.xbm"
 #include "xbm/check.xbm"
 #include "xbm/radio.xbm"
@@ -329,6 +331,7 @@ Pixmap shortLineTypePixmap[MAXLINETYPES];
 Pixmap shortLineStylePixmap[MAXLINESTYLES];
 Pixmap shortDashPixmap[MAXDASHES];
 
+Pixmap wireBitmap=None;
 Pixmap runBitmap=None;
 Pixmap checkBitmap=None;
 Pixmap radioBitmap=None;
@@ -944,6 +947,8 @@ void InitPattern()
          mainWindow, (char *)png_bits, png_width, png_height);
    whereToPrintPixmap[JPEG_FILE] = XCreateBitmapFromData(mainDisplay,
          mainWindow, (char *)jpeg_bits, jpeg_width, jpeg_height);
+   whereToPrintPixmap[NETLIST_FILE] = XCreateBitmapFromData(mainDisplay,
+         mainWindow, (char *)netlist_bits, netlist_width, netlist_height);
 
    shortLineTypePixmap[0] = XCreateBitmapFromData(mainDisplay, mainWindow,
          (char *)lt0s_bits, lt0s_width, lt0s_height);
@@ -1015,6 +1020,9 @@ void InitPattern()
    choicePixmap[ROTATEMODE] = XCreateBitmapFromData(mainDisplay, mainWindow,
          (char *)rot_mode_bits, rot_mode_width, rot_mode_height);
 
+   wireBitmap = XCreateBitmapFromData(mainDisplay, mainWindow,
+         (char *)wire_bits, wire_width, wire_height);
+
    runBitmap = XCreateBitmapFromData(mainDisplay, mainWindow,
          (char *)run_bits, run_width, run_height);
 
@@ -1168,6 +1176,7 @@ void CleanUpRasters()
    for (i = 0; i < MAXDASHES; i++) {
       XFreePixmap(mainDisplay, shortDashPixmap[i]);
    }
+   XFreePixmap(mainDisplay, wireBitmap);
    XFreePixmap(mainDisplay, runBitmap);
    XFreePixmap(mainDisplay, checkBitmap);
    XFreePixmap(mainDisplay, radioBitmap);
