@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/remote.c,v 4.34 2000/08/07 20:05:25 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/remote.c,v 4.35 2000/10/25 19:04:27 william Exp $
  */
 
 #define _INCLUDE_FROM_REMOTE_C_
@@ -1399,7 +1399,7 @@ void FillMimeTypesInfo()
    if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"MimeTypesFile")) != NULL) {
       strcpy(fname, c_ptr);
    } else {
-      sprintf(fname, "%s/.mime.types", homeDir);
+      sprintf(fname, "%s%c.mime.types", homeDir, DIR_SEP);
    }
    if ((fp=fopen(fname, "r")) == NULL) return;
    while ((buf=UtilGetALine(fp)) != NULL) {
@@ -1442,7 +1442,7 @@ void FillMailCapInfo()
    if ((c_ptr=getenv("MAILCAPS")) != NULL) {
       spec = UtilStrDup(c_ptr);
    } else {
-      sprintf(gszMsgBox, "%s/.mailcap", homeDir);
+      sprintf(gszMsgBox, "%s%c.mailcap", homeDir, DIR_SEP);
       spec = UtilStrDup(gszMsgBox);
    }
    if (spec == NULL) FailAllocMessage();
