@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/ruler.c,v 4.10 2000/01/15 22:24:09 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/ruler.c,v 4.17 2000/05/01 18:15:55 william Exp $
  */
 
 #define _INCLUDE_FROM_RULER_C_
@@ -47,6 +47,7 @@
 #include "setup.e"
 #include "strtbl.e"
 #include "util.e"
+#include "xpixmap.e"
 
 #define INCH_H (RULER_W-2)
 
@@ -58,8 +59,11 @@
 #define MIN_CM_H 4
 
 int showMeasurement=FALSE;
+int showMeasurementInTooltip=TRUE;
 int showCrossHair=FALSE;
 
+static int measureTooltipVerbose=FALSE;
+
 static float gfPixelsPerUnit=(float)1.0;
 static float gfNumUnits=(float)1.0;
 static float gfNumFracUnits=(float)1.0;
@@ -475,8 +479,10 @@ void RedrawVRulerWindow()
       PixelToMeasurementUnit(x_buf, ABS_X(oldXOff));
       PixelToMeasurementUnit(y_buf, ABS_Y(oldYOff));
       sprintf(buf, "[%s,%s]", x_buf, y_buf);
-      XDrawString(mainDisplay, hRuleWindow, revDefaultGC, 8, 2+defaultFontAsc,
-            buf, strlen(buf));
+      if (!showMeasurementInTooltip) {
+         XDrawString(mainDisplay, hRuleWindow, revDefaultGC,
+               8, 2+defaultFontAsc, buf, strlen(buf));
+      }
       hRulerJustRedrawn = TRUE;
    }
    oldYOff = (-1);
@@ -509,8 +515,10 @@ void MarkRulers(XOff, YOff)
          PixelToMeasurementUnit(y_buf, ABS_Y(oldYOff));
          sprintf(buf, "[%s,%s]", x_buf, y_buf);
       }
-      XDrawString(mainDisplay, hRuleWindow, revDefaultGC, 8, 2+defaultFontAsc,
-            buf, strlen(buf));
+      if (!showMeasurementInTooltip) {
+         XDrawString(mainDisplay, hRuleWindow, revDefaultGC,
+               8, 2+defaultFontAsc, buf, strlen(buf));
+      }
    }
    DrawHRuleTick(XOff-1);
    DrawVRuleTick(YOff-1);
@@ -525,8 +533,23 @@ void MarkRulers(XOff, YOff)
       PixelToMeasurementUnit(x_buf, ABS_X(XOff));
       PixelToMeasurementUnit(y_buf, ABS_Y(YOff));
       sprintf(buf, "[%s,%s]", x_buf, y_buf);
-      XDrawString(mainDisplay, hRuleWindow, revDefaultGC, 8, 2+defaultFontAsc,
-            buf, strlen(buf));
+      if (showMeasurementInTooltip) {
+         SetMeasureTooltip(buf);
+      } else {
+         XDrawString(mainDisplay, hRuleWindow, revDefaultGC,
+               8, 2+defaultFontAsc, buf, strlen(buf));
+      }
+   } else if (showMeasurement && showMeasurementInTooltip &&
+         measureTooltipVerbose) {
+      switch (curChoice) {
+      case NOTHING:
+      case DRAWPOLY:
+      case DRAWPOLYGON:
+      case VERTEXMODE:
+         DoIntervalMeasureTooltip(frozenXOff, frozenYOff, XOff, YOff,
+               (frozenXOff+XOff)>>1, (frozenYOff+YOff)>>1);
+         break;
+      }
    }
    oldXOff = XOff;
    oldYOff = YOff;
@@ -610,6 +633,9 @@ void RulersEventHandler(input)
 
 void FreezeMarkRulerText()
 {
+   if (!freezeMarkRulerText) {
+      EndMeasureTooltip(FALSE);
+   }
    freezeMarkRulerText = TRUE;
    frozenXOff = oldXOff;
    frozenYOff = oldYOff;
@@ -664,6 +690,9 @@ void DoIntervalRulers()
       XDrawLine(mainDisplay, drawWindow, revDefaultGC, 0, oldRbY,
             ZOOMED_SIZE(drawWinW), oldRbY);
    }
+   if (showMeasurement && showMeasurementInTooltip && measureTooltipVerbose) {
+      DoIntervalMeasureTooltip(oldLtX, oldLtY, oldRbX, oldRbY, oldMdX, oldMdY);
+   }
 }
 
 void BeginIntervalRulers(ltx, lty, rbx, rby)
@@ -722,7 +751,13 @@ void StartShowMeasureCursor(XOff, YOff, Str, ExtraSpace)
    if (!showMeasurement) return;
    MarkRulers(XOff, YOff);
    FreezeMarkRulerText();
-   if (Str != NULL && *Str != '\0') {
+   if (Str == NULL || *Str == '\0') return;
+
+   if (showMeasurementInTooltip) {
+      if (!measureTooltipVerbose) {
+         StartMeasureTooltip(Str);
+      }
+   } else {
       int x=(ExtraSpace ? XOff+18 : XOff+4), y=YOff+defaultFontAsc;
       char *c_ptr=strchr(Str, '\n'), *line=Str;
 
@@ -747,7 +782,22 @@ void ShowMeasureCursor(XOff, YOff, Str, ExtraSpace)
    char *Str;
 {
    if (!showMeasurement) return;
-   if (Str != NULL && *Str != '\0') {
+   if (Str == NULL || *Str == '\0') return;
+
+   if (showMeasurementInTooltip) {
+      if (measureTooltipVerbose) {
+         switch (curChoice) {
+         case DRAWARC:
+         case DRAWEDGEARC:
+         case FREEHAND:
+         case ROTATEMODE:
+            SetMeasureTooltip(Str);
+            break;
+         }
+      } else {
+         SetMeasureTooltip(Str);
+      }
+   } else {
       int x=(ExtraSpace ? XOff+18 : XOff+4), y=YOff+defaultFontAsc;
       char *c_ptr=strchr(Str, '\n'), *line=Str;
 
@@ -772,7 +822,13 @@ void EndShowMeasureCursor(XOff, YOff, Str, ExtraSpace)
    char *Str;
 {
    if (!showMeasurement) return;
-   if (Str != NULL && *Str != '\0') {
+   if (Str == NULL || *Str == '\0') return;
+
+   if (showMeasurementInTooltip) {
+      if (!measureTooltipVerbose) {
+         StartMeasureTooltip(Str);
+      }
+   } else {
       int x=(ExtraSpace ? XOff+18 : XOff+4), y=YOff+defaultFontAsc;
       char *c_ptr=strchr(Str, '\n'), *line=Str;
 
@@ -802,6 +858,357 @@ void ToggleShowMeasurement()
    } else {
       Msg(TgLoadString(STID_SHOW_MEASUREMENT_DISABLED));
    }
-   UpdatePinnedMenu(MENU_LAYOUT);
 }
 
+void ToggleShowMeasurementInTooltip()
+{
+   showMeasurementInTooltip = !showMeasurementInTooltip;
+   RedrawRulers();
+   if (showMeasurementInTooltip) {
+      Msg(TgLoadString(STID_SHOW_MEASUREMENT_IN_TT_ENAB));
+   } else {
+      Msg(TgLoadString(STID_SHOW_MEASUREMENT_IN_TT_DISB));
+   }
+   EndMeasureTooltip(TRUE);
+}
+
+/* ------------------- Measure Tooltip Related Routines ------------------- */
+
+/*
+ * Some code in the rest of this file is based on the contributed code
+ *       by Raphael Dechenaux <raph_d@club-internet.fr>.
+ */
+
+typedef struct tagMeasureTooltipInfo {
+   int ltx, lty, w, h; /* this is for the window */
+   int bbox_w, bbox_h; /* this is for the text */
+   int x_padding, y_padding; /* padding around the text */
+   int mapped;
+   Window win;
+   GC gc;
+   /* configuration parameters -- how to display */
+   int x_follow_mouse, y_follow_mouse, position_x, position_y;
+} MeasureTooltipInfo;
+
+static MeasureTooltipInfo gmti;
+
+static Pixmap tooltipBgPixmap=None;
+
+static
+void CalcMeasureTooltipPosition()
+{
+   if (gmti.x_follow_mouse || gmti.y_follow_mouse) {
+      Window root_win=None, child_win=None;
+      unsigned int status;
+      int root_x=0, root_y=0;
+
+      XQueryPointer(mainDisplay, rootWindow, &root_win, &child_win,
+            &root_x, &root_y, &gmti.ltx, &gmti.lty, &status);
+      if (gmti.x_follow_mouse) gmti.ltx += 16;
+      if (gmti.y_follow_mouse) gmti.lty += 16;
+      if (gmti.x_follow_mouse && gmti.y_follow_mouse) {
+         return;
+      }
+   }
+   if (!gmti.x_follow_mouse) {
+      int dpy_w=DisplayWidth(mainDisplay,mainScreen);
+
+      switch (gmti.position_x) {
+      case ALIGN_L: gmti.ltx = 0; break;
+      case ALIGN_C: gmti.ltx = ((dpy_w-gmti.bbox_w-gmti.x_padding)>>1); break;
+      case ALIGN_R: gmti.ltx = (dpy_w-gmti.bbox_w-(gmti.x_padding<<1)); break;
+      default: gmti.ltx = 0; break;
+      }
+   }
+   if (!gmti.y_follow_mouse) {
+      int dpy_h=DisplayHeight(mainDisplay,mainScreen);
+
+      switch (gmti.position_y) {
+      case ALIGN_T: gmti.lty = 0; break;
+      case ALIGN_M: gmti.lty = ((dpy_h-gmti.bbox_h-gmti.y_padding)>>1); break;
+      case ALIGN_B: gmti.lty = (dpy_h-gmti.bbox_h-(gmti.y_padding<<1)); break;
+      default: gmti.lty = 0; break;
+      }
+   }
+}
+
+static
+void SetMeasureTooltipBBox(msg)
+   char *msg;
+{
+   int bbox_w=0, bbox_h=0;
+   char *psz=NULL, *line=NULL;
+
+   bbox_h = ((msgFontSet==NULL && msgFontPtr==NULL) ? defaultFontHeight :
+         msgFontHeight);
+   line = msg;
+   psz = strchr(line, '\n');
+   while (line != NULL) {
+      char saved_ch='\0';
+      int len=0, w=0;
+
+      if (psz != NULL) {
+         saved_ch = (*psz);
+         *psz = '\0';
+      }
+      len = strlen(line);
+      if (msgFontSet == NULL && msgFontPtr == NULL) {
+         w = (defaultFontWidth*len);
+      } else {
+         w = MsgTextWidth(msgFontPtr, line, len);
+      }
+      if (w > bbox_w) bbox_w = w;
+
+      if (psz == NULL) {
+         break;
+      }
+      *psz = saved_ch;
+      line = &psz[1];
+      psz = strchr(line, '\n');
+
+      bbox_h += ((msgFontSet==NULL && msgFontPtr==NULL) ? defaultFontHeight :
+            msgFontHeight);
+   }
+   gmti.bbox_w = bbox_w;
+   gmti.bbox_h = bbox_h;
+   gmti.w = bbox_w+((gmti.x_padding)<<1);
+   gmti.h = bbox_h+((gmti.y_padding)<<1);
+
+   CalcMeasureTooltipPosition();
+}
+
+void DoIntervalMeasureTooltip(ltx, lty, rbx, rby, mdx, mdy)
+   int ltx, lty, rbx, rby, mdx, mdy;
+{
+   char w_buf[80], h_buf[80], buf[256];
+   char lx_buf[80], cx_buf[80], rx_buf[80];
+   char ty_buf[80], my_buf[80], by_buf[80];
+
+   PixelToMeasurementUnit(w_buf, ABS_SIZE(abs(rbx-ltx)));
+   PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(rby-lty)));
+   PixelToMeasurementUnit(lx_buf, ABS_X(ltx < rbx ? ltx : rbx));
+   PixelToMeasurementUnit(cx_buf, ABS_X(mdx));
+   PixelToMeasurementUnit(rx_buf, ABS_X(ltx > rbx ? ltx : rbx));
+   PixelToMeasurementUnit(ty_buf, ABS_Y(lty < rby ? lty : rby));
+   PixelToMeasurementUnit(my_buf, ABS_Y(mdy));
+   PixelToMeasurementUnit(by_buf, ABS_Y(lty > rby ? lty : rby));
+   sprintf(buf, "lx=%s cx=%s rx=%s\nty=%s my=%s by=%s\nw=%s h=%s",
+         lx_buf, cx_buf, rx_buf, ty_buf, my_buf, by_buf, w_buf, h_buf);
+   SetMeasureTooltip(buf);
+}
+
+void StartMeasureTooltip(msg)
+   char *msg;
+{
+   SetMeasureTooltip(msg);
+}
+
+void SetMeasureTooltip(msg)
+   char *msg;
+{
+   int x=0, y=0;
+   char *psz=NULL, *line=NULL;
+
+   if (!showMeasurementInTooltip) return;
+
+   if (!gmti.mapped) {
+      XMapWindow(mainDisplay, gmti.win);
+      gmti.mapped = TRUE;
+   }
+   XRaiseWindow(mainDisplay, gmti.win);
+
+   SetMeasureTooltipBBox(msg);
+   XMoveResizeWindow(mainDisplay, gmti.win, gmti.ltx, gmti.lty, gmti.w, gmti.h);
+
+   XClearWindow(mainDisplay, gmti.win);
+
+   x = gmti.x_padding;
+   y = gmti.y_padding + ((msgFontSet==NULL && msgFontPtr==NULL) ?
+         defaultFontAsc : msgFontAsc);;
+   line = msg;
+   psz = strchr(line, '\n');
+   while (line != NULL) {
+      char saved_ch='\0';
+
+      if (psz != NULL) {
+         saved_ch = (*psz);
+         *psz = '\0';
+      }
+      DrawMsgString(mainDisplay, gmti.win, gmti.gc, x, y, line, strlen(line));
+      if (psz == NULL) {
+         break;
+      }
+      *psz = saved_ch;
+      line = &psz[1];
+      psz = strchr(line, '\n');
+
+      y += ((msgFontSet==NULL && msgFontPtr==NULL) ? defaultFontHeight :
+            msgFontHeight);
+   }
+}
+
+void EndMeasureTooltip(force)
+   int force;
+{
+   if (force || (showMeasurement && showMeasurementInTooltip)) {
+      if (gmti.mapped) {
+         XUnmapWindow(mainDisplay, gmti.win);
+         gmti.mapped = FALSE;
+      }
+   }
+}
+
+void CleanUpMeasureTooltip()
+{
+   if (tooltipBgPixmap != None) {
+      XFreePixmap(mainDisplay, tooltipBgPixmap);
+      tooltipBgPixmap = None;
+   }
+   if (gmti.gc != NULL) {
+      XFreeGC(mainDisplay, gmti.gc);
+   }
+   if (gmti.win != None) {
+      XDestroyWindow(mainDisplay, gmti.win);
+   }
+   memset(&gmti, 0, sizeof(MeasureTooltipInfo));
+   showMeasurementInTooltip = FALSE;
+   gmti.win = None;
+}
+
+static
+int CreateMeasureTooltipWindow()
+{
+   int bg_pixel=(threeDLook ? myLtGryPixel : myBgPixel);
+   XGCValues values;
+   XWMHints wmhints;
+   XSizeHints sizehints;
+   XSetWindowAttributes win_attrs;
+
+   if ((gmti.win=XCreateSimpleWindow(mainDisplay, rootWindow, 0, 0, 10, 10,
+         1, myBorderPixel, bg_pixel)) == 0) {
+      return FailToCreateWindowMessage("CreateMeasureTooltipWindow()", NULL,
+            FALSE);
+   }
+   win_attrs.save_under = True;
+   win_attrs.override_redirect = True;
+   win_attrs.colormap = mainColormap;
+   XChangeWindowAttributes(mainDisplay, gmti.win,
+         CWSaveUnder | CWOverrideRedirect | CWColormap, &win_attrs);
+
+   wmhints.flags = InputHint | StateHint;
+   wmhints.input = True;
+   wmhints.initial_state = NormalState;
+   XSetWMHints(mainDisplay, gmti.win, &wmhints);
+
+   sizehints.flags = PPosition | PSize | USPosition | PMinSize | PMaxSize;
+   sizehints.x = 0;
+   sizehints.y = 0;
+   sizehints.width = sizehints.min_width = sizehints.max_width = 10;
+   sizehints.height = sizehints.min_height = sizehints.max_height = 10;
+#ifdef NOTR4MODE
+   XSetNormalHints(mainDisplay, gmti.win, &sizehints);
+#else
+   XSetWMNormalHints(mainDisplay, gmti.win, &sizehints);
+#endif
+
+   XSetTransientForHint(mainDisplay, gmti.win, mainWindow);
+
+   values.foreground = myFgPixel;
+   values.background = bg_pixel;
+   values.font = (msgFontPtr==NULL ? defaultFontPtr->fid : msgFontPtr->fid);
+   gmti.gc = XCreateGC(mainDisplay, gmti.win,
+         GCForeground | GCBackground | GCFont, &values);
+   return TRUE;
+}
+
+static
+void CreateLightYellowBackgroundPixmapForTooltip()
+{
+   XImage *image=NULL;
+
+   tooltipBgPixmap = XCreatePixmap(mainDisplay, mainWindow, 2, 2, mainDepth);
+   XSetForeground(mainDisplay, xpmGC, myYellowPixel);
+   XFillRectangle(mainDisplay, tooltipBgPixmap, xpmGC, 0, 0, 2, 2);
+   XSetForeground(mainDisplay, xpmGC, myFgPixel);
+   image = XGetImage(mainDisplay, tooltipBgPixmap, 0, 0, 2, 2, AllPlanes,
+         ZPixmap);
+   XPutPixel(image, 0, 0, myWhitePixel);
+   XPutPixel(image, 1, 1, myWhitePixel);
+   XPutImage(mainDisplay, tooltipBgPixmap, xpmGC, image, 0, 0, 0, 0, 2, 2);
+   XDestroyImage(image);
+
+   XSetWindowBackgroundPixmap(mainDisplay, gmti.win, tooltipBgPixmap);
+}
+
+int InitMeasureTooltip()
+{
+   char *c_ptr=NULL;
+
+   tooltipBgPixmap = None;
+
+   memset(&gmti, 0, sizeof(MeasureTooltipInfo));
+   gmti.win = None;
+   gmti.mapped = FALSE;
+   gmti.x_padding = 4;
+   gmti.y_padding = 2;
+
+   showMeasurementInTooltip = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "UseMeasureTooltip")) != NULL &&
+         UtilStrICmp(c_ptr, "true") == 0) {
+      showMeasurementInTooltip = TRUE;
+   }
+   measureTooltipVerbose = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "MeasureTooltipVerbose")) != NULL &&
+         UtilStrICmp(c_ptr, "true") == 0) {
+      measureTooltipVerbose = TRUE;
+   }
+   gmti.x_follow_mouse = gmti.y_follow_mouse = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "MeasureTooltipXFollowMouse")) != NULL &&
+         UtilStrICmp(c_ptr, "true") == 0) {
+      gmti.x_follow_mouse = TRUE;
+   }
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "MeasureTooltipYFollowMouse")) != NULL &&
+         UtilStrICmp(c_ptr, "true") == 0) {
+      gmti.y_follow_mouse = TRUE;
+   }
+   gmti.position_y = ALIGN_T;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "MeasureTooltipVerticalPosition")) != NULL) {
+      if (UtilStrICmp(c_ptr, "top") == 0) {
+         gmti.position_y = ALIGN_T;
+      } else if (UtilStrICmp(c_ptr, "middle") == 0) {
+         gmti.position_y = ALIGN_M;
+      } else if (UtilStrICmp(c_ptr, "bottom") == 0) {
+         gmti.position_y = ALIGN_B;
+      } else {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
+               TOOL_NAME, "MeasureTooltipVerticalPosition", c_ptr, "Top");
+         fprintf(stderr, "\n");
+      }
+   }
+   gmti.position_x = ALIGN_L;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "MeasureTooltipHorizontalPosition")) != NULL) {
+      if (UtilStrICmp(c_ptr, "left") == 0) {
+         gmti.position_x = ALIGN_L;
+      } else if (UtilStrICmp(c_ptr, "center") == 0) {
+         gmti.position_x = ALIGN_C;
+      } else if (UtilStrICmp(c_ptr, "right") == 0) {
+         gmti.position_x = ALIGN_R;
+      } else {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
+               TOOL_NAME, "MeasureTooltipHorizontalPosition", c_ptr, "Left");
+         fprintf(stderr, "\n");
+      }
+   }
+   if (!CreateMeasureTooltipWindow()) {
+      gmti.win = None;
+   } else if (threeDLook) {
+      CreateLightYellowBackgroundPixmapForTooltip();
+   }
+   return TRUE;
+}
