@@ -28,13 +28,14 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/select.c,v 4.27 2000/04/15 19:19:08 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/select.c,v 4.37 2000/08/03 18:05:18 william Exp $
  */
 
 #define _INCLUDE_FROM_SELECT_C_
 
 #include "tgifdefs.h"
 
+#include "attr.e"
 #include "auxtext.e"
 #include "button.e"
 #include "choice.e"
@@ -69,6 +70,7 @@
 #include "scroll.e"
 #include "select.e"
 #include "setup.e"
+#include "special.e"
 #include "stk.e"
 #include "stretch.e"
 #include "strtbl.e"
@@ -156,8 +158,8 @@ void CalcVertexBBox(LtX, LtY, RbX, RbY)
    }
 }
 
-void UnSelNonVertexObjs(highlight)
-   int highlight;
+void UnSelNonVertexObjs(highlight, no_locked_obj_only)
+   int highlight, no_locked_obj_only;
 {
    register struct ObjRec *obj_ptr;
    register struct SelRec *sel_ptr, *prev_sel;
@@ -167,10 +169,9 @@ void UnSelNonVertexObjs(highlight)
       obj_ptr = sel_ptr->obj;
 
       if ((obj_ptr->type==OBJ_POLY || obj_ptr->type==OBJ_POLYGON) &&
-            !obj_ptr->locked) {
+            (!no_locked_obj_only || !obj_ptr->locked)) {
          continue;
       }
-
       if (highlight) HighLightAnObj(obj_ptr);
 
       if (sel_ptr->prev == NULL) {
@@ -330,7 +331,6 @@ void RemoveAllSel()
    numObjSelected = 0;
 }
 
-static
 struct AttrRec *FindObjAttrWithName(ObjPtr, AttrName)
    struct ObjRec *ObjPtr;
    char *AttrName;
@@ -583,25 +583,63 @@ struct ObjRec *FindAnObj(XOff, YOff, OwnerObj, ConnectObj, ReturnedObjName)
    if (ReturnedObjName != NULL) {
       char *c_ptr;
 
-      strcpy(ReturnedObjName, "NodeName: ");
-      c_ptr = &ReturnedObjName[10];
+      *ReturnedObjName = '\0';
+      c_ptr = ReturnedObjName;
       if (returned_obj != NULL) {
          struct ObjRec *prev_obj=NULL;
+         int see_sym_or_icon=FALSE;
 
          for (obj_ptr=actual_obj; obj_ptr != NULL; obj_ptr=obj_ptr->tmp_child) {
-            if ((attr_ptr=FindObjAttrWithName(obj_ptr, "name=")) != NULL) {
-               *c_ptr++ = '!';
-               strcpy(c_ptr, attr_ptr->attr_value.s);
-               c_ptr = &c_ptr[strlen(c_ptr)];
+            /* do not translate -- program constants */
+            if (connectingPortsFromInternalCommand) {
+               /* this is pretty much left for compatibility only */
+               if ((attr_ptr=FindObjAttrWithName(obj_ptr, "name=")) != NULL) {
+                  *c_ptr++ = '!';
+                  strcpy(c_ptr, attr_ptr->attr_value.s);
+                  c_ptr = &c_ptr[strlen(c_ptr)];
+               } else {
+                  if (prev_obj != NULL) prev_obj->tmp_child = NULL;
+                  break;
+               }
             } else {
-               if (prev_obj != NULL) prev_obj->tmp_child = NULL;
-               break;
+               int just_seen_sym_or_icon=FALSE;
+
+               if (!see_sym_or_icon) {
+                  if (obj_ptr->type == OBJ_SYM || obj_ptr->type == OBJ_ICON) {
+                     see_sym_or_icon = TRUE;
+                     just_seen_sym_or_icon = TRUE;
+                  }
+               }
+               if (see_sym_or_icon) {
+                  if ((attr_ptr=FindObjAttrWithName(obj_ptr, "name=")) ==
+                        NULL) {
+                     /* broken name is invalid */
+                     c_ptr = ReturnedObjName;
+                     if (prev_obj != NULL) prev_obj->tmp_child = NULL;
+                     break;
+                  } else if (BlankStr(attr_ptr->attr_value.s)) {
+                     /* do not translate -- program constants */
+                     *c_ptr++ = '!';
+                     strcpy(c_ptr, "(unknown)");
+                     c_ptr = &c_ptr[strlen(c_ptr)];
+                  } else {
+                     *c_ptr++ = '!';
+                     strcpy(c_ptr, attr_ptr->attr_value.s);
+                     c_ptr = &c_ptr[strlen(c_ptr)];
+                  }
+                  if (!just_seen_sym_or_icon && (obj_ptr->type == OBJ_SYM ||
+                        obj_ptr->type == OBJ_ICON)) {
+                     obj_ptr->tmp_child = NULL;
+                     prev_obj = obj_ptr;
+                     break;
+                  }
+               }
             }
             prev_obj = obj_ptr;
          }
          if (ConnectObj != NULL) *ConnectObj = prev_obj;
       }
-      if (c_ptr == &ReturnedObjName[10]) {
+      if (c_ptr == ReturnedObjName) {
          *ReturnedObjName = '\0';
       } else {
          *c_ptr = '\0';
@@ -1388,7 +1426,7 @@ void ContinueSel(XOff, YOff, ShiftKeyDown)
                   &top_sel_ptr, &bot_sel_ptr) != NULL) {
                topSel = top_sel_ptr;
                botSel = bot_sel_ptr;
-               UnSelNonVertexObjs(FALSE); /* do not highlight */
+               UnSelNonVertexObjs(FALSE, TRUE); /* do not highlight */
                UpdSelBBox();
                HighLightForward();
             }
@@ -2488,8 +2526,8 @@ void GroupSingleObj(ForcePromoteAttrs)
    numObjSelected = 1;
 }
 
-void GroupSelObj(highlight, record_cmd)
-   int highlight, record_cmd;
+void GroupSelObj(highlight, record_cmd, redraw)
+   int highlight, record_cmd, redraw;
 {
    if (topSel == NULL) {
       MsgBox(TgLoadCachedString(CSTID_NO_OBJ_SELECTED), TOOL_NAME, INFO_MB);
@@ -2521,8 +2559,10 @@ void GroupSelObj(highlight, record_cmd)
    UpdSelBBox();
 
    if (record_cmd) RecordCmd(CMD_MANY_TO_ONE, NULL, topSel, botSel, 1);
-   RedrawAnArea(botObj, selLtX-GRID_ABS_SIZE(1), selLtY-GRID_ABS_SIZE(1),
-         selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1));
+   if (redraw) {
+      RedrawAnArea(botObj, selLtX-GRID_ABS_SIZE(1), selLtY-GRID_ABS_SIZE(1),
+            selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1));
+   }
    if (highlight) HighLightForward();
    SetFileModified(TRUE);
    justDupped = FALSE;
