@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/setup.c,v 4.56 2000/08/04 13:30:28 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/setup.c,v 4.60 2000/10/30 15:09:00 william Exp $
  */
 
 #define _INCLUDE_FROM_SETUP_C_
@@ -1428,7 +1428,7 @@ void Setup()
    } else {
       strcpy(homeDir, c_ptr);
    }
-   sprintf(tgifDir, "%s/.%s", homeDir, TOOL_NAME);
+   sprintf(tgifDir, "%s%c.%s", homeDir, DIR_SEP, TOOL_NAME);
    memset(&stat_buf, 0, sizeof(struct stat));
    if (stat(tgifDir, &stat_buf) != 0) {
       /* directory does not exist */
@@ -1483,6 +1483,16 @@ void Setup()
       }
 #endif /* ~_TMP_DIR_IN_HOME_DIR */
    }
+   enableMouseWheel = TRUE;
+   if (threeDLook && (c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "EnableMouseWheel")) != NULL && UtilStrICmp(c_ptr, "false") == 0) {
+      enableMouseWheel = FALSE;
+   }
+   btn2PopupMainMenu = TRUE;
+   if (threeDLook && (c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "Btn2PopupMainMenu")) != NULL && UtilStrICmp(c_ptr, "false") == 0) {
+      btn2PopupMainMenu = FALSE;
+   }
    InitFonts();
 
    initialMenubarWindowH = defaultFontHeight;
