@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/shortcut.c,v 4.62 2000/05/20 22:31:36 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/shortcut.c,v 4.70 2000/08/04 13:39:13 william Exp $
  */
 
 #define _INCLUDE_FROM_SHORTCUT_C_
@@ -387,6 +387,11 @@ static struct ShortCutRec shortCutXlateTbl[] = {
    { '\0', 0, "ImportJPEGFile()", 0, CMDID_IMPORTJPEGFILE },
    { '\0', 0, "ToggleAutoEPSPreviewBitmap()", 0,
                CMDID_TOGGLEAUTOEPSPREVIEWBITMAP },
+   { '\0', 0, "CreateThumbnails()", 0, CMDID_CREATETHUMBNAILS },
+   { '\0', 0, "ConnectTwoPortsByAWire()", 0, CMDID_CONNECT2PORTSBYWIRE },
+   { '\0', 0, "RenameSignalNameForAPort()", 0, CMDID_RENAMESIGNAMEFORAPORT },
+   { '\0', 0, "ClearSignalNameForAPort()", 0, CMDID_CLEARSIGNAMEFORAPORT },
+   { '\0', 0, "ToggleShowWireSignalName()", 0, CMDID_TOGGLESHOWWIRESIGNALNAME },
    { '\0', 0, "", 0, 0 }
 };
 
