@@ -28,12 +28,13 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/special.c,v 4.27 2000/04/15 20:27:04 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/special.c,v 4.43 2000/08/17 23:06:32 william Exp $
  */
 
 #define _INCLUDE_FROM_SPECIAL_C_
 
 #include "tgifdefs.h"
+#include "cmdids.h"
 
 #include "animate.e"
 #include "attr.e"
@@ -52,6 +53,7 @@
 #include "grid.e"
 #include "group.e"
 #include "mainloop.e"
+#include "mainmenu.e"
 #include "mark.e"
 #include "menu.e"
 #include "menuinfo.e"
@@ -61,7 +63,10 @@
 #include "names.e"
 #include "obj.e"
 #include "page.e"
+#include "pattern.e"
+#include "poly.e"
 #include "raster.e"
+#include "rect.e"
 #include "remote.e"
 #include "ruler.e"
 #include "scroll.e"
@@ -69,13 +74,17 @@
 #include "setup.e"
 #include "special.e"
 #include "stk.e"
+#include "stretch.e"
 #include "strtbl.e"
 #include "text.e"
 #include "util.e"
 #include "version.e"
 
 int placingTopObj=FALSE;
+int connectingPortsByWire=FALSE;
+int connectingPortsFromInternalCommand=TRUE;
 
+static int showWireSignalName=TRUE;
 static int gnPinWarning=FALSE;
 
 static
@@ -573,6 +582,561 @@ void Instantiate()
  */
 }
 
+int ObjIsAPort(obj_ptr)
+   struct ObjRec *obj_ptr;
+{
+   struct AttrRec *attr_ptr=NULL;
+
+   attr_ptr = FindAttrWithName(obj_ptr, "type=", NULL);
+   if (attr_ptr != NULL && strcmp(attr_ptr->attr_value.s, "port") == 0) {
+      /*
+       * if this is not called from ConnectTwoPortsByAWire() but from
+       * an internal command, "type=port" is enough for a point
+       */
+      if (connectingPortsFromInternalCommand) return TRUE;
+
+      if (FindAttrWithName(obj_ptr, "signal_name=", NULL) != NULL &&
+            FindAttrWithName(obj_ptr, "name=", NULL) != NULL) {
+         struct ObjRec *owner_obj=obj_ptr->tmp_parent, *tmp_obj=NULL;
+
+         owner_obj = obj_ptr->tmp_parent;
+         if (owner_obj == NULL) {
+            /* obj_ptr is a top-level object */
+            return (obj_ptr->type == OBJ_ICON || obj_ptr->type == OBJ_SYM);
+         } else {
+            if (owner_obj->type == OBJ_ICON || owner_obj->type == OBJ_SYM) {
+               if (FindAttrWithName(owner_obj, "name=", NULL) == NULL) {
+                  return FALSE;
+               }
+            } else {
+               return FALSE;
+            }
+         }
+         for (tmp_obj=owner_obj->tmp_parent; tmp_obj != NULL;
+               tmp_obj=tmp_obj->tmp_parent) {
+            if (tmp_obj->type == OBJ_ICON || tmp_obj->type == OBJ_SYM) {
+               return FALSE;
+            }
+         }
+         return TRUE;
+      }
+   }
+   return FALSE;
+}
+
+static
+int DoConnectTwoPorts()
+{
+   int poly_created=FALSE;
+   XGCValues values;
+
+   MakeQuiescent();
+
+   ResetWiringNodeInfo();
+   connectingPortsByWire = TRUE;
+   connectingPortsFromInternalCommand = FALSE;
+   UseWireMenuItemInModeItem(TRUE);
+
+   values.line_width = 3;
+   XChangeGC(mainDisplay, revGrayGC, GCLineWidth, &values);
+   SetCurChoice(DRAWPOLY);
+   gstWiringInfo.num_ports_to_connect = 2;
+   drawPolyHighlightedNode = NULL;
+   SetStringStatus(TgLoadString(STID_DRAW_AN_EDGE_BETWEEN_PORTS));
+   polyDrawn = FALSE;
+   for (;;) {
+      XEvent input;
+
+      XNextEvent(mainDisplay, &input);
+      if (input.type == Expose || input.type == VisibilityNotify) {
+         ExposeEventHandler(&input, TRUE);
+      } else if (input.type == ConfigureNotify) {
+         Reconfigure(FALSE);
+      } else if (input.xany.window == drawWindow) {
+         polyDrawn = FALSE;
+         DrawingEventHandler(&input);
+         if (curChoice == DRAWPOLY) {
+            if (polyDrawn) {
+               break;
+            } else if (gstWiringInfo.num_ports_to_connect == (-1)) {
+               break;
+            }
+         } else {
+            polyDrawn = FALSE;
+            break;
+         }
+      }
+   }
+   gstWiringInfo.num_ports_to_connect = 0;
+   poly_created = polyDrawn;
+   values.line_width = 1;
+   XChangeGC(mainDisplay, revGrayGC, GCLineWidth, &values);
+   connectingPortsByWire = FALSE;
+   connectingPortsFromInternalCommand = TRUE;
+   SetCurChoice(NOTHING);
+   RedrawModeWindow();
+   UseWireMenuItemInModeItem(FALSE);
+   UpdatePinnedMenu(MENU_MODE);
+
+   return poly_created;
+}
+
+static
+void AddWireAttributes(poly_obj_ptr, signal_name, p_obbox, auto_center)
+   struct ObjRec *poly_obj_ptr;
+   char *signal_name;
+   struct BBRec *p_obbox;
+   int auto_center;
+{
+   struct ObjRec *text_obj_ptr=NULL;
+
+   PrepareToReplaceAnObj(poly_obj_ptr);
+
+   AddAttrByNameAndValue(poly_obj_ptr, "signal_name=", signal_name);
+   poly_obj_ptr->fattr->shown = TRUE;
+   poly_obj_ptr->fattr->nameshown = FALSE;
+   UpdAttr(poly_obj_ptr->fattr);
+   text_obj_ptr = poly_obj_ptr->fattr->obj;
+   if (auto_center) {
+      CenterObjInOBBox(text_obj_ptr, *p_obbox, NULL);
+      memcpy(p_obbox, &text_obj_ptr->obbox, sizeof(struct BBRec));
+      poly_obj_ptr->fattr->shown = FALSE;
+   } else {
+      MoveObj(text_obj_ptr, p_obbox->ltx-text_obj_ptr->obbox.ltx,
+            p_obbox->lty-text_obj_ptr->obbox.lty);
+   }
+   AddAttrByNameAndValue(poly_obj_ptr, "type=", "tgWire");
+   text_obj_ptr = poly_obj_ptr->fattr->obj;
+   MoveObj(text_obj_ptr, p_obbox->ltx-text_obj_ptr->obbox.ltx,
+         p_obbox->rby-text_obj_ptr->obbox.lty);
+
+   AdjObjBBox(poly_obj_ptr);
+
+   RecordReplaceAnObj(poly_obj_ptr);
+}
+
+typedef struct tagChangeConnectionViewInfo {
+   int change_to_connect_view;
+   struct ObjRec *conn_view_obj, *disconn_view_obj;
+   int conn_fill, conn_pen, disconn_fill, disconn_pen;
+} ChangeConnectionViewInfo;
+
+static
+int ChangeConnectionView(pravci)
+   ReplAttrValueCallbackInfo *pravci;
+{
+   ChangeConnectionViewInfo *pccvi=(
+         (ChangeConnectionViewInfo*)pravci->pv_userdata);
+
+   if (pccvi == NULL) return FALSE;
+
+   if (pccvi->change_to_connect_view) {
+      ChangeObjFill(pccvi->disconn_view_obj, NONEPAT);
+      ChangeObjPen(pccvi->disconn_view_obj, NONEPAT);
+      ChangeObjFill(pccvi->conn_view_obj, pccvi->conn_fill);
+      ChangeObjPen(pccvi->conn_view_obj, pccvi->conn_pen);
+   } else {
+      ChangeObjFill(pccvi->conn_view_obj, NONEPAT);
+      ChangeObjPen(pccvi->conn_view_obj, NONEPAT);
+      ChangeObjFill(pccvi->disconn_view_obj, pccvi->disconn_fill);
+      ChangeObjPen(pccvi->disconn_view_obj, pccvi->disconn_pen);
+   }
+   return TRUE;
+}
+
+static
+int HasConnAndDisConnViewObjSubObjs(obj_ptr, pccvi)
+   struct ObjRec *obj_ptr;
+   ChangeConnectionViewInfo *pccvi;
+{
+   struct ObjRec *subobj=NULL;
+
+   if (obj_ptr == NULL) return FALSE;
+   if (obj_ptr->type != OBJ_SYM && obj_ptr->type != OBJ_ICON &&
+         obj_ptr->type != OBJ_GROUP) {
+      return FALSE;
+   }
+   for (subobj=obj_ptr->detail.r->first; subobj != NULL; subobj=subobj->next) {
+      struct AttrRec *attr_ptr=FindAttrWithName(subobj, "view=", NULL);
+
+      if (attr_ptr != NULL) {
+         char *psz_fill=NULL, *psz_pen=NULL, buf[MAXSTRING];
+
+         UtilStrCpyN(buf, sizeof(buf), attr_ptr->attr_value.s);
+         psz_fill = strchr(buf, ',');
+         if (psz_fill != NULL) {
+            *psz_fill++ = '\0';
+            psz_pen = strchr(psz_fill, ',');
+            if (psz_pen != NULL) *psz_pen++ = '\0';
+         }
+         if (psz_fill != NULL && psz_pen != NULL) {
+            int fill=0, pen=0, ok=TRUE;
+
+            UtilTrimBlanks(buf);
+            UtilTrimBlanks(psz_fill);
+            UtilTrimBlanks(psz_pen);
+            if (sscanf(psz_fill, "%d", &fill) != 1) ok = FALSE;
+            if (sscanf(psz_pen, "%d", &pen) != 1) ok = FALSE;
+            if (ok) {
+               fill = fill % MAXPATTERNS;
+               pen = pen % MAXPATTERNS;
+               if (ok && UtilStrICmp(buf, "conn") == 0) {
+                  pccvi->conn_fill = fill;
+                  pccvi->conn_pen = pen;
+                  pccvi->conn_view_obj = subobj;
+               } else if (ok && UtilStrICmp(buf, "disconn") == 0) {
+                  pccvi->disconn_fill = fill;
+                  pccvi->disconn_pen = pen;
+                  pccvi->disconn_view_obj = subobj;
+               }
+            }
+         }
+      }
+   }
+   return (pccvi->conn_view_obj != NULL && pccvi->disconn_view_obj != NULL);
+}
+
+static
+int SetPortSignalName(attr_ptr, attr_owner_obj, new_signal_name)
+   struct AttrRec *attr_ptr;
+   struct ObjRec *attr_owner_obj;
+   char *new_signal_name;
+{
+   if (attr_ptr != NULL &&
+         strcmp(new_signal_name, attr_ptr->attr_value.s) != 0) {
+      char *old_signal_name=attr_ptr->attr_value.s;
+      ChangeConnectionViewInfo ccvi;
+
+      memset(&ccvi, 0, sizeof(ChangeConnectionViewInfo));
+      if ((*old_signal_name == '\0' || *new_signal_name == '\0') &&
+            HasConnAndDisConnViewObjSubObjs(attr_ptr->owner, &ccvi)) {
+         if (*old_signal_name == '\0') {
+            /* the port will become connected */
+            ccvi.change_to_connect_view = TRUE;
+         } else {
+            /* the port will become disconnected */
+            ccvi.change_to_connect_view = FALSE;
+         }
+         SetReplaceAttrValueCallback(
+               (ReplAttrValueCallback*)ChangeConnectionView, &ccvi);
+      }
+      ReplaceAttrFirstValue(attr_owner_obj, attr_ptr, new_signal_name);
+      SetReplaceAttrValueCallback(NULL, NULL);
+      RecursivelyAdjObjBBox(attr_ptr->owner, attr_ptr->owner,
+            attr_owner_obj);
+      return TRUE;
+   }
+   return FALSE;
+}
+
+static
+void DoNameWire()
+{
+   struct AttrRec *first_attr_ptr=NULL, *last_attr_ptr=NULL;
+   struct ObjRec *first_attr_owner_obj=NULL, *last_attr_owner_obj=NULL;
+   struct BBRec bbox;
+   char signal_name[MAXSTRING], *existing_signal_name=NULL;
+   int need_to_clear=FALSE, do_prompt=TRUE, need_to_redraw=FALSE;
+
+   memcpy(&bbox, &topObj->bbox, sizeof(struct BBRec));
+   first_attr_ptr = FindAttrWithName(gstWiringInfo.first_port_obj,
+         "signal_name=", &first_attr_owner_obj);
+   while (first_attr_owner_obj != NULL &&
+         first_attr_owner_obj->tmp_parent != NULL) {
+      first_attr_owner_obj = first_attr_owner_obj->tmp_parent;
+   }
+   last_attr_ptr = FindAttrWithName(gstWiringInfo.last_port_obj,
+         "signal_name=", &last_attr_owner_obj);
+   while (last_attr_owner_obj != NULL &&
+         last_attr_owner_obj->tmp_parent != NULL) {
+      last_attr_owner_obj = last_attr_owner_obj->tmp_parent;
+   }
+   if (first_attr_ptr == NULL && last_attr_ptr == NULL) return;
+   if (first_attr_ptr != NULL && last_attr_ptr != NULL) {
+      if (strcmp(first_attr_ptr->attr_value.s, last_attr_ptr->attr_value.s) !=
+            0) {
+         if (*first_attr_ptr->attr_value.s == '\0' &&
+               *last_attr_ptr->attr_value.s == '\0') {
+            do_prompt = TRUE;
+         } else if (*first_attr_ptr->attr_value.s == '\0') {
+            existing_signal_name = last_attr_ptr->attr_value.s;
+         } else if (*last_attr_ptr->attr_value.s == '\0') {
+            existing_signal_name = first_attr_ptr->attr_value.s;
+         } else {
+            sprintf(gszMsgBox, TgLoadString(STID_Q_CNFLCT_SIG_NAME_OKAY_TO_CLR),
+                  first_attr_ptr->attr_value.s, gstWiringInfo.first_port_name,
+                  last_attr_ptr->attr_value.s, gstWiringInfo.last_port_name);
+            switch (MsgBox(gszMsgBox, TOOL_NAME, YNC_MB)) {
+            case MB_ID_YES:
+               need_to_clear = TRUE;
+               do_prompt = TRUE;
+               break;
+            case MB_ID_NO:
+            case MB_ID_CANCEL:
+               return;
+            }
+         }
+      } else {
+         existing_signal_name = first_attr_ptr->attr_value.s;
+      }
+   } else if (first_attr_ptr == NULL) {
+      if (*last_attr_ptr->attr_value.s != '\0') {
+         existing_signal_name = last_attr_ptr->attr_value.s;
+      }
+   } else {
+      if (*first_attr_ptr->attr_value.s != '\0') {
+         existing_signal_name = first_attr_ptr->attr_value.s;
+      }
+   }
+   StartCompositeCmd();
+   if (need_to_clear) {
+      if (SetPortSignalName(first_attr_ptr, first_attr_owner_obj, "")) {
+         UnionRect(&bbox, &first_attr_owner_obj->bbox, &bbox);
+         need_to_redraw = TRUE;
+      }
+      if (SetPortSignalName(last_attr_ptr, last_attr_owner_obj, "")) {
+         UnionRect(&bbox, &last_attr_owner_obj->bbox, &bbox);
+         need_to_redraw = TRUE;
+      }
+   }
+   if (do_prompt) {
+      struct ObjRec *signal_name_obj=NULL;
+      struct BBRec obbox;
+      XEvent ev;
+
+      *signal_name = '\0';
+      if (existing_signal_name != NULL) {
+         UtilStrCpyN(signal_name, sizeof(signal_name), existing_signal_name);
+      }
+      Dialog(TgLoadString(STID_PLS_ENT_SIG_NAME), NULL, signal_name);
+      UtilTrimBlanks(signal_name);
+      if (*signal_name == '\0') {
+         if (need_to_clear) {
+            sprintf(gszMsgBox, TgLoadString(STID_SIGNAL_NAMES_CLEARED),
+                  gstWiringInfo.first_port_name, gstWiringInfo.last_port_name);
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         }
+         EndCompositeCmd();
+
+         if (need_to_redraw) {
+            SetFileModified(TRUE);
+            justDupped = FALSE;
+            RedrawAnArea(botObj,
+                  bbox.ltx-QUARTER_INCH-GRID_ABS_SIZE(1),
+                  bbox.lty-QUARTER_INCH-GRID_ABS_SIZE(1),
+                  bbox.rbx+QUARTER_INCH+GRID_ABS_SIZE(1),
+                  bbox.rby+QUARTER_INCH+GRID_ABS_SIZE(1));
+         }
+         return;
+      }
+      if (showWireSignalName) {
+         SaveStatusStrings();
+         sprintf(gszMsgBox, TgLoadString(STID_PLACING_NAMED_SIGNAL),
+               signal_name);
+         SetStringStatus(gszMsgBox);
+         PasteString(signal_name, FALSE, FALSE);
+         RestoreStatusStrings();
+
+         memcpy(&obbox, &topObj->obbox, sizeof(struct BBRec));
+         signal_name_obj = topObj;
+         UnlinkObj(signal_name_obj);
+         FreeObj(signal_name_obj);
+
+         XSync(mainDisplay, False);
+         if (XCheckMaskEvent(mainDisplay, ExposureMask, &ev)) {
+            ExposeEventHandler(&ev, TRUE);
+         }
+      } else {
+         memcpy(&obbox, &topObj->obbox, sizeof(struct BBRec));
+      }
+      AddWireAttributes(topObj, signal_name, &obbox, !showWireSignalName);
+      UnionRect(&bbox, &topObj->bbox, &bbox);
+      if (SetPortSignalName(first_attr_ptr, first_attr_owner_obj,
+            signal_name)) {
+         UnionRect(&bbox, &first_attr_owner_obj->bbox, &bbox);
+         need_to_redraw = TRUE;
+      }
+      if (SetPortSignalName(last_attr_ptr, last_attr_owner_obj, signal_name)) {
+         UnionRect(&bbox, &last_attr_owner_obj->bbox, &bbox);
+         need_to_redraw = TRUE;
+      }
+   }
+   EndCompositeCmd();
+
+   if (need_to_redraw) {
+      SetFileModified(TRUE);
+      justDupped = FALSE;
+      RedrawAnArea(botObj,
+            bbox.ltx-QUARTER_INCH-GRID_ABS_SIZE(1),
+            bbox.lty-QUARTER_INCH-GRID_ABS_SIZE(1),
+            bbox.rbx+QUARTER_INCH+GRID_ABS_SIZE(1),
+            bbox.rby+QUARTER_INCH+GRID_ABS_SIZE(1));
+   }
+}
+
+void ConnectTwoPortsByAWire()
+{
+   if (DoConnectTwoPorts()) {
+      DoNameWire();
+   }
+}
+
+#define DO_RENAME_SIGNAL_NAME 99
+#define DO_CLEAR_SIGNAL_NAME 999
+
+static
+void DoNameOrClearAPort(which)
+   int which;
+{
+   struct AttrRec *first_attr_ptr=NULL;
+   struct ObjRec *first_attr_owner_obj=NULL;
+   char signal_name[MAXSTRING];
+   ChangeConnectionViewInfo ccvi;
+
+   memset(&ccvi, 0, sizeof(ChangeConnectionViewInfo));
+   first_attr_ptr = FindAttrWithName(gstWiringInfo.first_port_obj,
+         "signal_name=", &first_attr_owner_obj);
+   while (first_attr_owner_obj != NULL &&
+         first_attr_owner_obj->tmp_parent != NULL) {
+      first_attr_owner_obj = first_attr_owner_obj->tmp_parent;
+   }
+   if (first_attr_ptr == NULL) {
+#ifdef _TGIF_DBG /* debug, do not translate */
+      TgAssert(FALSE,
+            "first_attr_ptr is NULL in DoNameOrClearAPort()", NULL);
+      return;
+#endif /* _TGIF_DBG */
+   }
+   *signal_name = '\0';
+   if (which == DO_CLEAR_SIGNAL_NAME && *first_attr_ptr->attr_value.s == '\0') {
+      /* already cleared */
+      return;
+   }
+   if (which == DO_RENAME_SIGNAL_NAME) {
+      UtilStrCpyN(signal_name, sizeof(signal_name),
+            first_attr_ptr->attr_value.s);
+      sprintf(gszMsgBox, TgLoadString(STID_PLS_ENT_SIG_NAME_FOR_PORT),
+            gstWiringInfo.first_port_name);
+      if (Dialog(gszMsgBox, NULL, signal_name) == INVALID) return;
+      UtilTrimBlanks(signal_name);
+   }
+   if (strcmp(signal_name, first_attr_ptr->attr_value.s) == 0) {
+      return;
+   }
+   if (HasConnAndDisConnViewObjSubObjs(first_attr_ptr->owner, &ccvi)) {
+      if (*signal_name != '\0') {
+         /* the port will become connected */
+         ccvi.change_to_connect_view = TRUE;
+      } else {
+         /* the port will become disconnected */
+         ccvi.change_to_connect_view = FALSE;
+      }
+      SetReplaceAttrValueCallback(
+            (ReplAttrValueCallback*)ChangeConnectionView, &ccvi);
+   }
+   ReplaceAttrFirstValue(first_attr_owner_obj, first_attr_ptr,
+         signal_name);
+   RecursivelyAdjObjBBox(first_attr_ptr->owner, first_attr_ptr->owner,
+         first_attr_owner_obj);
+
+   SetFileModified(TRUE);
+   justDupped = FALSE;
+   RedrawAnArea(botObj,
+         first_attr_owner_obj->bbox.ltx-QUARTER_INCH-GRID_ABS_SIZE(1),
+         first_attr_owner_obj->bbox.lty-QUARTER_INCH-GRID_ABS_SIZE(1),
+         first_attr_owner_obj->bbox.rbx+QUARTER_INCH+GRID_ABS_SIZE(1),
+         first_attr_owner_obj->bbox.rby+QUARTER_INCH+GRID_ABS_SIZE(1));
+}
+
+static
+void DoRenameOrClearSignalNameForAPort(which)
+   int which;
+{
+   int naming=TRUE;
+
+   ShowCurChoiceMouseStatus(DRAWPOLY, 0, FALSE);
+   SetHandCursor(drawWindow);
+   while (naming) {
+      XEvent input;
+
+      XNextEvent(mainDisplay, &input);
+      if (input.type == Expose || input.type == VisibilityNotify) {
+         ExposeEventHandler(&input, TRUE);
+      } else if (input.type == ConfigureNotify) {
+         Reconfigure(FALSE);
+      } else if (input.xany.window == drawWindow && input.type == ButtonPress) {
+         if (input.xbutton.button == Button1) {
+            if (drawPolyHighlightedNode != NULL) {
+               HandlePressForPortInDrawWindow(FALSE);
+               DoNameOrClearAPort(which);
+               drawPolyHighlightedNode = NULL;
+               SetWiringNodeInfo(NULL, NULL, NULL, TRUE);
+               SetHandCursor(drawWindow);
+               ShowCurChoiceMouseStatus(DRAWPOLY, 0, FALSE);
+            }
+         } else {
+            XUngrabPointer(mainDisplay, CurrentTime);
+            naming = FALSE;
+            HandlePressForPortInDrawWindow(TRUE);
+         }
+      } else if (input.xany.window == drawWindow &&
+            input.type == MotionNotify) {
+         XEvent ev;
+
+         HandleMotionForPortInDrawWindow(input.xmotion.x, input.xmotion.y);
+         while (XCheckMaskEvent(mainDisplay, PointerMotionMask, &ev)) ;
+      } else if (input.type == KeyPress) {
+         if (KeyPressEventIsEscape(&input.xkey)) {
+            XUngrabPointer(mainDisplay, CurrentTime);
+            naming = FALSE;
+            HandlePressForPortInDrawWindow(TRUE);
+         }
+      }
+   }
+   SetDefaultCursor(drawWindow);
+}
+
+static
+void RenameOrClearSignalNameForAPort(which)
+   int which;
+{
+   XGCValues values;
+
+   MakeQuiescent();
+
+   ResetWiringNodeInfo();
+   connectingPortsByWire = TRUE;
+   connectingPortsFromInternalCommand = FALSE;
+   gstWiringInfo.num_ports_to_connect = which;
+   drawPolyHighlightedNode = NULL;
+
+   values.line_width = 3;
+   XChangeGC(mainDisplay, revGrayGC, GCLineWidth, &values);
+
+   SaveStatusStrings();
+   DoRenameOrClearSignalNameForAPort(which);
+   RestoreStatusStrings();
+
+   values.line_width = 1;
+   XChangeGC(mainDisplay, revGrayGC, GCLineWidth, &values);
+
+   gstWiringInfo.num_ports_to_connect = 0;
+   connectingPortsByWire = FALSE;
+   connectingPortsFromInternalCommand = TRUE;
+   ResetWiringNodeInfo();
+   drawPolyHighlightedNode = NULL;
+}
+
+void RenameSignalNameForAPort()
+{
+   RenameOrClearSignalNameForAPort(DO_RENAME_SIGNAL_NAME);
+}
+
+void ClearSignalNameForAPort()
+{
+   RenameOrClearSignalNameForAPort(DO_CLEAR_SIGNAL_NAME);
+}
+
 void MakeSymbolic()
 {
    int ltx=selLtX, lty=selLtY, rbx=selRbX, rby=selRbY;
@@ -1289,7 +1853,7 @@ void MergeWithTable()
    }
    while (val_ptr != NULL) {
       if (c_ptr != NULL) *c_ptr = '\0';
-      if (strlen(val_ptr) == 0) {
+      if (*val_ptr == '\0') {
          free(spec);
          fclose(fp);
          sprintf(gszMsgBox, TgLoadString(STID_MALFORMED_COL_NAMES_ABORT_MRG),
@@ -1770,6 +2334,43 @@ void ExportToTable()
    }
 }
 
+void ToggleShowWireSignalName()
+{
+   showWireSignalName = !showWireSignalName;
+   sprintf(gszMsgBox, TgLoadString(showWireSignalName ?
+         STID_WILL_SHOW_WIRE_SIGNAL_NAME : STID_WILL_HIDE_WIRE_SIGNAL_NAME));
+   Msg(gszMsgBox);
+}
+
+int RefreshPortMenu(menu)
+   TgMenu *menu;
+{
+   int ok=TRUE;
+
+   /* Show Wire Signal Name */
+   ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESHOWWIRESIGNALNAME,
+         showWireSignalName);
+
+   return ok;
+}
+
+TgMenu *CreatePortMenu(parent_menu, x, y, menu_info, status_str_xlated)
+   TgMenu *parent_menu;
+   int x, y;
+   TgMenuInfo *menu_info;
+   int status_str_xlated; /* ignored, always 0 */
+{
+   TgMenu *menu=TgCreateMenuFromMenuInfo(parent_menu, x, y, menu_info, FALSE);
+
+   if (menu != NULL) {
+      if (!RefreshPortMenu(menu)) {
+         return TgDestroyMenu(menu, TRUE);
+      }
+      menu->refresh_proc = ((RefreshMenuFunc*)RefreshPortMenu);
+   }
+   return menu;
+}
+
 void RefreshSpecialMenu(menu)
    TgMenu *menu;
 {
@@ -1791,3 +2392,19 @@ int SpecialMenu(X, Y, TrackMenubar)
    }
    return rc;
 }
+
+void CleanUpSpecial()
+{
+}
+
+int InitSpecial()
+{
+   char *c_ptr=NULL;
+
+   showWireSignalName = TRUE;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"ShowWireSignalName")) !=
+         NULL && UtilStrICmp(c_ptr, "false") == 0) {
+      showWireSignalName = FALSE;
+   }
+   return TRUE;
+}
