@@ -29,7 +29,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/tangram2.c,v 4.18 2000/08/11 19:59:54 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tangram2.c,v 4.20 2000/10/11 13:53:09 william Exp $
  */
 
 #define _INCLUDE_FROM_TANGRAM2_C_
@@ -227,7 +227,7 @@ ExecInfo *Tangram2GetExecInfo(func_name)
 
 /* --------------- Tangram2 Internal Commands Implementation --------------- */
 
-#define MAXVARS              20
+#define MAXVARS             200
 #define MAXVARNAME          250
 #define MAXCONNECTTRIES       3
 #define PORT               6743
@@ -294,7 +294,7 @@ void ExecStartSimulator(obj_ptr, orig_cmd)
     else
         if( (sin.sin_port = htons( (u_short)atoi( service ) )) == 0 )
         {
-            fprintf( stderr, "TGIF: Can't get \"%s\" service entry\n", service );
+            fprintf( stderr,"TGIF: Can't get \"%s\" service entry\n", service );
             return;
         }
     
@@ -328,7 +328,8 @@ void ExecStartSimulator(obj_ptr, orig_cmd)
         return;
     }
     
-    if( setsockopt( s, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof( one ) ) < 0 )
+    if( setsockopt( s, SOL_SOCKET, SO_REUSEADDR, (char *)&one,
+        sizeof( one ) ) < 0 )
     {
         perror( "TGIF: REUSEADDR" );
         return;
@@ -349,7 +350,7 @@ void ExecStartSimulator(obj_ptr, orig_cmd)
     FD_ZERO( &soc_pool );
     FD_SET( s, &soc_pool );
     max_desc = s + 1;
-    timeout.tv_sec  = 20;
+    timeout.tv_sec  = 10;
     timeout.tv_usec = 0;
     if( select( max_desc, &soc_pool, NULL, NULL, &timeout ) < 0 )
     {
@@ -359,7 +360,8 @@ void ExecStartSimulator(obj_ptr, orig_cmd)
     if( FD_ISSET( s, &soc_pool ) )
     {
         sin_size = sizeof( sin );
-        if( recvfrom( s, buffer, sizeof( T_Packet ), 0, (struct sockaddr *)&sin, &sin_size ) < 0 )
+        if( recvfrom( s, buffer, sizeof( T_Packet ), 0, (struct sockaddr *)&sin,
+                      &sin_size ) < 0 )
         {
             perror( "TGIF: recv" );
             return;
@@ -387,7 +389,7 @@ void ExecStartSimulator(obj_ptr, orig_cmd)
     
     simulator_socket = s;
     
-    printf("TGIF: Cliente Conectado.\n");
+    printf("TGIF: Client Connected.\n");
 }
 
 void ExecEndSimulator(obj_ptr, orig_cmd)
@@ -416,24 +418,75 @@ void ExecEndSimulator(obj_ptr, orig_cmd)
    printf("TGIF: Simulation finished.\n");
 }
 
-static
-void monta_buffer(char *buffer, T_Packet pack)
+static int isVector( char *varname )
 {
-   int i;
-   char temp[MAXSTRING+1];
+    char *pt = varname;
+    while( (*pt != '[') && (*pt != '\0'))
+        pt++;
 
-   memset(buffer, 0, sizeof(buffer));
-   for (i=0; i < pack.varcount; i++) {
-      strcat(buffer, pack.vars[i].name);
-      sprintf(temp, "=%d@", pack.vars[i].value);
-      strcat(buffer, temp);
-   }
-
-   /* printf("monta: %s\n", buffer); */
+    return( *pt == '[' ? pt - varname : 0 );
 }
 
-static
-int recv_packet(int soc, T_Packet *pack_in)
+/******************************************************************************/
+/* This function extracts the state var names and values from the packet that */
+/* came from markfind and fullfill the buffer attribute of simulation step    */
+/* button.                                                                    */
+/******************************************************************************/
+static void monta_buffer(char *buffer, T_Packet pack)
+{
+    int i, pos;
+    int initial_index, final_index;
+    int temp_index;
+    char temp[MAXSTRING+1];
+    char value[MAXSTRING+1];
+    char basename[MAXSTRING+1];
+
+    buffer[0] = 0;
+    for (i=0; i < pack.varcount; i++)
+    {
+        if( ( pos = isVector( pack.vars[i].name ) ) > 0 )
+        {
+            final_index = -1;
+            strncpy( basename, pack.vars[i].name, pos );
+            basename[ pos ] = 0;
+            initial_index = atoi( &pack.vars[i].name[ pos + 1 ] );
+
+            sprintf( value, "%d", pack.vars[i].value );
+
+            i++;
+            while( (strncmp( basename, pack.vars[i].name, pos ) == 0) && 
+                   (pack.vars[i].name[pos] == '[') && (i < pack.varcount) )
+            {
+                temp_index = atoi( &pack.vars[i].name[ pos + 1 ] );
+                if( (final_index == -1 && temp_index == initial_index + 1)
+                    || temp_index == final_index + 1 )
+                {
+                    final_index = temp_index;
+                    sprintf( temp, ",%d", pack.vars[i].value );
+                    strcat( value, temp );
+                    i++;
+                }
+                else
+                    break;
+            }
+            i--;
+            if( final_index != -1 )
+                sprintf( temp, "%s[%d:%d]=[%s]@", basename, initial_index,
+                         final_index, value);
+            else
+                sprintf( temp, "%s[%d]=%s@", basename, initial_index, value);
+            strcat( buffer, temp );
+        }
+        else
+        {
+            strcat( buffer, pack.vars[i].name );
+            sprintf( temp, "=%d@", pack.vars[i].value );
+            strcat( buffer, temp );
+        }
+    }
+}
+
+static int recv_packet(int soc, T_Packet *pack_in)
 {
    int  byread, status;   /* bytes read from the socket */
    char buffer[sizeof(T_Packet)];
@@ -462,8 +515,7 @@ int recv_packet(int soc, T_Packet *pack_in)
    }
 }
 
-static
-int send_packet(int soc, T_Packet pack_out)
+static int send_packet(int soc, T_Packet pack_out)
 {
    int  bywrite, status;
    char buffer[sizeof(T_Packet)];
@@ -489,27 +541,34 @@ int send_packet(int soc, T_Packet pack_out)
    }
 }
 
-
-static
-void desmonta_buffer(char *buffer, T_Packet *pack)
+/******************************************************************************/
+/* This function extracts the state var names and values from buffer which    */
+/* came from simulation step button and builds a packet that goes to markfind */
+/******************************************************************************/
+static void desmonta_buffer(char *buffer, T_Packet *pack)
 {
    int i;
    char *token;
-
-   /* printf("desmonta: %s\n", buffer); */
+   char *pt;
 
    i = 0;
-   token = strtok(buffer, "=");
-   strcpy(pack->vars[i].name, token);
+   token = strtok(buffer, "@");
    while (token != NULL) {
-      token = strtok(NULL, "@");
-      pack->vars[i].value = atoi(token);
-      i++;
-      token = strtok(NULL, "=");
-      if (token == NULL) {
-         break;
-      }
-      strcpy(pack->vars[i].name, token);
+        pt = token;
+        while( (*pt != '=') && (*pt != '\0') )
+            pt++;
+        *pt = '\0';
+
+        strcpy( pack->vars[i].name, token );
+        /* FIXME - if necessary feedback markfind with the state vars values */
+        /* this is the right place to get them */
+        pack->vars[i].value = 0;
+
+        i++;
+        if( i >= MAXVARS )
+            break;
+
+        token = strtok(NULL, "@");
    }
 
    pack->varcount = i;
@@ -539,6 +598,7 @@ void ExecSimulateStep(obj_ptr, orig_cmd)
       perror("invalid simulator_socket");
       return;
    }
+   
    /* pega o valor do atributo buffer */
    strcpy(execDummyStr, "buffer=");
    attr_ptr = FindAttrWithName(obj_ptr, execDummyStr, &attr_owner_obj);
@@ -557,10 +617,13 @@ void ExecSimulateStep(obj_ptr, orig_cmd)
    if (step <= 0) {
       step = 1;
    }
+
    /* monta o pacote */
    memset((char *)&pack_out, 0, sizeof(T_Packet));
    pack_out.code = STEP_SIMULATION;
    pack_out.step = step;
+
+
    desmonta_buffer(buffer, &pack_out);
 
    /* envia o pacote ao simulador */
@@ -782,19 +845,18 @@ static char *GenerateChainWriteObj[] = {
   "append_attr_into_file(str,$(file_name));",
   NULL
 };
-
-
+/* ----------------------------------------------------------------------- */
 static char *StartSimulatorSetModeB[] = {
-  "strcpy(command, \"simulate_chain $(__GENERATE_CHAIN__.basename) B $(__GENERATE_CHAIN__.dbglevel) $(Event) $(Transitions) $(Time) $(Runs) >> $(__GENERATE_CHAIN__.output) & \");",
-  "launch(command);",
+/*  "strcpy(command, \"simulate_chain $(__GENERATE_CHAIN__.basename) B $(__GENERATE_CHAIN__.dbglevel) $(Event) $(Transitions) $(Time) $(Runs) >> $(__GENERATE_CHAIN__.output) & \");", */
+/*  "launch(command);", */
   NULL
 };
 
 static char *StartSimulatorSetModeI[] = {
  "strcpy(Transitions, \"0\");",
  "strcpy(Time, \"0.0\");",
- "strcpy(command, \"simulate_chain $(__GENERATE_CHAIN__.basename) I $(__GENERATE_CHAIN__.dbglevel) >> $(__GENERATE_CHAIN__.output) & \");",
- "launch(command);",
+/* "strcpy(command, \"simulate_chain $(__GENERATE_CHAIN__.basename) I $(__GENERATE_CHAIN__.dbglevel) >> $(__GENERATE_CHAIN__.output) & \");", */
+/* "launch(command);", */
  "start_simulator();",
   NULL
 };
@@ -821,16 +883,7 @@ static char *StartSimulatorExec[] = {
  "enable_undo();",
  NULL
 };
-
-static char *SimulationStepWriteVarLine[] = {
- "tokenize(state_var_tok,$(state_var_line),'=');",
- "get_line_in_attr(state_var_name,state_var_tok,1);",
- "get_line_in_attr(state_var_value,state_var_tok,2);",
- "strcpy(new_line, \"$(cur_obj).$(state_var_name)=$(state_var_value)\");",
- "strcpy(buffer,\"$(buffer)$(new_line)@\");",
-  NULL
-};
-
+/* ----------------------------------------------------------------------- */
 static char *SimulationStepLoopAnimation[] = {
  "for_i(Ani_C_Step,1,$(Ani_T_Steps),1,Do_Animation);",
  "round(sleep_time,$(Time_Scale)*$(step_time)*1000);",
@@ -838,12 +891,6 @@ static char *SimulationStepLoopAnimation[] = {
   NULL
 };
 
-static char *SimulationStepAnimateObj[] = {
- "get_line_in_attr(cur_obj,obj_list,$(obj_index));",
- "exec(\"$(cur_obj).Animation\");",
-  NULL
-};
-
 static char *SimulationStepDoAnimation[] = {
  "find_obj_names(obj_list,\"\",\"Animation=*\");",
  "get_line_in_attr(no_objs,obj_list,0);",
@@ -851,6 +898,13 @@ static char *SimulationStepDoAnimation[] = {
   NULL
 };
 
+static char *SimulationStepAnimateObj[] = {
+ "get_line_in_attr(cur_obj,obj_list,$(obj_index));",
+ "exec(\"$(cur_obj).Animation\");",
+  NULL
+};
+
+
 static char *SimulationStepCallSimulator[] = {
   "simulate_step();",
   NULL
@@ -864,15 +918,26 @@ static char *SimulationStepPlayOneStep[] = {
   NULL
 };
 
-static char *SimulationStepUpdateObjects[] = {
- "tokenize(state_var_list,$(buffer),'@');",
- "get_line_in_attr(no_state_vars,state_var_list,0);",
- "for_i(var_index,1,\"$(no_state_vars)-1\",1,update_vars);",
+static char *SimulationStepStop[] = {
+  "stop();",
   NULL
 };
 
-static char *SimulationStepStop[] = {
-  "stop();",
+static char *SimulationStepExec[] = {
+ "disable_undo();",
+ /*"show_attr(Step);",
+ "show_attr(Delay);",
+ "show_attr(Animation);",
+ "show_attr(Time_Scale);",*/
+ "if(\"$(Step)\"==\"0\",play_simulation,step_simulation);",
+ "enable_undo();",
+ "redraw_drawing_area();",
+  NULL
+};
+
+
+static char *SimulationStepPlaySimulation[] = {
+  "while(1,play_one_step);",
   NULL
 };
 
@@ -880,7 +945,9 @@ static char *SimulationStepStepSimulation[] = {
  "strcpy(buffer,\"\");",
  "find_obj_names(obj_list,\"\",\"State_vars=*\");",
  "get_line_in_attr(no_objs,obj_list,0);",
+
  "for_i(obj_index,1,$(no_objs),1,parse_obj);",
+/* aqui furou */
  "exec(call_simulator);",
  "exec(update_objects);",
  "redraw_drawing_area();",
@@ -888,8 +955,41 @@ static char *SimulationStepStepSimulation[] = {
   NULL
 };
 
-static char *SimulationStepPlaySimulation[] = {
-  "while(1,play_one_step);",
+static char *SimulationStepParseObj[] = {
+ "get_line_in_attr(cur_obj,obj_list,$(obj_index));",
+ "get_number_of_lines_in_attr(no_state_vars,$(cur_obj).State_vars);",
+ "for_i(var_index,1,$(no_state_vars)-1,1,write_vars);",
+/* "for_i(var_index,1,2,1,write_vars);", */
+ "clear_attr($(cur_obj).State_vars);",
+  NULL
+};
+
+static char *SimulationStepWriteVars[] = {
+  /* FIXME this part must be used to feedback (state vars) to markfind */
+ "get_line_in_attr(state_var_line,$(cur_obj).State_vars,$(var_index));",
+ "trim(state_var_line);",
+ "strlen(line_size, $(state_var_line));",
+ "if($(line_size)>=1,write_var_line,NULL);",
+/* "if(1==1,write_var_line,NULL);", */
+  NULL
+};
+
+static char *SimulationStepWriteVarLine[] = {
+ /* wipes out the spaces inside the lines eg: S = 3 -> S=3 */
+ "tokenize(state_var_tok,$(state_var_line),'=');",
+ "get_line_in_attr(state_var_name,state_var_tok,1);",
+ "get_line_in_attr(state_var_value,state_var_tok,2);",
+ "strcpy(new_line, \"$(cur_obj).$(state_var_name)=$(state_var_value)\");",
+ /* append state var assignment (without spaces) in buffer attribute */
+ "strcpy(buffer,\"$(buffer)$(new_line)@\");",
+/* "strcpy(buffer,\"<obj.state_var>=<value>@\");", */
+  NULL
+};
+
+static char *SimulationStepUpdateObjects[] = {
+ "tokenize(state_var_list,$(buffer),'@');",
+ "get_line_in_attr(no_state_vars,state_var_list,0);",
+ "for_i(var_index,1,\"$(no_state_vars)-1\",1,update_vars);",
   NULL
 };
 
@@ -901,38 +1001,18 @@ static char *SimulationStepUpdateVars[] = {
  "tokenize(state_var_tok,$(state_var_str),'=');",
  "get_line_in_attr(state_var_name,state_var_tok,1);",
  "get_line_in_attr(state_var_value,state_var_tok,2);",
+ /* sets the color, font and size of the State_vars attribute */
+ "select_obj_by_name($(obj_name));",
+/* "set_selected_obj_pen(0000ff);", */
+/* "set_selected_text_size(14);", */
+/* "set_selected_text_font(\"Courier\");", */
+/*"set_color(\"black\");", */
  "append_line_into_attr($(obj_name).State_vars,\" $(state_var_name)=$(state_var_value)\");",
+ "show_attr($(obj_name).State_vars);",
   NULL
 };
 
-static char *SimulationStepParseObj[] = {
- "get_line_in_attr(cur_obj,obj_list,$(obj_index));",
- "get_number_of_lines_in_attr(no_state_vars,$(cur_obj).State_vars);",
- "for_i(var_index,1,$(no_state_vars)-1,1,write_vars);",
- "clear_attr($(cur_obj).State_vars);",
-  NULL
-};
-
-static char *SimulationStepExec[] = {
- "disable_undo();",
- "show_attr(Step);",
- "show_attr(Delay);",
- "show_attr(Animation);",
- "show_attr(Time_Scale);",
- "if(\"$(Step)\"==\"0\",play_simulation,step_simulation);",
- "enable_undo();",
- "redraw_drawing_area();",
-  NULL
-};
-
-static char *SimulationStepWriteVars[] = {
- "get_line_in_attr(state_var_line,$(cur_obj).State_vars,$(var_index));",
- "trim(state_var_line);",
- "strlen(line_size, $(state_var_line));",
- "if(\"$(line_size)\">=\"3\",write_var_line,NULL);",
-  NULL
-};
-
+/* ----------------------------------------------------------------------- */
 
 static
 void AppendLinesToAttr(attr_ptr, ppsz_lines)
