@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/types.h,v 4.43 2000/04/12 19:25:39 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/types.h,v 4.44 2000/08/19 21:17:47 william Exp $
  */
 
 #ifndef _TGIF_TYPES_H_
@@ -245,6 +245,10 @@ typedef struct StrSegRec {
    char	* font_name; /* only used in PRTGIF */
    struct DynStrRec	dyn_str;
    struct StrBlockRec	* owner;
+   /*
+    * for file version 38!
+    */
+   int	dontreencode;
 } StrSegInfo;
 
 #define SB_SIMPLE        0
