@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/wb.c,v 4.55 2000/02/26 22:55:30 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/wb.c,v 4.57 2000/06/07 22:33:46 william Exp $
  */
 
 #define _INCLUDE_FROM_WB_C_
@@ -149,9 +149,8 @@ int CreateWBListenSocket()
 {
    gstWBInfo.listening = TRUE;
 
-   sprintf(gstWBInfo.listen_fname, "%sTgifLXXXXXX", TMP_DIR);
-   mktemp(gstWBInfo.listen_fname);
-   unlink(gstWBInfo.listen_fname);
+   MkTempFile(gstWBInfo.listen_fname, sizeof(gstWBInfo.listen_fname), tmpDir,
+         TOOL_NAME);
    if ((gstWBInfo.listen_fp=fopen(gstWBInfo.listen_fname, "w+")) == NULL) {
       fprintf(stderr, "Fail to create '%s' for deserialization.\n",
             gstWBInfo.listen_fname);
@@ -1334,16 +1333,15 @@ int InitWhiteBoard()
       }
       /* fim do trecho */
 
-      sprintf(gstWBInfo.send_fname, "%sTgifSXXXXXX", TMP_DIR);
-      mktemp(gstWBInfo.send_fname);
-      unlink(gstWBInfo.send_fname);
+      MkTempFile(gstWBInfo.send_fname, sizeof(gstWBInfo.send_fname), tmpDir,
+            TOOL_NAME);
       if ((gstWBInfo.send_fp=fopen(gstWBInfo.send_fname, "w+")) == NULL) {
          fprintf(stderr, "Fail to create '%s' for serialization.\n",
                gstWBInfo.send_fname);
          exit(-1);
       }
       /* Creation of multicast socket */
-      sprintf(logfile, "%snome.txt", TMP_DIR);
+      sprintf(logfile, "%snome.txt", tmpDir);
       PCM_initialize(logfile);
       PCM_setoption(SM_INTERVAL,3);
 
