@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/xprtfltr.c,v 4.14 2000/05/10 19:22:39 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/xprtfltr.c,v 4.17 2000/08/03 22:32:47 william Exp $
  */
 
 #define _INCLUDE_FROM_XPRTFLTR_C_
@@ -45,6 +45,7 @@
 #include "names.e"
 #include "obj.e"
 #include "page.e"
+#include "ps.e"
 #include "raster.e"
 #include "setup.e"
 #include "strtbl.e"
@@ -346,22 +347,24 @@ int InitExportFilters()
 /* do not translate -- program constants */
 static char *gpszColorExportName[MAXDEFWHERETOPRINT] = {
    "Printer", "EPS", "PS", "XPM", "ASCII", "EPSI", "GIF/ISMAP", "HTML/USEMAP",
-   "PDF", "TiffEPSI", "PNG", "JPEG"
+   "PDF", "TiffEPSI", "PNG", "JPEG", "NetList"
 };
 
 static char *gpszBWExportName[MAXDEFWHERETOPRINT] = {
    "Printer", "EPS", "PS", "XBM", "ASCII", "EPSI", "GIF/ISMAP", "HTML/USEMAP",
-   "PDF", "TiffEPSI", "PNG", "JPEG"
+   "PDF", "TiffEPSI", "PNG", "JPEG", "NetList"
 };
 
 static char *gpszColorExportExt[MAXDEFWHERETOPRINT] = {
    NULL, EPSF_FILE_EXT, PS_FILE_EXT, XPM_FILE_EXT, TEXT_FILE_EXT,
-   EPSF_FILE_EXT, "gif", "html", "pdf", EPSF_FILE_EXT, "png", "jpg"
+   EPSF_FILE_EXT, "gif", "html", "pdf", EPSF_FILE_EXT, "png", "jpg",
+   NETLIST_FILE_EXT
 };
 
 static char *gpszBWExportExt[MAXDEFWHERETOPRINT] = {
    NULL, EPSF_FILE_EXT, PS_FILE_EXT, XBM_FILE_EXT, TEXT_FILE_EXT,
-   EPSF_FILE_EXT, "gif", "html", "pdf", EPSF_FILE_EXT, "png", "jpg"
+   EPSF_FILE_EXT, "gif", "html", "pdf", EPSF_FILE_EXT, "png", "jpg",
+   NETLIST_FILE_EXT
 };
 
 int GetExportIndex(nWhereToPrint, nMode)
@@ -384,7 +387,10 @@ char *GetExportName(nWhereToPrint, nMode)
    nWhereToPrint = GetExportIndex(nWhereToPrint, nMode);
 
    if (nWhereToPrint < MAXDEFWHERETOPRINT) {
-      if (colorDump) {
+      if (nWhereToPrint == NETLIST_FILE && preDumpSetup) {
+         /* do not translate -- program constants */
+         return "Component";
+      } else if (colorDump) {
          return gpszColorExportName[nWhereToPrint];
       } else {
          return gpszBWExportName[nWhereToPrint];
@@ -411,7 +417,9 @@ char *GetExportExt(nWhereToPrint)
    TgEFInfo *pTgEFInfo=NULL;
 
    if (nWhereToPrint < MAXDEFWHERETOPRINT) {
-      if (colorDump) {
+      if (nWhereToPrint == NETLIST_FILE && preDumpSetup) {
+         return COMP_FILE_EXT;
+      } else if (colorDump) {
          return gpszColorExportExt[nWhereToPrint];
       } else {
          return gpszBWExportExt[nWhereToPrint];
